@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5380 $:
+$Revision: 5895 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-08-10 00:15:35 +0200 (Wed, 10 Aug 2011) $
+$Date: 2012-03-06 01:43:16 +0100 (Tue, 06 Mar 2012) $
 
 ********************************************************************/
 
@@ -468,12 +468,16 @@ ChangeWireCommand::ChangeWireCommand(SketchWidget* sketchWidget, long fromID,
 
 void ChangeWireCommand::undo()
 {
-    m_sketchWidget->changeWire(m_fromID, m_oldLine, m_oldPos, m_updateConnections, m_updateRatsnest);
+	if (!m_redoOnly) {
+		m_sketchWidget->changeWire(m_fromID, m_oldLine, m_oldPos, m_updateConnections, m_updateRatsnest);
+	}
 }
 
 void ChangeWireCommand::redo()
 {
-    m_sketchWidget->changeWire(m_fromID, m_newLine, m_newPos, m_updateConnections, m_updateRatsnest);
+	if (!m_undoOnly) {
+		m_sketchWidget->changeWire(m_fromID, m_newLine, m_newPos, m_updateConnections, m_updateRatsnest);
+	}
 }
 
 QString ChangeWireCommand::getParamString() const {
@@ -1022,7 +1026,7 @@ QString ChangeZCommand::getParamString() const {
 		+ BaseCommand::getParamString();
 }
 
-/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
+//////////////////////////////////////////singlev///////////////////////////////////////////////////////////////////////
 
 CheckStickyCommand::CheckStickyCommand(SketchWidget* sketchWidget, BaseCommand::CrossViewType crossViewType, long itemID, bool checkCurrent, CheckType checkType, QUndoCommand *parent)
 : BaseCommand(crossViewType, sketchWidget, parent)
@@ -1169,19 +1173,19 @@ void CleanUpWiresCommand::addTrace(SketchWidget * sketchWidget, Wire * wire)
 	addSubCommand(new WireWidthChangeCommand(sketchWidget, wire->id(), wire->width(), wire->width(), NULL));
 	
 	foreach (ConnectorItem * toConnectorItem, wire->connector0()->connectedToItems()) {	
-		addSubCommand(new ChangeConnectionCommand(sketchWidget, BaseCommand::SingleView, toConnectorItem->attachedToID(), toConnectorItem->connectorSharedID(),
+		addSubCommand(new ChangeConnectionCommand(sketchWidget, BaseCommand::CrossView, toConnectorItem->attachedToID(), toConnectorItem->connectorSharedID(),
 								wire->id(), "connector0", 
 								ViewLayer::specFromID(wire->viewLayerID()),
 								false, NULL));
 	}
 	foreach (ConnectorItem * toConnectorItem, wire->connector1()->connectedToItems()) {	
-		addSubCommand(new ChangeConnectionCommand(sketchWidget, BaseCommand::SingleView, toConnectorItem->attachedToID(), toConnectorItem->connectorSharedID(),
+		addSubCommand(new ChangeConnectionCommand(sketchWidget, BaseCommand::CrossView, toConnectorItem->attachedToID(), toConnectorItem->connectorSharedID(),
 							wire->id(), "connector1", 
 							ViewLayer::specFromID(wire->viewLayerID()),
 							false, NULL));
 	}
 
-	addSubCommand(new DeleteItemCommand(sketchWidget, BaseCommand::SingleView, ModuleIDNames::WireModuleIDName, wire->viewLayerSpec(), wire->getViewGeometry(), wire->id(), wire->modelPart()->modelIndex(), NULL));
+	addSubCommand(new DeleteItemCommand(sketchWidget, BaseCommand::CrossView, ModuleIDNames::WireModuleIDName, wire->viewLayerSpec(), wire->getViewGeometry(), wire->id(), wire->modelPart()->modelIndex(), NULL));
 }
 
 bool CleanUpWiresCommand::hasTraces(SketchWidget * sketchWidget) {
@@ -1362,6 +1366,37 @@ QString MoveLabelCommand::getParamString() const {
 
 ///////////////////////////////////////////////
 
+MoveLockCommand::MoveLockCommand(SketchWidget *sketchWidget, long id, bool oldLock, bool newLock, QUndoCommand *parent)
+    : BaseCommand(BaseCommand::SingleView, sketchWidget, parent)
+{
+    m_itemID = id;
+    m_oldLock = oldLock;
+    m_newLock = newLock;
+}
+
+void MoveLockCommand::undo()
+{
+    m_sketchWidget->setMoveLock(m_itemID, m_oldLock);
+}
+
+void MoveLockCommand::redo()
+{
+    m_sketchWidget->setMoveLock(m_itemID, m_newLock);
+}
+
+
+QString MoveLockCommand::getParamString() const {
+	return QString("MoveLockCommand ") 
+		+ BaseCommand::getParamString()
+		+ QString(" id:%1 o:%2 n:%3") 
+			.arg(m_itemID)
+			.arg(m_oldLock)
+			.arg(m_newLock);
+
+}
+
+///////////////////////////////////////////////
+
 ChangeLabelTextCommand::ChangeLabelTextCommand(SketchWidget *sketchWidget, long id, 
 											   const QString & oldText, const QString & newText, 
 											   QUndoCommand *parent)
@@ -1827,3 +1862,105 @@ QString SetDropOffsetCommand::getParamString() const {
 			.arg(m_itemID).arg(m_dropOffset.x()).arg(m_dropOffset.y());
 
 }
+
+///////////////////////////////////////////////
+
+RenamePinsCommand::RenamePinsCommand(SketchWidget *sketchWidget, long id, const QStringList & oldOnes, const QStringList & newOnes, bool singleRow, QUndoCommand *parent) :
+	BaseCommand(BaseCommand::CrossView, sketchWidget, parent)
+{
+	m_oldLabels = oldOnes;
+	m_newLabels = newOnes;
+	m_itemID = id;
+	m_singleRow = singleRow;
+}
+
+void RenamePinsCommand::undo() {
+	m_sketchWidget->renamePins(m_itemID, m_oldLabels, m_singleRow);
+}
+
+void RenamePinsCommand::redo() {
+	m_sketchWidget->renamePins(m_itemID, m_newLabels, m_singleRow);
+}
+
+QString RenamePinsCommand::getParamString() const {
+	return QString("RenamePinsCommand ") 
+		+ BaseCommand::getParamString()
+		+ QString(" id:%1 sr:%2") 
+			.arg(m_itemID).arg(m_singleRow);
+
+}
+
+/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
+
+GroundFillSeedCommand::GroundFillSeedCommand(SketchWidget* sketchWidget, QUndoCommand *parent)
+    : BaseCommand(BaseCommand::SingleView, sketchWidget, parent)
+{
+	setText(QObject::tr("Set Ground Fill Seed"));
+}
+
+void GroundFillSeedCommand::undo()
+{
+	foreach(GFSThing gfsThing, m_items) {
+		m_sketchWidget->setGroundFillSeed(gfsThing.id, gfsThing.connectorID, !gfsThing.seed);
+	}
+}
+
+void GroundFillSeedCommand::redo()
+{
+	foreach(GFSThing gfsThing, m_items) {
+		m_sketchWidget->setGroundFillSeed(gfsThing.id, gfsThing.connectorID, gfsThing.seed);
+	}
+}
+
+void GroundFillSeedCommand::addItem(long id, const QString & connectorID, bool seed)
+{
+	GFSThing gfsThing;
+	gfsThing.id = id;
+	gfsThing.connectorID = connectorID;
+	gfsThing.seed = seed;
+	m_items.append(gfsThing);
+}
+
+QString GroundFillSeedCommand::getParamString() const {
+	return QString("GroundFillSeedCommand ") 
+		+ BaseCommand::getParamString() + 
+		QString(" items:%1")
+		.arg(m_items.count())
+		;
+}
+
+/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
+
+WireExtrasCommand::WireExtrasCommand(SketchWidget* sketchWidget, long fromID,
+									 const QDomElement & oldExtras, const QDomElement & newExtras,
+									 QUndoCommand *parent)
+    : BaseCommand(BaseCommand::SingleView, sketchWidget, parent)
+{
+
+    m_fromID = fromID;
+	m_newExtras = newExtras;
+    m_oldExtras = oldExtras;
+}
+
+void WireExtrasCommand::undo()
+{
+	if (!m_redoOnly) {
+		m_sketchWidget->setWireExtras(m_fromID, m_oldExtras);
+	}
+}
+
+void WireExtrasCommand::redo()
+{
+	if (!m_undoOnly) {
+		m_sketchWidget->setWireExtras(m_fromID, m_newExtras);
+	}
+}
+
+QString WireExtrasCommand::getParamString() const {
+	return QString("WireExtrasCommand ") 
+		+ BaseCommand::getParamString() + 
+		QString(" fromid:%1 ")
+		.arg(m_fromID)
+		;
+}
+
