@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5417 $:
+$Revision: 5939 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-08-18 12:47:41 +0200 (Thu, 18 Aug 2011) $
+$Date: 2012-04-03 09:44:37 +0200 (Tue, 03 Apr 2012) $
 
 ********************************************************************/
 
@@ -211,8 +211,6 @@ parts editor support
 #include <QPen>
 #include <QColor>
 #include <limits>
-#include <QMutex>
-#include <QMutexLocker>
 #include <QSet>
 #include <QToolTip>
 #include <QBitmap>
@@ -231,6 +229,7 @@ parts editor support
 #include "../utils/ratsnestcolors.h"
 #include "../utils/bezier.h"
 #include "../utils/bezierdisplay.h"
+#include "../utils/cursormaster.h"
 #include "ercdata.h"
 
 /////////////////////////////////////////////////////////
@@ -282,13 +281,6 @@ bool wireLessThan(ConnectorItem * c1, ConnectorItem * c2)
 	return c1->zValue() > c2->zValue();
 }
 
-QCursor * ConnectorItem::BendpointCursor = NULL;
-QCursor * ConnectorItem::NewBendpointCursor = NULL;
-QCursor * ConnectorItem::MakeWireCursor = NULL;
-QCursor * ConnectorItem::MakeCurveCursor = NULL;
-QCursor * ConnectorItem::RubberbandCursor = NULL;
-QCursor * ConnectorItem::MoveCursor = NULL;
-
 QColor addColor(QColor & color, int offset)
 {
     QColor rgb = color.toRgb();
@@ -298,6 +290,27 @@ QColor addColor(QColor & color, int offset)
     return rgb.convertTo(color.spec());
 }
 
+/////////////////////////////////////////////////////////////
+
+ConnectorItemAction::ConnectorItemAction(QAction * action) : QAction(action) {
+	m_connectorItem = NULL;
+	this->setText(action->text());
+	this->setStatusTip(action->statusTip());
+	this->setCheckable(action->isCheckable());
+}
+
+ConnectorItemAction::ConnectorItemAction(const QString & title, QObject * parent) : QAction(title, parent) {
+	m_connectorItem = NULL;
+}
+
+void ConnectorItemAction::setConnectorItem(ConnectorItem * c) {
+	m_connectorItem = c;
+}
+
+ConnectorItem * ConnectorItemAction::connectorItem() {
+	return m_connectorItem;
+}
+
 /////////////////////////////////////////////////////////
 
 ConnectorItem::ConnectorItem( Connector * connector, ItemBase * attachedTo )
@@ -305,6 +318,7 @@ ConnectorItem::ConnectorItem( Connector * connector, ItemBase * attachedTo )
 {
 	// initialize m_connectorT, otherwise will trigger qWarning("QLine::unitVector: New line does not have unit length");
 	// TODO: figure out why paint is being called with m_connectorT not initialized
+	m_groundFillSeed = false;
 	m_connectorDetectT = m_connectorDrawT = 0;
 	m_draggingCurve = m_draggingLeg = m_rubberBandLeg = m_bigDot = m_hybrid = false;
 	m_marked = false;
@@ -317,7 +331,7 @@ ConnectorItem::ConnectorItem( Connector * connector, ItemBase * attachedTo )
 		connector->addViewItem(this);
 	}
     setAcceptHoverEvents(true);
-    this->setCursor((attachedTo && attachedTo->itemType() == ModelPart::Wire) ? Qt::CrossCursor : *MakeWireCursor);
+    this->setCursor((attachedTo && attachedTo->itemType() == ModelPart::Wire) ? *CursorMaster::BendpointCursor : *CursorMaster::MakeWireCursor);
 
 	//DebugDialog::debug(QString("%1 attached to %2")
 			//.arg(this->connector()->connectorShared()->id())
@@ -364,28 +378,26 @@ void ConnectorItem::hoverEnterEvent ( QGraphicsSceneHoverEvent * event ) {
 	}
 
 	//DebugDialog::debug("---CI set override cursor");
-	QApplication::setOverrideCursor(cursor());
+	CursorMaster::instance()->addCursor(this, cursor());
 	bool setDefaultCursor = true;
 	m_hoverEnterSpaceBarWasPressed = false;
 	setHoverColor();
 	if (infoGraphicsView != NULL) {
 		infoGraphicsView->hoverEnterConnectorItem(event, this);
 		if (m_rubberBandLeg) {
-			QApplication::instance()->installEventFilter(this);
 			updateLegCursor(event->pos(), event->modifiers());
 			setDefaultCursor = false;
 		}
 	}
 	if (this->m_attachedTo != NULL) {
 		if (this->attachedToItemType() == ModelPart::Wire) {
-			QApplication::instance()->installEventFilter(this);
 			updateWireCursor(event->modifiers());
 			setDefaultCursor = false;
 		}
 		m_attachedTo->hoverEnterConnectorItem(event, this);
 	}
 
-	if (setDefaultCursor) QApplication::changeOverrideCursor(*MakeWireCursor);
+	if (setDefaultCursor) CursorMaster::instance()->addCursor(this, *CursorMaster::MakeWireCursor);
 }
 
 void ConnectorItem::hoverLeaveEvent ( QGraphicsSceneHoverEvent * event ) {
@@ -398,25 +410,16 @@ void ConnectorItem::hoverLeaveEvent ( QGraphicsSceneHoverEvent * event ) {
 	InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
 	if (infoGraphicsView != NULL) {
 		infoGraphicsView->hoverLeaveConnectorItem(event, this);
-		if (m_rubberBandLeg) {
-			QApplication::instance()->removeEventFilter(this);
-		}
 	}
 
-	if (this->attachedToItemType() == ModelPart::Wire) {
-		QApplication::instance()->removeEventFilter(this);
-		this->setCursor(Qt::CrossCursor);
-	}
-	else if (m_rubberBandLeg) {
-		this->setCursor(*MakeWireCursor);
-	}
+	CursorMaster::instance()->removeCursor(this);
 
 	if (this->m_attachedTo != NULL) {
 		m_attachedTo->hoverLeaveConnectorItem(event, this);
 	}
 
 	//DebugDialog::debug("------CI restore override cursor");
-	QApplication::restoreOverrideCursor();
+	CursorMaster::instance()->removeCursor(this);
 }
 
 void ConnectorItem::hoverMoveEvent ( QGraphicsSceneHoverEvent * event ) {
@@ -468,8 +471,6 @@ void ConnectorItem::connectTo(ConnectorItem * connected) {
 	if (m_attachedTo != NULL) {
 		m_attachedTo->connectionChange(this, connected, true);
 	}
-
-	updateTooltip();
 }
 
 ConnectorItem * ConnectorItem::removeConnection(ItemBase * itemBase) {
@@ -485,7 +486,6 @@ ConnectorItem * ConnectorItem::removeConnection(ItemBase * itemBase) {
 				.arg((long) this, 0, 16)
 				.arg(itemBase->modelPartShared()->title())
 				.arg(m_connectedTo.count()) );
-			updateTooltip();
 			return removed;
 		}
 	}
@@ -501,19 +501,16 @@ void ConnectorItem::removeConnection(ConnectorItem * connectedItem, bool emitCha
 	if (emitChange) {
 		m_attachedTo->connectionChange(this, connectedItem, false);
 	}
-	updateTooltip();
 }
 
 void ConnectorItem::tempConnectTo(ConnectorItem * item, bool applyColor) {
 	if (!m_connectedTo.contains(item)) m_connectedTo.append(item);
-	updateTooltip();
 
 	if(applyColor) restoreColor(true, 0, true);
 }
 
 void ConnectorItem::tempRemove(ConnectorItem * item, bool applyColor) {
 	m_connectedTo.removeOne(item);
-	updateTooltip();
 
 	if(applyColor) restoreColor(true, 0, true);
 }
@@ -664,12 +661,18 @@ void ConnectorItem::mouseReleaseEvent(QGraphicsSceneMouseEvent *event) {
 	InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
 
 	if (m_rubberBandLeg && m_draggingLeg) {
+		m_draggingLeg = false;
+
 		if (m_insertBendpointPossible) {
 			// didn't move far enough; bail
 			return;
 		}
 
-		m_draggingLeg = false;
+		if (m_moveCount == 0) { 
+			// never moved
+			return;
+		}
+
 		ConnectorItem * to = releaseDrag();
 
 		if (m_draggingCurve) {
@@ -744,7 +747,7 @@ void ConnectorItem::mouseDoubleClickEvent(QGraphicsSceneMouseEvent *event) {
 }
 
 void ConnectorItem::mouseMoveEvent(QGraphicsSceneMouseEvent *event) {
-
+	m_moveCount++;
 	if (m_rubberBandLeg && m_draggingLeg) {
 		if (m_draggingCurve) {
 			Bezier * bezier = m_legCurves.at(m_draggingLegIndex);
@@ -814,6 +817,7 @@ void ConnectorItem::mouseMoveEvent(QGraphicsSceneMouseEvent *event) {
 
 void ConnectorItem::mousePressEvent(QGraphicsSceneMouseEvent *event) {
 	m_draggingCurve = m_draggingLeg = false;
+	m_moveCount = 0;
 
 	if (event->button() != Qt::LeftButton) {
 		QGraphicsRectItem::mousePressEvent(event);
@@ -861,6 +865,11 @@ void ConnectorItem::attachedMoved() {
 	foreach (ConnectorItem * toConnector, m_connectedTo) {
 		ItemBase * itemBase = toConnector->attachedTo();
 		if (itemBase == NULL) continue;
+		if (!itemBase->isVisible()) {
+			//this->debugInfo("continue");
+			//itemBase->debugInfo("    ");
+			continue;
+		}
 
 		itemBase->connectedMoved(this, toConnector);
 	}
@@ -870,7 +879,18 @@ ConnectorItem * ConnectorItem::firstConnectedToIsh() {
 	if (m_connectedTo.count() <= 0) return NULL;
 
 	foreach (ConnectorItem * connectorItem, m_connectedTo) {
-		if (!connectorItem->attachedTo()->getRatsnest()) return connectorItem;
+		if (connectorItem->attachedTo()->getRatsnest()) continue;
+		
+		if (connectorItem->isVisible()) return connectorItem;
+	}
+
+	// TODO: not sure whether to return invisible connectors
+	// TODO: get rid of this function?
+
+	foreach (ConnectorItem * connectorItem, m_connectedTo) {
+		if (connectorItem->attachedTo()->getRatsnest()) continue;
+		
+		return connectorItem;
 	}
 
 	return NULL;
@@ -958,6 +978,9 @@ bool ConnectorItem::isHybrid() {
 
 void ConnectorItem::setBigDot(bool bd) {
 	m_bigDot = bd;
+	//if (bd) {
+	//	this->debugInfo("big dot");
+	//}
 }
 
 bool ConnectorItem::isBigDot() {
@@ -977,7 +1000,7 @@ void ConnectorItem::setHiddenOrInactive() {
 	}
 	else {
 		this->setAcceptedMouseButtons(ALLMOUSEBUTTONS);
-		this->setCursor(attachedToItemType() == ModelPart::Wire ? Qt::CrossCursor : *MakeWireCursor);
+		this->setCursor(attachedToItemType() == ModelPart::Wire ? *CursorMaster::BendpointCursor : *CursorMaster::MakeWireCursor);
 		setAcceptHoverEvents(true);
 	}
 	this->update();
@@ -1080,20 +1103,18 @@ bool ConnectorItem::chained() {
 	return false;
 }
 
-void ConnectorItem::writeTopLevelAttributes(QXmlStreamWriter & writer) {
-	// do not write anything other than attributes in this routine.l
-	writer.writeAttribute("layer", ViewLayer::viewLayerXmlNameFromID(attachedToViewLayerID()));
-}
-
 void ConnectorItem::saveInstance(QXmlStreamWriter & writer) {
-	if (m_connectedTo.count() <= 0 && !m_rubberBandLeg) {
+	if (m_connectedTo.count() <= 0 && !m_rubberBandLeg && !m_groundFillSeed) {
 		// no need to save if there's no connection
 		return;
 	}
 
 	writer.writeStartElement("connector");
 	writer.writeAttribute("connectorId", connectorSharedID());
-	writeTopLevelAttributes(writer);
+	writer.writeAttribute("layer", ViewLayer::viewLayerXmlNameFromID(attachedToViewLayerID()));
+	if (m_groundFillSeed) {
+		writer.writeAttribute("groundFillSeed", "true");
+	}
 
 	writer.writeStartElement("geometry");
 	QPointF p = this->pos();
@@ -1152,14 +1173,6 @@ void ConnectorItem::writeOtherElements(QXmlStreamWriter & writer) {
 	Q_UNUSED(writer);
 }
 
-bool ConnectorItem::maleToFemale(ConnectorItem * other) {
-	
-	if (this->connectorType() == Connector::Male && other->connectorType() == Connector::Female) return true;
-	if (this->connectorType() == Connector::Female && other->connectorType() == Connector::Male) return true;
-
-	return false;
-}
-
 bool ConnectorItem::wiredTo(ConnectorItem * target, ViewGeometry::WireFlags skipFlags) {
 	QList<ConnectorItem *> connectorItems;
 	connectorItems.append(this);
@@ -1460,7 +1473,16 @@ void ConnectorItem::updateTooltip() {
 	}
 
 	if (connectors.count() == 1) {
-		setToolTip(connectors[0]->m_baseTooltip);
+		QString name = connectors[0]->connectorSharedName();
+		QString descr = connectors[0]->connectorSharedDescription();
+		if (name.compare(descr) == 0) {
+			name = connectors[0]->connectorSharedID();
+		}
+        QString tt = QString("<b>%1</b>: %2<br /><span style='font-size:small;'>%3</span>")
+                .arg(name)
+                .arg(descr)
+                .arg(connectors[0]->attachedToTitle());
+		setToolTip(tt);
 		return;
 	}
 
@@ -1474,11 +1496,6 @@ void ConnectorItem::updateTooltip() {
 
 }
 
-void ConnectorItem::setBaseTooltip(const QString & tooltip) {
-	m_baseTooltip = tooltip;
-	setToolTip(tooltip);
-}
-
 void ConnectorItem::clearConnector() {
 	m_connector = NULL;
 }
@@ -1542,7 +1559,10 @@ bool ConnectorItem::isGrounded(ConnectorItem * c1, ConnectorItem * c2) {
 bool ConnectorItem::isGrounded() {
 	QString name = connectorSharedName();
 	return ((name.compare("gnd", Qt::CaseInsensitive) == 0) || 
-			(name.compare("ground", Qt::CaseInsensitive) == 0));
+			// (name.compare("-", Qt::CaseInsensitive) == 0) || 
+			(name.compare("vss", Qt::CaseInsensitive) == 0) || 
+			(name.compare("ground", Qt::CaseInsensitive) == 0)
+			);
 }
 
 ConnectorItem * ConnectorItem::getCrossLayerConnectorItem() {
@@ -1596,15 +1616,14 @@ void ConnectorItem::paint(QPainter * painter, const QStyleOptionGraphicsItem * o
 		return;
 	}
 
-	if (!m_checkedEffectively) {
+	if (m_effectively == EffectivelyUnknown) {
 		if (!m_circular && m_shape.isEmpty()) {
 			if (this->attachedTo()->viewIdentifier() == ViewIdentifierClass::PCBView) {
 				QRectF r = rect();
-				m_effectivelyCircular = qAbs(r.width() - r.height()) < 0.01;
-				m_effectivelyRectangular = !m_effectivelyCircular;
+				if (qAbs(r.width() - r.height()) < 0.01) m_effectively = EffectivelyCircular;
+				else m_effectively = EffectivelyRectangular;
 			}
 		}
-		m_checkedEffectively = true;
 	}
 
 	NonConnectorItem::paint(painter, option, widget);
@@ -1752,7 +1771,7 @@ bool ConnectorItem::connectedToWires() {
 	return false;
 }
 
-void ConnectorItem::displayRatsnest(QList<ConnectorItem *> & partConnectorItems) {
+void ConnectorItem::displayRatsnest(QList<ConnectorItem *> & partConnectorItems, ViewGeometry::WireFlags myFlag) {
 	bool formerColorWasNamed = false;
 	bool gotFormerColor = false;
 	QColor formerColor;
@@ -1792,7 +1811,7 @@ void ConnectorItem::displayRatsnest(QList<ConnectorItem *> & partConnectorItems)
 	}
 
 	ConnectorPairHash result;
-	GraphUtils::chooseRatsnestGraph(partConnectorItems, result);
+	GraphUtils::chooseRatsnestGraph(&partConnectorItems, (ViewGeometry::RatsnestFlag | ViewGeometry::NormalFlag | ViewGeometry::PCBTraceFlag | ViewGeometry::SchematicTraceFlag) ^ myFlag, result);
 
 	foreach (ConnectorItem * key, result.uniqueKeys()) {
 		foreach (ConnectorItem * value, result.values(key)) {
@@ -1831,6 +1850,7 @@ void ConnectorItem::clearRatsnestDisplay(QList<ConnectorItem *> & connectorItems
 			c2->tempRemove(vw->connector1(), false);
 		}
 
+		vw->debugInfo("removing rat 1");
 		vw->scene()->removeItem(vw);
 		delete vw;
 	}
@@ -1860,7 +1880,7 @@ double ConnectorItem::calcClipRadius() {
 		return radius() - (strokeWidth() / 2.0);
 	}
 
-	if (m_effectivelyCircular) {
+	if (m_effectively == EffectivelyCircular) {
 		double rad = rect().width() / 2;
 		return rad - (rad / 5);
 	}
@@ -1869,27 +1889,29 @@ double ConnectorItem::calcClipRadius() {
 }
 
 bool ConnectorItem::isEffectivelyCircular() {
-	return m_circular || m_effectivelyCircular;
+	return m_circular || m_effectively == EffectivelyCircular;
 }
 
 void ConnectorItem::debugInfo(const QString & msg) 
 {
 
 #ifndef QT_NO_DEBUG
-	DebugDialog::debug(QString("%1 cid:%2 %3 %4 id:%5 %6 vlid:%7 vid:%8 spec:%9 flg:%10 hy:%11 bus:%12")
-		.arg(msg)
-		.arg(this->connectorSharedID())
-		.arg(this->connectorSharedName())
-		.arg(this->attachedToTitle())
-		.arg(this->attachedToID())
-		.arg(this->attachedToInstanceTitle())
-		.arg(this->attachedToViewLayerID())
-		.arg(this->attachedToViewIdentifier())
-		.arg(this->attachedToViewLayerSpec())
-		.arg(this->attachedTo()->wireFlags())
-		.arg(this->m_hybrid)
-		.arg((long) this->bus(), 0, 16)
-	);
+	QString s = QString("%1 cid:%2 %3 %4 id:%5 %6 vlid:%7 vid:%8 spec:%9 flg:%10 hy:%11 bus:%12")
+	//QString s = QString("%1 %2 %3 %4 id:%5 %6 vlid:%7 vid:%8")
+			.arg(msg)
+			.arg(this->connectorSharedID())
+			.arg(this->connectorSharedName())
+			.arg(this->attachedToTitle())
+			.arg(this->attachedToID())
+			.arg(this->attachedToInstanceTitle())
+			.arg(this->attachedToViewLayerID())
+			.arg(this->attachedToViewIdentifier())
+			.arg(this->attachedToViewLayerSpec())
+			.arg(this->attachedTo()->wireFlags())
+			.arg(this->m_hybrid)
+			.arg((long) this->bus(), 0, 16);
+	//s.replace(" ", "_");
+	DebugDialog::debug(s);
 #else
 	Q_UNUSED(msg);
 #endif
@@ -2304,6 +2326,15 @@ const QString & ConnectorItem::legID(ViewIdentifierClass::ViewIdentifier viewID,
 void ConnectorItem::setRubberBandLeg(QColor color, double strokeWidth, QLineF parentLine) {
 	// assumes this is only called once, when the connector is first set up
 
+	/*
+	this->debugInfo(QString("set rubber band leg %1 %2 %3 %4")
+		.arg(parentLine.p1().x())
+		.arg(parentLine.p1().y())
+		.arg(parentLine.p2().x())
+		.arg(parentLine.p2().y())
+		);
+	*/
+
 	m_rubberBandLeg = true;
 	setFlag(QGraphicsItem::ItemIsMovable, true);
 	setFlag(QGraphicsItem::ItemSendsGeometryChanges, true);
@@ -2481,6 +2512,11 @@ void ConnectorItem::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
 		return;
 	}
 
+	InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
+	if (infoGraphicsView != NULL) {
+		infoGraphicsView->setActiveConnectorItem(this);
+	}
+
 	if ((acceptedMouseButtons() & Qt::RightButton) == 0) {
 		event->ignore();
 		return;
@@ -2683,47 +2719,41 @@ void ConnectorItem::replaceBezier(int index, const Bezier * newBezier)
 	}
 }
 
-bool ConnectorItem::eventFilter(QObject * object, QEvent * event)
+void ConnectorItem::cursorKeyEvent(Qt::KeyboardModifiers modifiers)
 {
-	Q_UNUSED(object);
 	if (m_rubberBandLeg) {
-		if (!m_draggingLeg) {
-			if (event->type() == QEvent::KeyPress || event->type() == QEvent::KeyRelease) {
-				InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);;
-				if (infoGraphicsView) {
-					QPoint p = infoGraphicsView->mapFromGlobal(QCursor::pos());
-					QPointF r = infoGraphicsView->mapToScene(p);
-					QKeyEvent *keyEvent = static_cast<QKeyEvent*>(event);
-					// DebugDialog::debug(QString("got key event %1").arg(keyEvent->modifiers()));
-					updateLegCursor(mapFromScene(r), keyEvent->modifiers());
-				}
-			}
+		if (m_draggingLeg) return;
+
+		InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);;
+		if (infoGraphicsView) {
+			QPoint p = infoGraphicsView->mapFromGlobal(QCursor::pos());
+			QPointF r = infoGraphicsView->mapToScene(p);
+			// DebugDialog::debug(QString("got key event %1").arg(keyEvent->modifiers()));
+			updateLegCursor(mapFromScene(r), modifiers);
 		}
 	}
 	else if (attachedToItemType() == ModelPart::Wire) {
-		if (event->type() == QEvent::KeyPress || event->type() == QEvent::KeyRelease) {
-			QKeyEvent *keyEvent = static_cast<QKeyEvent*>(event);
-			updateWireCursor(keyEvent->modifiers());
-		}
+		updateWireCursor(modifiers);
 	}
-
-	return false;
 }
 
 void ConnectorItem::updateWireCursor(Qt::KeyboardModifiers modifiers)
 {
-	QCursor cursor = Qt::CrossCursor;
+	//DebugDialog::debug("uwc");
+	QCursor cursor = *CursorMaster::BendpointCursor;
 	if (isBendpoint()) {
-		cursor = *ConnectorItem::BendpointCursor;
+		//DebugDialog::debug("uwc bend");
 		if (modifiers & altOrMetaModifier()) {
+			DebugDialog::debug("uwc alt");
 			Wire * wire = qobject_cast<Wire *>(attachedTo());
 			if (wire != NULL && wire->canChainMultiple()) {
-				cursor = *ConnectorItem::MakeWireCursor;
+				//DebugDialog::debug("uwc make wire");
+				cursor = *CursorMaster::MakeWireCursor;
 			}
 		}
 	}
 
-	QApplication::changeOverrideCursor(cursor);
+	CursorMaster::instance()->addCursor(this, cursor);
 }
 
 void ConnectorItem::updateLegCursor(QPointF p, Qt::KeyboardModifiers modifiers)
@@ -2736,48 +2766,19 @@ void ConnectorItem::updateLegCursor(QPointF p, Qt::KeyboardModifiers modifiers)
 			cursor = *attachedTo()->getCursor(modifiers);
 			break;
 		case InBendpoint:
-			cursor = *BendpointCursor;
+			cursor = *CursorMaster::BendpointCursor;
 			break;
 		case InSegment:
-			cursor = curvyWiresIndicated(modifiers) ? *MakeCurveCursor : *NewBendpointCursor;
+			cursor = curvyWiresIndicated(modifiers) ? *CursorMaster::MakeCurveCursor : *CursorMaster::NewBendpointCursor;
 			break;
 		case InConnector:
-			cursor = (modifiers & altOrMetaModifier()) ? *MakeWireCursor : Qt::CrossCursor;
+			cursor = (modifiers & altOrMetaModifier()) ? *CursorMaster::MakeWireCursor : *CursorMaster::BendlegCursor;
 			break;
 		default:
 			cursor = Qt::ArrowCursor;
 			break;
 	}
-	QApplication::changeOverrideCursor(cursor);
-}
-
-void ConnectorItem::initCursors()
-{
-	if (BendpointCursor == NULL) {
-		QBitmap bitmap1(":resources/images/cursor/bendpoint.bmp");
-		QBitmap bitmap1m(":resources/images/cursor/bendpoint_mask.bmp");
-		BendpointCursor = new QCursor(bitmap1, bitmap1m, 0, 0);
-
-		QBitmap bitmap2(":resources/images/cursor/new_bendpoint.bmp");
-		QBitmap bitmap2m(":resources/images/cursor/new_bendpoint_mask.bmp");
-		NewBendpointCursor = new QCursor(bitmap2, bitmap2m, 0, 0);
-
-		QBitmap bitmap3(":resources/images/cursor/make_wire.bmp");
-		QBitmap bitmap3m(":resources/images/cursor/make_wire_mask.bmp");
-		MakeWireCursor = new QCursor(bitmap3, bitmap3m, 0, 0);
-
-		QBitmap bitmap4(":resources/images/cursor/curve.bmp");
-		QBitmap bitmap4m(":resources/images/cursor/curve_mask.bmp");
-		MakeCurveCursor = new QCursor(bitmap4, bitmap4m, 0, 0);
-
-		QBitmap bitmap5(":resources/images/cursor/rubberband_move.bmp");
-		QBitmap bitmap5m(":resources/images/cursor/rubberband_move_mask.bmp");
-		RubberbandCursor = new QCursor(bitmap5, bitmap5m, 0, 0);
-
-		QBitmap bitmap6(":resources/images/cursor/part_move.bmp");
-		QBitmap bitmap6m(":resources/images/cursor/part_move_mask.bmp");
-		MoveCursor = new QCursor(bitmap6, bitmap6m, 0, 0);
-	}
+	CursorMaster::instance()->addCursor(this, cursor);
 }
 
 bool ConnectorItem::curvyWiresIndicated(Qt::KeyboardModifiers modifiers)
@@ -2800,3 +2801,23 @@ bool ConnectorItem::isBendpoint()
 
 	return true;
 }
+
+void ConnectorItem::setConnectorLocalName(const QString & name) 
+{
+	if (m_connector) {
+		m_connector->setConnectorLocalName(name);
+	}
+}
+
+bool ConnectorItem::isGroundFillSeed() 
+{
+	return m_groundFillSeed;
+}
+
+void ConnectorItem::setGroundFillSeed(bool seed) 
+{
+	m_groundFillSeed = seed;
+	ConnectorItem * cross = this->getCrossLayerConnectorItem();
+	if (cross) cross->m_groundFillSeed = seed;
+}
+
