@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5417 $:
+$Revision: 5929 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-08-18 12:47:41 +0200 (Thu, 18 Aug 2011) $
+$Date: 2012-03-28 12:13:01 +0200 (Wed, 28 Mar 2012) $
 
 ********************************************************************/
 
@@ -42,11 +42,14 @@ $Date: 2011-08-18 12:47:41 +0200 (Thu, 18 Aug 2011) $
 #include "partsbinpalette/binmanager/binmanager.h"
 #include "help/tipsandtricks.h"
 #include "utils/folderutils.h"
+#include "utils/lockmanager.h"
 #include "dialogs/translatorlistmodel.h"
 #include "partsbinpalette/svgiconwidget.h"
 #include "items/moduleidnames.h"
 #include "partsbinpalette/searchlineedit.h"
 #include "utils/ratsnestcolors.h"
+#include "utils/cursormaster.h"
+#include "utils/textutils.h"
 #include "infoview/htmlinfoview.h"
 #include "svg/gedaelement2svg.h"
 #include "svg/kicadmodule2svg.h"
@@ -56,9 +59,9 @@ $Date: 2011-08-18 12:47:41 +0200 (Thu, 18 Aug 2011) $
 #include "items/pinheader.h"
 #include "items/partfactory.h"
 #include "dialogs/recoverydialog.h"
-#include "lib/qtlockedfile/qtlockedfile.h"
 #include "lib/qtsysteminfo/QtSystemInfo.h"
 #include "processeventblocker.h"
+#include "autoroute/cmrouter/panelizer.h"
 
 // dependency injection :P
 #include "referencemodel/sqlitereferencemodel.h"
@@ -79,7 +82,6 @@ $Date: 2011-08-18 12:47:41 +0200 (Thu, 18 Aug 2011) $
 #include <QNetworkRequest>
 #include <QMultiHash>
 
-static int kBottomOfAlpha = 204;
 static QNetworkAccessManager * NetworkAccessManager = NULL;
 
 #ifdef LINUX_32
@@ -117,7 +119,7 @@ static const double LoadProgressEnd = 0.6;
 
 FApplication::FApplication( int & argc, char ** argv) : QApplication(argc, argv)
 {
-	MainWindow::RestartNeeded = RestartNeeded;
+	MainWindow::RestartNeeded = FApplication::RestartNeeded;
 	m_spaceBarIsPressed = false;
 	m_mousePressed = false;
 	m_referenceModel = NULL;
@@ -125,11 +127,8 @@ FApplication::FApplication( int & argc, char ** argv) : QApplication(argc, argv)
 	m_started = false;
 	m_updateDialog = NULL;
 	m_lastTopmostWindow = NULL;
-	m_runAsService = false;
-	m_gerberService = false;
-	m_gedaService = false;
-	m_kicadFootprintService = false;
-	m_kicadSchematicService = false;
+	m_serviceType = NoService;
+	m_splash = NULL;
 
 	m_arguments = arguments();
 }
@@ -140,6 +139,8 @@ bool FApplication::init() {
 		//DebugDialog::debug(QString("argument %1").arg(argument));
 	//}
 
+	m_serviceType = NoService;
+
 	QList<int> toRemove;
 	for (int i = 0; i < m_arguments.length(); i++) {
 		if ((m_arguments[i].compare("-h", Qt::CaseInsensitive) == 0) || 
@@ -149,6 +150,21 @@ bool FApplication::init() {
 			return false;
 		}
 
+		if ((m_arguments[i].compare("-e", Qt::CaseInsensitive) == 0) ||
+			(m_arguments[i].compare("-examples", Qt::CaseInsensitive) == 0)||
+			(m_arguments[i].compare("--examples", Qt::CaseInsensitive) == 0)) {
+			m_serviceType = ExampleService;
+			m_outputFolder = " ";					// otherwise program will bail out
+			toRemove << i;
+		}
+
+		if ((m_arguments[i].compare("-d", Qt::CaseInsensitive) == 0) ||
+			(m_arguments[i].compare("-debug", Qt::CaseInsensitive) == 0)||
+			(m_arguments[i].compare("--debug", Qt::CaseInsensitive) == 0)) {
+			DebugDialog::setEnabled(true);
+			toRemove << i;
+		}
+
 		if (i + 1 >= m_arguments.length()) continue;
 
 		if ((m_arguments[i].compare("-f", Qt::CaseInsensitive) == 0) ||
@@ -157,62 +173,69 @@ bool FApplication::init() {
 		{
 			FolderUtils::setApplicationPath(m_arguments[i + 1]);
 			// delete these so we don't try to process them as files later
-			toRemove << i;
-			toRemove << i + 1;
+			toRemove << i << i + 1;
 		}
 
 		if ((m_arguments[i].compare("-geda", Qt::CaseInsensitive) == 0) ||
 			(m_arguments[i].compare("--geda", Qt::CaseInsensitive) == 0)) {
-			m_runAsService = true;
-			m_gedaService = true;
+			m_serviceType = GedaService;
 			m_outputFolder = m_arguments[i + 1];
-			toRemove << i;
-			toRemove << i + 1;
+			toRemove << i << i + 1;
 		}
 
 		if ((m_arguments[i].compare("-kicad", Qt::CaseInsensitive) == 0) ||
 			(m_arguments[i].compare("--kicad", Qt::CaseInsensitive) == 0)) {
-			m_runAsService = true;
-			m_kicadFootprintService = true;
+			m_serviceType = KicadFootprintService;
 			m_outputFolder = m_arguments[i + 1];
-			toRemove << i;
-			toRemove << i + 1;
+			toRemove << i << i + 1;
 		}
 
 		if ((m_arguments[i].compare("-kicadschematic", Qt::CaseInsensitive) == 0) ||
 			(m_arguments[i].compare("--kicadschematic", Qt::CaseInsensitive) == 0)) {
-			m_runAsService = true;
-			m_kicadSchematicService = true;
+			m_serviceType = KicadSchematicService;
 			m_outputFolder = m_arguments[i + 1];
-			toRemove << i;
-			toRemove << i + 1;
+			toRemove << i << i + 1;
 		}
 
 		if ((m_arguments[i].compare("-g", Qt::CaseInsensitive) == 0) ||
 			(m_arguments[i].compare("-gerber", Qt::CaseInsensitive) == 0)||
 			(m_arguments[i].compare("--gerber", Qt::CaseInsensitive) == 0)) {
-			m_runAsService = true;
-			m_gerberService = true;
+			m_serviceType = GerberService;
 			m_outputFolder = m_arguments[i + 1];
-			toRemove << i;
+			toRemove << i << i + 1;
+		}
+
+		if ((m_arguments[i].compare("-p", Qt::CaseInsensitive) == 0) ||
+			(m_arguments[i].compare("-panel", Qt::CaseInsensitive) == 0)||
+			(m_arguments[i].compare("--panel", Qt::CaseInsensitive) == 0)) {
+			m_serviceType = PanelizerService;
+			m_panelFilename = m_arguments[i + 1];
+			m_outputFolder = " ";					// otherwise program will bail out
+			toRemove << i << i + 1;
+		}
+
+		if ((m_arguments[i].compare("-i", Qt::CaseInsensitive) == 0) ||
+			(m_arguments[i].compare("-inscription", Qt::CaseInsensitive) == 0)||
+			(m_arguments[i].compare("--inscription", Qt::CaseInsensitive) == 0)) {
+			m_serviceType = InscriptionService;
+			m_panelFilename = m_arguments[i + 1];
+			m_outputFolder = " ";					// otherwise program will bail out
+			toRemove << i << i + 1;
 		}
 
 		if (m_arguments[i].compare("-ep", Qt::CaseInsensitive) == 0) {
 			m_externalProcessPath = m_arguments[i + 1];
-			toRemove << i;
-			toRemove << (i + 1);
+			toRemove << i << i + 1;
 		}
 
 		if (m_arguments[i].compare("-eparg", Qt::CaseInsensitive) == 0) {
 			m_externalProcessArgs << m_arguments[i + 1];
-			toRemove << i;
-			toRemove << (i + 1);
+			toRemove << i << i + 1;
 		}
 
 		if (m_arguments[i].compare("-epname", Qt::CaseInsensitive) == 0) {
 			m_externalProcessName = m_arguments[i + 1];
-			toRemove << i;
-			toRemove << (i + 1);
+			toRemove << i << i + 1;
 		}
 
 	}
@@ -238,7 +261,7 @@ bool FApplication::init() {
 
 	// tell app where to search for plugins (jpeg export and sql lite)
 	m_libPath = FolderUtils::getLibraryPath();
-	addLibraryPath(m_libPath);	
+	QApplication::addLibraryPath(m_libPath);	
 
 	/*QFile file("libpath.txt");
 	if (file.open(QIODevice::WriteOnly | QIODevice::Text)) {
@@ -258,6 +281,7 @@ bool FApplication::init() {
 	MainWindow::initNames();
 	FSvgRenderer::calcPrinterScale();
 	ViewIdentifierClass::initNames();
+	RatsnestColors::initNames();
 	Wire::initNames();
 	ItemBase::initNames();
 	ViewLayer::initNames();
@@ -266,9 +290,10 @@ bool FApplication::init() {
 	PartsEditorMainWindow::initText();
 	BinManager::initNames();
 	PaletteModel::initNames();
-	RatsnestColors::initNames();
 	SvgIconWidget::initNames();
 	PinHeader::initNames();
+	CursorMaster::initCursors();
+
 	return true;
 }
 
@@ -393,7 +418,7 @@ bool FApplication::findTranslator(const QString & translationsPath) {
 
     bool loaded = m_translator.load(QString("fritzing_") + suffix, translationsPath);
 	if (loaded) {
-		this->installTranslator(&m_translator);
+		QApplication::installTranslator(&m_translator);
 	}
 
 	return loaded;
@@ -415,14 +440,14 @@ void FApplication::registerFonts() {
 	*/	
 }
 
-void FApplication::loadReferenceModel() {
+ReferenceModel * FApplication::loadReferenceModel() {
 	m_referenceModel = new CurrentReferenceModel();	
 	ItemBase::setReferenceModel(m_referenceModel);
 	connect(m_referenceModel, SIGNAL(loadedPart(int, int)), this, SLOT(loadedPart(int, int)));
 	m_referenceModel->loadAll(true);								// this is very slow
-	//DebugDialog::debug("after new current reference model");
 	m_paletteBinModel = new PaletteModel(true, false, false);
 	//DebugDialog::debug("after new palette model");
+	return m_referenceModel;
 }
 
 bool FApplication::loadBin(QString binToOpen) {
@@ -438,9 +463,9 @@ bool FApplication::loadBin(QString binToOpen) {
 	return true;
 }
 
-MainWindow * FApplication::loadWindows(int & loaded) {
+MainWindow * FApplication::loadWindows(int & loaded, bool lockFiles) {
 	// our MainWindows use WA_DeleteOnClose so this has to be added to the heap (via new) rather than the stack (for local vars)
-	MainWindow * mainWindow = MainWindow::newMainWindow(m_paletteBinModel, m_referenceModel, "", false);   // this is also slow
+	MainWindow * mainWindow = MainWindow::newMainWindow(m_paletteBinModel, m_referenceModel, "", false, lockFiles);   // this is also slow
 	mainWindow->setReportMissingModules(false);
 
 	loaded = 0;
@@ -460,29 +485,50 @@ int FApplication::serviceStartup() {
 		return -1;
 	}
 
-	if (m_gedaService) {
-		runGedaService();
-		return 0;
-	}
-	else if (m_kicadFootprintService) {
-		runKicadFootprintService();
-		return 0;
-	}
-	else if (m_kicadSchematicService) {
-		runKicadSchematicService();
-		return 0;
-	}
+	switch (m_serviceType) {
+		case GedaService:
+			runGedaService();
+			return 0;
+	
+		case KicadFootprintService:
+			runKicadFootprintService();
+			return 0;
 
-	if (!m_gerberService) {
-		return 0;
-	}
+		case KicadSchematicService:
+			runKicadSchematicService();
+			return 0;
 
+		case GerberService:
+			runGerberService();
+			return 0;
+
+		case PanelizerService:
+			runPanelizerService();
+			return 0;
+
+		case InscriptionService:
+			runInscriptionService();
+			return 0;
+
+		case ExampleService:
+			runExampleService();
+			return 0;
+
+		default:
+			DebugDialog::debug("unknown service");
+			return -1;
+	}
+}
+
+
+void FApplication::runGerberService()
+{
 	createUserDataStoreFolderStructure();
 
 	registerFonts();
 	loadReferenceModel();
 	if (!loadBin("")) {
-		return -1;
+		return;
 	}
 
 	QDir dir(m_outputFolder);
@@ -493,19 +539,18 @@ int FApplication::serviceStartup() {
 	foreach (QString filename, filenames) {
 		QString filepath = dir.absoluteFilePath(filename);
 		int loaded = 0;
-		MainWindow * mainWindow = loadWindows(loaded);
+		MainWindow * mainWindow = loadWindows(loaded, false);
+		mainWindow->noBackup();
 		m_started = true;
 
 		FolderUtils::setOpenSaveFolderAux(m_outputFolder);
-		if (mainWindow->loadWhich(filepath, false, false, true)) {
+		if (mainWindow->loadWhich(filepath, false, false, "")) {
 			mainWindow->exportToGerber(m_outputFolder);
 		}
 
 		mainWindow->setCloseSilently(true);
 		mainWindow->close();
 	}
-
-	return 0;
 }
 
 void FApplication::runGedaService() {
@@ -627,12 +672,24 @@ void FApplication::runKicadSchematicService() {
 
 int FApplication::startup(bool firstRun)
 {
-	QPixmap pixmap(":/resources/images/splash_2010.png");
+	//DebugDialog::setEnabled(true);
+
+	QString splashName = ":/resources/images/splash/splash_screen_start.png";
+	QDateTime now = QDateTime::currentDateTime();
+	if (now.date().month() == 4 && now.date().day() == 1) {
+		QString aSplashName = ":/resources/images/splash/april1st.png";
+		QFileInfo info(aSplashName);
+		if (info.exists()) {
+			splashName = aSplashName;
+		}
+	}
+
+	QPixmap pixmap(splashName);
 	FSplashScreen splash(pixmap);
 	m_splash = &splash;
 	ProcessEventBlocker::processEvents();								// seems to need this (sometimes?) to display the splash screen
 
-	initSplash(splash, pixmap);
+	initSplash(splash);
 	ProcessEventBlocker::processEvents();
 
 	// DebugDialog::debug("Data Location: "+QDesktopServices::storageLocation(QDesktopServices::DataLocation));
@@ -640,7 +697,7 @@ int FApplication::startup(bool firstRun)
 	if(firstRun) {		
 		registerFonts();
 
-		splash.showProgress(m_progressIndex, LoadProgressStart);
+		if (m_progressIndex >= 0) splash.showProgress(m_progressIndex, LoadProgressStart);
 		ProcessEventBlocker::processEvents();
 
 		#ifdef Q_WS_WIN
@@ -659,6 +716,8 @@ int FApplication::startup(bool firstRun)
 							   .arg("%1") );
 		#endif
 
+
+		cleanFzzs();
 	} 
 	else 
 	{
@@ -668,11 +727,19 @@ int FApplication::startup(bool firstRun)
 
 	loadReferenceModel();
 
-	QSettings settings;
-	QString prevVersion = settings.value("version").toString();
-	QString currVersion = Version::versionString();
-	if(prevVersion != currVersion) {
-		settings.clear();
+	QString prevVersion;
+	{
+		// put this in a block so that QSettings is closed
+		QSettings settings;
+		prevVersion = settings.value("version").toString();
+		QString currVersion = Version::versionString();
+		if(prevVersion != currVersion) {
+			QVariant pid = settings.value("pid");
+			settings.clear();
+			if (!pid.isNull()) {
+				settings.setValue("pid", pid);
+			}
+		}
 	}
 
 	//bool fabEnabled = settings.value(ORDERFABENABLED, QVariant(false)).toBool();
@@ -683,36 +750,39 @@ int FApplication::startup(bool firstRun)
 		NetworkAccessManager->get(QNetworkRequest(QUrl(QString("http://fab.fritzing.org/launched%1").arg(makeRequestParamsString()))));
 	//}
 
-	splash.showProgress(m_progressIndex, LoadProgressEnd);
+	if (m_progressIndex >= 0) splash.showProgress(m_progressIndex, LoadProgressEnd);
 
 	createUserDataStoreFolderStructure();
 
 	//DebugDialog::debug("after createUserDataStoreFolderStructure");
 
-	splash.showProgress(m_progressIndex, 0.65);
+	if (m_progressIndex >= 0) splash.showProgress(m_progressIndex, 0.65);
 	ProcessEventBlocker::processEvents();
 
+	QSettings settings;
 	if (!loadBin(settings.value("lastBin").toString())) {
 			// TODO: we're really screwed, what now?
 		QMessageBox::warning(NULL, QObject::tr("Fritzing"), QObject::tr("Friting cannot load the parts bin"));
 		return -1;
 	}
 
-	splash.showProgress(m_progressIndex, 0.825);
+	if (m_progressIndex >= 0) splash.showProgress(m_progressIndex, 0.825);
 	ProcessEventBlocker::processEvents();
 
 	m_updateDialog = new UpdateDialog();
 	connect(m_updateDialog, SIGNAL(enableAgainSignal(bool)), this, SLOT(enableCheckUpdates(bool)));
 	checkForUpdates(false);
 
-	splash.showProgress(m_progressIndex, 0.875);
+	if (m_progressIndex >= 0) splash.showProgress(m_progressIndex, 0.875);
 
+	DebugDialog::debug("load something");
 	loadSomething(firstRun, prevVersion);
 	m_started = true;
 
-	splash.showProgress(m_progressIndex, 0.99);
+	if (m_progressIndex >= 0) splash.showProgress(m_progressIndex, 0.99);
 	ProcessEventBlocker::processEvents();
 
+	m_splash = NULL;
 	return 0;
 }
 
@@ -737,8 +807,8 @@ void FApplication::finish()
 }
 
 void FApplication::loadNew(QString path) {
-	MainWindow * mw = MainWindow::newMainWindow(m_paletteBinModel, m_referenceModel, path, true);
-	if (!mw->loadWhich(path, false, true, false)) {
+	MainWindow * mw = MainWindow::newMainWindow(m_paletteBinModel, m_referenceModel, path, true, true);
+	if (!mw->loadWhich(path, false, true, "")) {
 		mw->close();
 	}
 	mw->clearFileProgressDialog();
@@ -747,7 +817,7 @@ void FApplication::loadNew(QString path) {
 void FApplication::loadOne(MainWindow * mw, QString path, int loaded) {
 	if (loaded == 0) {
 		mw->showFileProgressDialog(path);
-		mw->loadWhich(path, true, true, false);
+		mw->loadWhich(path, true, true, "");
 	}
 	else {
 		loadNew(path);
@@ -766,8 +836,8 @@ void FApplication::preferences() {
 	}
 
 	MainWindow * mainWindow = NULL;
-	foreach (QWidget * widget, m_orderedTopLevelWidgets) {
-		mainWindow = (MainWindow *) widget;
+	foreach (MainWindow * mw, orderedTopLevelMainWindows()) {
+		mainWindow = mw;
 		break;
 	}
 
@@ -798,20 +868,21 @@ void FApplication::updatePrefs(PrefsDialog & prefsDialog)
 	}
 
 	QHash<QString, QString> hash = prefsDialog.settings();
+	QList<MainWindow *> mainWindows = orderedTopLevelMainWindows();
 	foreach (QString key, hash.keys()) {
 		settings.setValue(key, hash.value(key));
 		if (key.compare("connectedColor") == 0) {
 			QColor c(hash.value(key));
 			ItemBase::setConnectedColor(c);
-			foreach (QWidget * widget, m_orderedTopLevelWidgets) {
-				((MainWindow *) widget)->redrawSketch();
+			foreach (MainWindow * mainWindow, mainWindows) {
+				mainWindow->redrawSketch();
 			}
 		}
 		else if (key.compare("unconnectedColor") == 0) {
 			QColor c(hash.value(key));
 			ItemBase::setUnconnectedColor(c);
-			foreach (QWidget * widget, m_orderedTopLevelWidgets) {
-				((MainWindow *) widget)->redrawSketch();
+			foreach (MainWindow * mainWindow, mainWindows) {
+				mainWindow->redrawSketch();
 			}
 		}
 		else if (key.compare("wheelMapping") == 0) {
@@ -824,8 +895,8 @@ void FApplication::updatePrefs(PrefsDialog & prefsDialog)
 			MainWindow::setAutosaveEnabled(hash.value(key).toInt());
 		}
 		else if (key.contains("gridsize", Qt::CaseInsensitive)) {
-			foreach (QWidget * widget, m_orderedTopLevelWidgets) {
-				foreach (SketchWidget * sketchWidget, ((MainWindow *) widget)->sketchWidgets()) {
+			foreach (MainWindow * mainWindow, mainWindows) {
+				foreach (SketchWidget * sketchWidget, mainWindow->sketchWidgets()) {
 					if (key.contains(sketchWidget->viewName())) {
 						sketchWidget->initGrid();
 					}
@@ -833,8 +904,8 @@ void FApplication::updatePrefs(PrefsDialog & prefsDialog)
 			}
 		}
 		else if (key.contains("curvy", Qt::CaseInsensitive)) {
-			foreach (QWidget * widget, m_orderedTopLevelWidgets) {
-				foreach (SketchWidget * sketchWidget, ((MainWindow *) widget)->sketchWidgets()) {
+			foreach (MainWindow * mainWindow, mainWindows) {
+				foreach (SketchWidget * sketchWidget, mainWindow->sketchWidgets()) {
 					if (key.contains(sketchWidget->getShortName())) {
 						sketchWidget->setCurvyWires(hash.value(key).compare("1") == 0);
 					}
@@ -846,8 +917,8 @@ void FApplication::updatePrefs(PrefsDialog & prefsDialog)
 	hash = prefsDialog.tempSettings();
 	foreach (QString key, hash.keys()) {
 		if (key.contains("background", Qt::CaseInsensitive)) {
-			foreach (QWidget * widget, m_orderedTopLevelWidgets) {
-				foreach (SketchWidget * sketchWidget, ((MainWindow *) widget)->sketchWidgets()) {
+			foreach (MainWindow * mainWindow, mainWindows) {
+				foreach (SketchWidget * sketchWidget, mainWindow->sketchWidgets()) {
 					if (key.contains(sketchWidget->getShortName())) {
 						sketchWidget->setBackground(hash.value(key));
 					}
@@ -857,25 +928,22 @@ void FApplication::updatePrefs(PrefsDialog & prefsDialog)
 	}
 }
 
-void FApplication::initSplash(FSplashScreen & splash, QPixmap & pixmap) {
-	QPixmap logo(":/resources/images/fhp_logo_small.png");
-	QPixmap progress(":/resources/images/splash_progressbar.png");
+void FApplication::initSplash(FSplashScreen & splash) {
+	QPixmap logo(":/resources/images/splash/fhp_logo_small.png");
+	QPixmap progress(":/resources/images/splash/splash_progressbar.png");
 
-	m_progressIndex = splash.showPixmap(progress, QPoint(0, pixmap.height() - progress.height()));
-	splash.showProgress(m_progressIndex, 0);
+	m_progressIndex = splash.showPixmap(progress, "progress");
+	if (m_progressIndex >= 0) splash.showProgress(m_progressIndex, 0);
 
 	// put this above the progress indicator
-	splash.showPixmap(logo, QPoint(5, pixmap.height() - 12));
+	splash.showPixmap(logo, "fhpLogo");
 
-	QColor w(0xea, 0xf4, 0xed);
-	QRect r1(45, kBottomOfAlpha, pixmap.width() - 45, 20);
 	QString msg1 = QObject::tr("<font face='Lucida Grande, Tahoma, Sans Serif' size='2' color='#eaf4ed'>"
 							   "&#169; 2007-%1 Fachhochschule Potsdam"
 							   "</font>")
-	.arg(Version::year());
-	splash.showMessage(msg1, r1, Qt::AlignLeft | Qt::AlignTop, w);
+							.arg(Version::year());
+	splash.showMessage(msg1, "fhpText", Qt::AlignLeft | Qt::AlignTop);
 
-	QRect r2(0, kBottomOfAlpha, pixmap.width() - 12, 20);
 	QString macBuildType;
 #ifdef Q_WS_MAC
 #ifdef QT_MAC_USE_COCOA
@@ -893,7 +961,7 @@ void FApplication::initSplash(FSplashScreen & splash, QPixmap & pixmap) {
 						.arg(Version::modifier())
 						.arg(Version::shortDate())
 						.arg(macBuildType);
-	splash.showMessage(msg2, r2, Qt::AlignRight | Qt::AlignTop, w);
+	splash.showMessage(msg2, "versionText", Qt::AlignRight | Qt::AlignTop);
     splash.show();
 }
 
@@ -945,12 +1013,14 @@ void FApplication::checkForUpdates(bool atUserRequest)
 
 void FApplication::enableCheckUpdates(bool enabled)
 {
+	DebugDialog::debug("before enable check updates");
     foreach (QWidget *widget, QApplication::topLevelWidgets()) {
         MainWindow *mainWindow = qobject_cast<MainWindow *>(widget);
 		if (mainWindow) {
 			mainWindow->enableCheckUpdates(enabled);
 		}
 	}
+	DebugDialog::debug("after enable check updates");
 }
 
 
@@ -969,6 +1039,7 @@ void FApplication::createUserDataStoreFolderStructure() {
     copyBin(BinManager::MyPartsBinLocation, BinManager::MyPartsBinTemplateLocation);
     copyBin(BinManager::SearchBinLocation, BinManager::SearchBinTemplateLocation);
 	PartFactory::initFolder();
+
 }
 
 void FApplication::copyBin(const QString & dest, const QString & source) {
@@ -1021,7 +1092,7 @@ void FApplication::changeActivation(bool activate, QWidget * originator) {
 }
 
 void FApplication::updateActivation() {
-	// DebugDialog::debug("updating activation");
+	//DebugDialog::debug("updating activation");
 
 	FritzingWindow * prior = m_lastTopmostWindow; 
 	m_lastTopmostWindow = NULL;
@@ -1029,7 +1100,10 @@ void FApplication::updateActivation() {
 		m_lastTopmostWindow = qobject_cast<FritzingWindow *>(m_orderedTopLevelWidgets.at(0));
 	}
 
-	if (prior == m_lastTopmostWindow) return;
+	if (prior == m_lastTopmostWindow) {
+		//DebugDialog::debug("done updating activation");
+		return;
+	}
 
 	//DebugDialog::debug(QString("last:%1, new:%2").arg((long) prior, 0, 16).arg((long) m_lastTopmostWindow.data(), 0, 16));
 
@@ -1043,6 +1117,9 @@ void FApplication::updateActivation() {
 		lastTopmostMainWindow->restoreDocks();
 		//DebugDialog::debug("restoring active window");
 	}
+
+	//DebugDialog::debug("done 2 updating activation");
+
 }
 
 void FApplication::topLevelWidgetDestroyed(QObject * object) {
@@ -1075,7 +1152,7 @@ which is really not intended for hundreds of widgets.
 
     bool did_close = true;
     QWidget *w;
-    while((w = activeModalWidget()) && did_close) {
+    while((w = QApplication::activeModalWidget()) && did_close) {
         if(!w->isVisible())
             break;
         did_close = w->close();
@@ -1109,16 +1186,15 @@ which is really not intended for hundreds of widgets.
 }
 
 bool FApplication::runAsService() {
-	return ((FApplication *) qApp)->m_runAsService;
+	return ((FApplication *) qApp)->m_serviceType != NoService;
 }
 
 void FApplication::loadedPart(int loaded, int total) {
 	if (total == 0) return;
 	if (m_splash == NULL) return;
 
-
-
-	m_splash->showProgress(m_progressIndex, LoadProgressStart + ((LoadProgressEnd - LoadProgressStart) * loaded / (double) total));
+	//DebugDialog::debug(QString("loaded %1 %2").arg(loaded).arg(total));
+	if (m_progressIndex >= 0) m_splash->showProgress(m_progressIndex, LoadProgressStart + ((LoadProgressEnd - LoadProgressStart) * loaded / (double) total));
 }
 
 void FApplication::externalProcessSlot(QString &name, QString &path, QStringList &args) {
@@ -1161,7 +1237,7 @@ void FApplication::loadSomething(bool firstRun, const QString & prevVersion) {
 
 	initBackups();
 
-	//DebugDialog::debug("checking for backups");
+	DebugDialog::debug("checking for backups");
     QList<MainWindow*> sketchesToLoad = recoverBackups();
 
 	bool loadPrevious = false;
@@ -1169,7 +1245,7 @@ void FApplication::loadSomething(bool firstRun, const QString & prevVersion) {
 		loadPrevious = !prevVersion.isEmpty() && Version::greaterThan(prevVersion, Version::FirstVersionWithDetachedUserData);
 	}
 
-	//DebugDialog::debug(QString("load previous %1").arg(loadPrevious));
+	DebugDialog::debug(QString("load previous %1").arg(loadPrevious));
 
 	if (!loadPrevious && sketchesToLoad.isEmpty()) {
 		if (!firstRun) {
@@ -1180,26 +1256,26 @@ void FApplication::loadSomething(bool firstRun, const QString & prevVersion) {
 
 	if (!loadPrevious && sketchesToLoad.isEmpty()) {
 		// Check for double-clicked files to load
-		//DebugDialog::debug(QString("check files to load %1").arg(m_filesToLoad.count()));
+		DebugDialog::debug(QString("check files to load %1").arg(m_filesToLoad.count()));
 
         foreach (QString filename, m_filesToLoad) {
             DebugDialog::debug(QString("Loading non-service file %1").arg(filename));
-            MainWindow *mainWindow = MainWindow::newMainWindow(m_paletteBinModel, m_referenceModel, filename, true);
-            mainWindow->loadWhich(filename, true, true, false);
+            MainWindow *mainWindow = MainWindow::newMainWindow(m_paletteBinModel, m_referenceModel, filename, true, true);
+            mainWindow->loadWhich(filename, true, true, "");
             sketchesToLoad << mainWindow;
         }
 	}
 
     // Find any previously open sketches to reload
     if (!loadPrevious && sketchesToLoad.isEmpty()) {
-		//DebugDialog::debug(QString("load last open"));
+		DebugDialog::debug(QString("load last open"));
 		sketchesToLoad = loadLastOpenSketch();
 	}
 
 	MainWindow * newBlankSketch = NULL;
 	if (sketchesToLoad.isEmpty()) {
-		//DebugDialog::debug(QString("empty sketch"));
-		newBlankSketch = MainWindow::newMainWindow(m_paletteBinModel, m_referenceModel, "", true);
+		DebugDialog::debug(QString("create empty sketch"));
+		newBlankSketch = MainWindow::newMainWindow(m_paletteBinModel, m_referenceModel, "", true, true);
 		if (newBlankSketch) {
 			// make sure to start an empty sketch with a board
 			newBlankSketch->addDefaultParts();   // do this before call to show()
@@ -1207,6 +1283,8 @@ void FApplication::loadSomething(bool firstRun, const QString & prevVersion) {
 		}
 	}
 
+	DebugDialog::debug(QString("finish up sketch loading"));
+
     // Finish loading the sketches and show them to the user
 	foreach (MainWindow* sketch, sketchesToLoad) {
 		sketch->show();
@@ -1216,6 +1294,9 @@ void FApplication::loadSomething(bool firstRun, const QString & prevVersion) {
 	if (loadPrevious) {
 		doLoadPrevious(newBlankSketch);
 	}
+	else if (newBlankSketch) {
+		newBlankSketch->hideTempBin();
+	}
 }
 
 QList<MainWindow *> FApplication::loadLastOpenSketch() {
@@ -1231,8 +1312,8 @@ QList<MainWindow *> FApplication::loadLastOpenSketch() {
 
     DebugDialog::debug(QString("Loading last open sketch %1").arg(lastSketchPath));
     settings.remove("lastOpenSketch");				// clear the preference, in case the load crashes
-    MainWindow *mainWindow = MainWindow::newMainWindow(m_paletteBinModel, m_referenceModel, lastSketchPath, true);
-    mainWindow->load(lastSketchPath, true, true, "");
+    MainWindow *mainWindow = MainWindow::newMainWindow(m_paletteBinModel, m_referenceModel, lastSketchPath, true, true);
+    mainWindow->loadWhich(lastSketchPath, true, true, "");
     sketches << mainWindow;
     settings.setValue("lastOpenSketch", lastSketchPath);	// the load works, so restore the preference
 	return sketches;
@@ -1263,10 +1344,14 @@ void FApplication::doLoadPrevious(MainWindow * sketchWindow) {
 
 QList<MainWindow *> FApplication::recoverBackups()
 {	
-
 	QFileInfoList backupList;
-	QStringList filters("*.fz");
-	FolderUtils::checkLockedFiles("backup", backupList, filters, m_lockedFiles, false);
+	LockManager::checkLockedFiles("backup", backupList, m_lockedFiles, false, LockManager::FastTime);
+	for (int i = backupList.size() - 1; i >=0; i--) {
+		QFileInfo fileInfo = backupList.at(i);
+		if (!fileInfo.fileName().endsWith(FritzingSketchExtension)) {
+			backupList.removeAt(i);
+		}
+	}
 
 	QList<MainWindow*> recoveredSketches;
     if (backupList.size() == 0) return recoveredSketches;
@@ -1276,22 +1361,40 @@ QList<MainWindow *> FApplication::recoverBackups()
     QList<QTreeWidgetItem*> fileItems = recoveryDialog.getFileList();
     DebugDialog::debug(QString("Recovering %1 files from recoveryDialog").arg(fileItems.size()));
     foreach (QTreeWidgetItem * item, fileItems) {
+		QString backupName = item->data(0, Qt::UserRole).value<QString>();
         if (result == QDialog::Accepted && item->isSelected()) {
 			QString originalBaseName = item->text(0);
             DebugDialog::debug(QString("Loading recovered sketch %1").arg(originalBaseName));
-            MainWindow *currentRecoveredSketch = MainWindow::newMainWindow(m_paletteBinModel, m_referenceModel, originalBaseName, true);
-			QString backupName = item->data(0, Qt::UserRole).value<QString>();
+			
 			QString originalPath = item->data(1, Qt::UserRole).value<QString>();
- 			currentRecoveredSketch->setRecovered(true);
-			currentRecoveredSketch->load(backupName, false, false, originalPath);		
-            currentRecoveredSketch->setCurrentFile(originalPath, false, true, backupName);
-            currentRecoveredSketch->setWindowModified(true);
-            currentRecoveredSketch->showAllFirstTimeHelp(false);
-            recoveredSketches << currentRecoveredSketch;
-        }
-        else {
-			QFile::remove(item->data(0, Qt::UserRole).value<QString>());
+			QString fileExt;
+			QString bundledFileName = FolderUtils::getSaveFileName(NULL, tr("Please specify an .fzz file name to save to (cancel will delete the backup)"), originalPath, tr("Fritzing (*%1)").arg(FritzingBundleExtension), &fileExt);
+			if (!bundledFileName.isEmpty()) {
+				MainWindow *currentRecoveredSketch = MainWindow::newMainWindow(m_paletteBinModel, m_referenceModel, originalBaseName, true, true);
+    			currentRecoveredSketch->mainLoad(backupName, bundledFileName);
+				currentRecoveredSketch->saveAsShareable(bundledFileName, true);
+				currentRecoveredSketch->setCurrentFile(bundledFileName, true, true);
+				currentRecoveredSketch->showAllFirstTimeHelp(false);
+				recoveredSketches << currentRecoveredSketch;
+
+				/*
+				if (originalPath.startsWith(untitledFileName())) {
+					DebugDialog::debug(QString("Comparing untitled documents: %1 %2").arg(filename).arg(untitledFileName()));
+					QRegExp regexp("\\d+");
+					int ix = regexp.indexIn(filename);
+					int untitledSketchNumber = ix >= 0 ? regexp.cap(0).toInt() : 1;
+					untitledSketchNumber++;
+					DebugDialog::debug(QString("%1 untitled documents open, currently thinking %2").arg(untitledSketchNumber).arg(UntitledSketchIndex));
+					UntitledSketchIndex = UntitledSketchIndex >= untitledSketchNumber ? UntitledSketchIndex : untitledSketchNumber;
+				}
+				*/
+
+
+
+			}
         }
+
+		QFile::remove(backupName);
     }
 
 	return recoveredSketches;
@@ -1308,11 +1411,11 @@ void FApplication::initFilesToLoad()
 }
 
 void FApplication::initBackups() {
-	FolderUtils::initLockedFiles("backup", MainWindow::BackupFolder, m_lockedFiles);
+	LockManager::initLockedFiles("backup", MainWindow::BackupFolder, m_lockedFiles, LockManager::FastTime);
 }
 
 void FApplication::cleanupBackups() {
-	FolderUtils::releaseLockedFiles(MainWindow::BackupFolder, m_lockedFiles);
+	LockManager::releaseLockedFiles(MainWindow::BackupFolder, m_lockedFiles);
 }
 
 void FApplication::gotOrderFab(QNetworkReply * networkReply) {
@@ -1346,3 +1449,85 @@ QString FApplication::makeRequestParamsString() {
 		.arg(siSystemVersion);
 	return string;
 }
+
+void FApplication::runPanelizerService()
+{	
+	m_started = true;
+	Panelizer::panelize(this, m_panelFilename);
+}
+
+void FApplication::runInscriptionService()
+{	
+	m_started = true;
+	Panelizer::inscribe(this, m_panelFilename);
+}
+
+QList<MainWindow *> FApplication::orderedTopLevelMainWindows() {
+	QList<MainWindow *> mainWindows;
+	foreach (QWidget * widget, m_orderedTopLevelWidgets) {
+		MainWindow * mainWindow = qobject_cast<MainWindow *>(widget);
+		if (mainWindow) mainWindows.append(mainWindow);
+	}
+	return mainWindows;
+}
+
+void FApplication::runExampleService()
+{	
+	m_started = true;
+
+	createUserDataStoreFolderStructure();
+	registerFonts();
+	loadReferenceModel();
+
+	if (!loadBin("")) {
+		DebugDialog::debug(QString("load bin failed"));
+		return;
+	}
+
+	QDir sketchesDir(FolderUtils::getApplicationSubFolderPath("sketches"));
+	runExampleService(sketchesDir);
+}
+
+void FApplication::runExampleService(QDir & dir) {
+	QStringList nameFilters;
+	nameFilters << ("*" + FritzingBundleExtension);   //  FritzingSketchExtension
+	QFileInfoList fileList = dir.entryInfoList(nameFilters, QDir::Files | QDir::NoSymLinks);
+	foreach (QFileInfo fileInfo, fileList) {
+		QString path = fileInfo.absoluteFilePath();
+		DebugDialog::debug("sketch file " + path);
+
+		int loaded = 0;
+		MainWindow * mainWindow = loadWindows(loaded, false);
+		if (mainWindow == NULL) continue;
+
+		mainWindow->noBackup();
+
+		FolderUtils::setOpenSaveFolderAux(dir.absolutePath());
+
+		if (!mainWindow->loadWhich(path, false, false, "")) {
+			DebugDialog::debug(QString("failed to load"));
+		}
+		else {
+			mainWindow->selectAllObsolete(false);
+			mainWindow->swapObsolete(false);
+			mainWindow->saveAsAux(path);    //   path + "z"
+			mainWindow->setCloseSilently(true);
+			mainWindow->close();
+		}
+	}
+
+	QFileInfoList dirList = dir.entryInfoList(QDir::AllDirs | QDir::NoDotAndDotDot | QDir::Hidden | QDir::NoSymLinks);
+	foreach (QFileInfo dirInfo, dirList) {
+		QDir dir(dirInfo.filePath());
+		runExampleService(dir);
+	}
+}
+
+void FApplication::cleanFzzs() {
+	QHash<QString, LockedFile *> lockedFiles;
+	QString folder;
+	LockManager::initLockedFiles("fzz", folder, lockedFiles, LockManager::SlowTime);
+	QFileInfoList backupList;
+	LockManager::checkLockedFiles("fzz", backupList, lockedFiles, true, LockManager::SlowTime);
+	LockManager::releaseLockedFiles(folder, lockedFiles);
+}
