@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5308 $:
+$Revision: 5948 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-07-30 21:09:56 +0200 (Sat, 30 Jul 2011) $
+$Date: 2012-04-07 11:02:13 +0200 (Sat, 07 Apr 2012) $
 
 ********************************************************************/
 
@@ -35,6 +35,7 @@ $Date: 2011-07-30 21:09:56 +0200 (Sat, 30 Jul 2011) $
 #include <QPainter>
 #include <QCoreApplication>
 #include <QGraphicsSvgItem>
+#include <qnumeric.h>
 
 QString FSvgRenderer::NonConnectorName("nonconn");
 
@@ -106,24 +107,12 @@ QByteArray FSvgRenderer::loadSvg(const QString & filename, const QStringList & c
 
 	return loadAux(contents, filename, connectorIDs, terminalIDs, legIDs, setColor, colorElementID, findNonConnectors);
 
-	/*
-
-	QXmlStreamReader xml(contents);
-	determineDefaultSize(xml);
-
-	if (readConnectors) {
-		file.seek(0);
-		m_svgXml = file.readAll();
-	}
-	file.close();
-
-	bool result = QSvgRenderer::load(filename);
-	if (result) {
-		m_filename = filename;
-	}
-	return result;
-	*/
+}
 
+bool FSvgRenderer::loadSvgString(const QString & svg) {
+	QByteArray byteArray(svg.toUtf8());
+	QByteArray result = loadSvg(byteArray, "");
+	return !result.isEmpty();
 }
 
 QByteArray FSvgRenderer::loadSvg(const QByteArray & contents, const QString & filename) {
@@ -147,7 +136,11 @@ QByteArray FSvgRenderer::loadAux(const QByteArray & contents, const QString & fi
 			// if svg has both Illustrator and Inkscape crap then converting back and forth between strings and QDomDocument
 			// in FixPixelDimensionsIn() can result in invalid xml
 			TextUtils::cleanSodipodi(string);
-			DebugDialog::debug("Illustrator and inkscape:" + filename);
+#ifndef QT_NO_DEBUG
+			if (!filename.contains("icon", Qt::CaseInsensitive)) {
+				DebugDialog::debug("Illustrator and inkscape:" + filename);
+			}
+#endif
 		}
 
 		//DebugDialog::debug("Illustrator " + filename);
@@ -196,7 +189,8 @@ QByteArray FSvgRenderer::loadAux(const QByteArray & contents, const QString & fi
 			}
 		}
 		if (connectorIDs.count() > 0) {
-			resetContents = resetContents || initConnectorInfo(doc, connectorIDs, terminalIDs, legIDs);
+			bool init =  initConnectorInfo(doc, connectorIDs, terminalIDs, legIDs);
+			resetContents = resetContents || init;
 		}
 		if (findNonConnectors) {
 			initNonConnectorInfo(doc);
@@ -223,21 +217,12 @@ QByteArray FSvgRenderer::loadAux(const QByteArray & contents, const QString & fi
 	//DebugDialog::debug(cleanContents.data());
 
 	QXmlStreamReader xml(cleanContents);
-	determineDefaultSize(xml);
-
-	/*
-	QString path = QCoreApplication::applicationDirPath();
-	path += "/../boom.svg";
-	QFile file(path);
-	if (file.open(QIODevice::WriteOnly | QIODevice::Text)) {
-		QTextStream out(&file);
-		out << QString(cleanContents);
-		file.close();
+	bool result = determineDefaultSize(xml);
+	if (!result) {
+		return QByteArray();
 	}
-	*/
 
-
-	bool result = QSvgRenderer::load(cleanContents);
+	result = QSvgRenderer::load(cleanContents);
 	if (result) {
 		m_filename = filename;
 		return cleanContents;
@@ -307,10 +292,12 @@ void FSvgRenderer::set(const QString & moduleID, ViewLayer::ViewLayerID viewLaye
 	rendererHash->insert(viewLayerID, renderer);
 }
 
-void FSvgRenderer::determineDefaultSize(QXmlStreamReader & xml)
+bool FSvgRenderer::determineDefaultSize(QXmlStreamReader & xml)
 {
 	QSizeF size = parseForWidthAndHeight(xml);
+
 	m_defaultSizeF = QSizeF(size.width() * m_printerScale, size.height() * m_printerScale);
+	return (size.width() != 0 && size.height() != 0);
 }
 
 QSizeF FSvgRenderer::parseForWidthAndHeight(QXmlStreamReader & xml)
@@ -320,33 +307,47 @@ QSizeF FSvgRenderer::parseForWidthAndHeight(QXmlStreamReader & xml)
 	QSizeF size(0,0);
 
 	bool isIllustrator = false;
+	bool bad = false;
 
-	while (!xml.atEnd()) {
+	while (!xml.atEnd() && !bad) {
         switch (xml.readNext()) {
 		case QXmlStreamReader::Comment:
-			isIllustrator = TextUtils::isIllustratorFile(xml.text().toString());
+			if (!isIllustrator) {
+				isIllustrator = TextUtils::isIllustratorFile(xml.text().toString());
+			}
 			break;
         case QXmlStreamReader::StartElement:
 			if (xml.name().toString().compare("svg") == 0) {
 				QString ws = xml.attributes().value("width").toString();
 				QString hs = xml.attributes().value("height").toString();
-				bool ok;
-				double w = TextUtils::convertToInches(ws, &ok, isIllustrator);
-				if (!ok) return size;
-
-				double h = TextUtils::convertToInches(hs, &ok, isIllustrator);
-				if (!ok) return size;
+				bool okw, okh;
+				double w = TextUtils::convertToInches(ws, &okw, isIllustrator);
+				double h = TextUtils::convertToInches(hs, &okh, isIllustrator);
+				if (!okw || qIsNaN(w) || qIsInf(w) || !okh || qIsNaN(h) || qIsInf(h)) {
+					bad = true;
+					break;
+				}
 
 				size.setWidth(w);
 				size.setHeight(h);
 				return size;
 			}
-			return size;
+			break;		
 		default:
-			break;
+			break;		
 		}
 	}
 
+	QIODevice * device = xml.device();
+	DebugDialog::debug("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
+	DebugDialog::debug("bad width and/or bad height in svg:");
+	if (device) {
+		device->reset();
+		QString string(device->readAll());
+		DebugDialog::debug(string);
+	}
+	DebugDialog::debug("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
+
 	return size;
 }
 
@@ -368,22 +369,6 @@ void FSvgRenderer::calcPrinterScale() {
 	VanillaConnectorInfo.gotCircle = false;				
 	m_printerScale = 90.0;
 
-	/*
-
-	QString filename(":/resources/parts/svg/core/breadboard/wire.svg");
-	
-	QGraphicsSvgItem item(filename);
-	QRectF b = item.boundingRect();
-	QFile file(filename);
-	file.open(QFile::ReadOnly);
-	QXmlStreamReader xml(&file);
-	QSizeF size = parseForWidthAndHeight(xml);
-	if (size.width() <= 0) return;
-
-	double pscale = b.width() / size.width();
-	DebugDialog::debug(QString("printerscale %1").arg(pscale));
-
-	*/
 }
 
 double FSvgRenderer::printerScale() {
@@ -401,7 +386,7 @@ void FSvgRenderer::initNonConnectorInfoAux(QDomElement & element)
 {
 	QString id = element.attribute("id");
 	if (id.startsWith(NonConnectorName, Qt::CaseInsensitive)) {
-		ConnectorInfo * connectorInfo = initConnectorInfo(element);
+		ConnectorInfo * connectorInfo = initConnectorInfoStruct(element);
 		m_nonConnectorInfoHash.insert(id, connectorInfo);
 	}
 	QDomElement child = element.firstChildElement();
@@ -433,10 +418,19 @@ void FSvgRenderer::initLegInfoAux(QDomElement & element, const QStringList & con
 	if (!id.isEmpty()) {
 		int ix = legIDs.indexOf(id);
 		if (ix >= 0) {
+			//DebugDialog::debug("init leg info " + id);
+			//foreach (QString lid, legIDs) {
+			//	DebugDialog::debug("\tleg id:" + lid);
+			//}
+
 			element.setTagName("g");			// don't want this element to actually be drawn
 			gotOne = true;
 			ConnectorInfo * connectorInfo = m_connectorInfoHash.value(connectorIDs.at(ix), NULL);
 			if (connectorInfo) {
+				//QString temp;
+				//QTextStream stream(&temp);
+				//element.save(stream, 0);
+				//DebugDialog::debug("\t matched " + connectorIDs.at(ix) + " " + temp);
 				connectorInfo->legMatrix = TextUtils::elementToMatrix(element);
 				connectorInfo->legColor = element.attribute("stroke");
 				connectorInfo->legLine = QLineF();
@@ -503,7 +497,7 @@ void FSvgRenderer::initConnectorInfoAux(QDomElement & element, const QStringList
 	QString id = element.attribute("id");
 	if (!id.isEmpty()) {
 		if (connectorIDs.contains(id)) {
-			ConnectorInfo * connectorInfo = initConnectorInfo(element);
+			ConnectorInfo * connectorInfo = initConnectorInfoStruct(element);
 			m_connectorInfoHash.insert(id, connectorInfo);
 		}
 		// don't return here, might miss other connectors
@@ -516,7 +510,7 @@ void FSvgRenderer::initConnectorInfoAux(QDomElement & element, const QStringList
 	}
 }
 
-ConnectorInfo * FSvgRenderer::initConnectorInfo(QDomElement & connectorElement) {
+ConnectorInfo * FSvgRenderer::initConnectorInfoStruct(QDomElement & connectorElement) {
 	ConnectorInfo * connectorInfo = new ConnectorInfo();
 	connectorInfo->radius = connectorInfo->strokeWidth = 0;
 	connectorInfo->gotCircle = false;
@@ -524,38 +518,34 @@ ConnectorInfo * FSvgRenderer::initConnectorInfo(QDomElement & connectorElement)
 	if (connectorElement.isNull()) return connectorInfo;
 
 	connectorInfo->matrix = TextUtils::elementToMatrix(connectorElement);
-	QList<QDomElement> stack;
-	stack.append(connectorElement);
-	initConnectorInfoAux(stack, connectorInfo);
+	initConnectorInfoStructAux(connectorElement, connectorInfo);
 	return connectorInfo;
 }
 
-bool FSvgRenderer::initConnectorInfoAux(QList<QDomElement> & connectorElements, ConnectorInfo * connectorInfo) 
+bool FSvgRenderer::initConnectorInfoStructAux(QDomElement & element, ConnectorInfo * connectorInfo) 
 {
-	if (connectorElements.isEmpty()) return false;
-
-	QDomElement connectorElement = connectorElements.takeFirst();
 	// right now we only handle circles
-	if (connectorElement.nodeName().compare("circle") != 0) {
-		QDomElement element = connectorElement.firstChildElement();
-		while (!element.isNull()) {
-			connectorElements.append(element);
-			element = element.nextSiblingElement();
+	if (element.nodeName().compare("circle") != 0) {
+		QDomElement child = element.firstChildElement();
+		while (!child.isNull()) {
+			if (initConnectorInfoStructAux(child, connectorInfo)) return true;
+
+			child = child.nextSiblingElement();
 		}
-		return initConnectorInfoAux(connectorElements, connectorInfo);
+		return false;
 	}
 
 	bool ok;
-	double cx = connectorElement.attribute("cx").toDouble(&ok);
+	element.attribute("cx").toDouble(&ok);
 	if (!ok) return false;
 
-	double cy = connectorElement.attribute("cy").toDouble(&ok);
+	element.attribute("cy").toDouble(&ok);
 	if (!ok) return false;
 
-	double r = connectorElement.attribute("r").toDouble(&ok);
+	double r = element.attribute("r").toDouble(&ok);
 	if (!ok) return false;
 
-	double sw = connectorElement.attribute("stroke-width").toDouble(&ok);	
+	double sw = element.attribute("stroke-width").toDouble(&ok);	
 	if (!ok) {
 		//QString strokewidth("stroke-width");
 		//QString s = element.attribute("style");
@@ -566,8 +556,19 @@ bool FSvgRenderer::initConnectorInfoAux(QList<QDomElement> & connectorElements,
 		//}
 	}
 
+	QMatrix matrix = TextUtils::elementToMatrix(element);
+	if (!matrix.isIdentity()) {
+		QRectF r1(0,0,r,r);
+		QRectF r2 = matrix.mapRect(r1);
+		if (r2.width() != r1.width()) {
+			r = r2.width();
+			sw = sw * r2.width() / r1.width();
+		}
+	}
+
+	//DebugDialog::debug("got a circle");
 	connectorInfo->gotCircle = true;
-	connectorInfo->bounds.setRect(cx - r - (sw / 2.0), cy - r - (sw / 2.0), (r * 2) + sw, (r * 2) + sw);
+	//connectorInfo->cbounds.setRect(cx - r - (sw / 2.0), cy - r - (sw / 2.0), (r * 2) + sw, (r * 2) + sw);
 	connectorInfo->radius = r;
 	connectorInfo->strokeWidth = sw;
 	return true;
@@ -603,7 +604,10 @@ bool FSvgRenderer::setUpConnector(SvgIdLayer * svgIdLayer, bool ignoreTerminalPo
 
 	QString connectorID = svgIdLayer->m_svgId;
 
+	// boundsOnElement seems to include any matrix on the element itself.
+	// I would swear this wasn't true before Qt4.7, but maybe I am crazy
 	QRectF bounds = this->boundsOnElement(connectorID);	
+
 	if (bounds.isNull() && !svgIdLayer->m_hybrid) {		// hybrids can have zero size
 		svgIdLayer->m_svgVisible = false;		
 		DebugDialog::debug("renderer::setupconnector: null bounds");
@@ -613,16 +617,27 @@ bool FSvgRenderer::setUpConnector(SvgIdLayer * svgIdLayer, bool ignoreTerminalPo
 	QSizeF defaultSizeF = this->defaultSizeF();
 	QRectF viewBox = this->viewBoxF();
 
-	ConnectorInfo * connectorInfo = getConnectorInfo(connectorID);		
+	ConnectorInfo * connectorInfo = getConnectorInfo(connectorID);	
+
+	/*
+	DebugDialog::debug(QString("connectorid:%1 m11:%2 m12:%3 m21:%4 m22:%5 dx:%6 dy:%7")
+						.arg(connectorID)
+						.arg(connectorInfo->matrix.m11())
+						.arg(connectorInfo->matrix.m12())
+						.arg(connectorInfo->matrix.m21())
+						.arg(connectorInfo->matrix.m22())
+						.arg(connectorInfo->matrix.dx())
+						.arg(connectorInfo->matrix.dy()),
+				bounds);
+	*/
+
 	if (connectorInfo && connectorInfo->gotCircle) {
 		svgIdLayer->m_radius = connectorInfo->radius * defaultSizeF.width() / viewBox.width();
 		svgIdLayer->m_strokeWidth = connectorInfo->strokeWidth * defaultSizeF.width() / viewBox.width();
-		bounds = connectorInfo->bounds;
+		//bounds = connectorInfo->cbounds;
 	}
 
-	// matrixForElement only grabs parent matrices, not any transforms in the element itself
-	QMatrix matrix0 = connectorInfo->matrix * this->matrixForElement(connectorID);  
-
+	
 	/*DebugDialog::debug(QString("identity matrix %11 %1 %2, viewbox: %3 %4 %5 %6, bounds: %7 %8 %9 %10, size: %12 %13").arg(m_modelPart->title()).arg(connectorSharedID())
 					   .arg(viewBox.x()).arg(viewBox.y()).arg(viewBox.width()).arg(viewBox.height())
 					   .arg(bounds.x()).arg(bounds.y()).arg(bounds.width()).arg(bounds.height())
@@ -633,7 +648,10 @@ bool FSvgRenderer::setUpConnector(SvgIdLayer * svgIdLayer, bool ignoreTerminalPo
 
 	// some strangeness in the way that svg items and non-svg items map to screen space
 	// might be a qt problem.
-	QRectF r1 = matrix0.mapRect(bounds);
+	//QMatrix matrix0 = connectorInfo->matrix * this->matrixForElement(connectorID);  
+	//QRectF r1 = matrix0.mapRect(bounds);
+	QRectF r1 = this->matrixForElement(connectorID).mapRect(bounds);
+
 	/*
 	svgIdLayer->m_rect.setRect(r1.x() * defaultSize.width() / viewBox.width(), 
 							   r1.y() * defaultSize.height() / viewBox.height(), 
@@ -664,6 +682,15 @@ void FSvgRenderer::calcLeg(SvgIdLayer * svgIdLayer, const QRectF & viewBox, Conn
 	QSizeF defaultSizeF = this->defaultSizeF();
 	svgIdLayer->m_legStrokeWidth = connectorInfo->legStrokeWidth * defaultSizeF.width() / viewBox.width();
 
+	/*
+	DebugDialog::debug(	QString("calcleg leg %1 %2 %3 %4")
+		.arg(connectorInfo->legLine.p1().x())
+		.arg(connectorInfo->legLine.p1().y())
+		.arg(connectorInfo->legLine.p2().x())
+		.arg(connectorInfo->legLine.p2().y())
+		);
+	*/
+
 	QMatrix matrix = this->matrixForElement(svgIdLayer->m_legId) * connectorInfo->legMatrix;
 	QPointF p1 = matrix.map(connectorInfo->legLine.p1());
 	QPointF p2 = matrix.map(connectorInfo->legLine.p2());
@@ -687,6 +714,7 @@ void FSvgRenderer::calcLeg(SvgIdLayer * svgIdLayer, const QRectF & viewBox, Conn
 
 QPointF FSvgRenderer::calcTerminalPoint(const QString & terminalId, const QRectF & connectorRect, bool ignoreTerminalPoint, const QRectF & viewBox, QMatrix & terminalMatrix)
 {
+	Q_UNUSED(terminalMatrix);
 	QPointF terminalPoint = connectorRect.center() - connectorRect.topLeft();    // default spot is centered
 	if (ignoreTerminalPoint) {
 		return terminalPoint;
@@ -713,8 +741,9 @@ QPointF FSvgRenderer::calcTerminalPoint(const QString & terminalId, const QRectF
 
 
 	// matrixForElement only grabs parent matrices, not any transforms in the element itself
-	QMatrix tMatrix = this->matrixForElement(terminalId) * terminalMatrix;
-	QRectF terminalRect = tMatrix.mapRect(tBounds);
+	//QMatrix tMatrix = this->matrixForElement(terminalId) ; //* terminalMatrix;
+	//QRectF terminalRect = tMatrix.mapRect(tBounds);
+	QRectF terminalRect = this->matrixForElement(terminalId).mapRect(tBounds);
 	QPointF c = terminalRect.center();
 	QPointF q(c.x() * defaultSizeF.width() / viewBox.width(), c.y() * defaultSizeF.height() / viewBox.height());
 	terminalPoint = q - connectorRect.topLeft();
@@ -753,14 +782,13 @@ QList<SvgIdLayer *> FSvgRenderer::setUpNonConnectors() {
 		if (connectorInfo && connectorInfo->gotCircle) {
 			svgIdLayer->m_radius = connectorInfo->radius * defaultSizeF.width() / viewBox.width();
 			svgIdLayer->m_strokeWidth = connectorInfo->strokeWidth * defaultSizeF.width() / viewBox.width();
-			bounds = connectorInfo->bounds;
+			//bounds = connectorInfo->cbounds;
 		}
 
 		// matrixForElement only grabs parent matrices, not any transforms in the element itself
-		QMatrix matrix0 = connectorInfo->matrix * this->matrixForElement(nonConnectorID);  
-
-
-		QRectF r1 = matrix0.mapRect(bounds);
+		//QMatrix matrix0 = connectorInfo->matrix * this->matrixForElement(nonConnectorID);  
+		//QRectF r1 = matrix0.mapRect(bounds);
+		QRectF r1 = this->matrixForElement(nonConnectorID).mapRect(bounds);
 		svgIdLayer->m_rect.setRect(r1.x() * defaultSize.width() / viewBox.width(), r1.y() * defaultSize.height() / viewBox.height(), r1.width() * defaultSize.width() / viewBox.width(), r1.height() * defaultSize.height() / viewBox.height());
 		svgIdLayer->m_point = svgIdLayer->m_rect.center() - svgIdLayer->m_rect.topLeft();
 		svgIdLayer->m_svgVisible = true;
@@ -771,3 +799,5 @@ QList<SvgIdLayer *> FSvgRenderer::setUpNonConnectors() {
 	return list;
 
 }
+
+
