@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5375 $:
+$Revision: 5782 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-08-08 21:38:25 +0200 (Mon, 08 Aug 2011) $
+$Date: 2012-01-26 06:04:40 +0100 (Thu, 26 Jan 2012) $
 
 ********************************************************************/
 
@@ -116,6 +116,7 @@ HtmlInfoView::HtmlInfoView(QWidget * parent) : QScrollArea(parent)
 	m_partTitle = NULL;
 	m_partUrl = NULL;
 	m_partVersion = NULL;
+	m_lockCheckbox = NULL;
 	m_connDescr = NULL;
 	m_tagsTextLabel = NULL;
 	m_lastSwappingEnabled = false;
@@ -133,6 +134,8 @@ HtmlInfoView::HtmlInfoView(QWidget * parent) : QScrollArea(parent)
 
 	m_titleEdit = new FLineEdit(mainFrame);
 	m_titleEdit->setObjectName("instanceTitleEditor");
+	m_titleEdit->setToolTip(tr("Change the part label here"));
+	m_titleEdit->setAlignment(Qt::AlignLeft);
 
 	connect(m_titleEdit, SIGNAL(editingFinished()), this, SLOT(setInstanceTitle()));
 	connect(m_titleEdit, SIGNAL(mouseEnter()), this, SLOT(instanceTitleEnter()));
@@ -156,12 +159,29 @@ HtmlInfoView::HtmlInfoView(QWidget * parent) : QScrollArea(parent)
 	hboxLayout->setContentsMargins(0, 0, 0, 0);
 	hboxLayout->addSpacing(IconSpace);
 	m_icon1 = addLabel(hboxLayout, NoIcon);
+	m_icon1->setToolTip(tr("Part breadboard view image"));
 	m_icon2 = addLabel(hboxLayout, NoIcon);
+	m_icon2->setToolTip(tr("Part schematic view image"));
 	m_icon3 = addLabel(hboxLayout, NoIcon);
+	m_icon3->setToolTip(tr("Part pcb view image"));
 
+	QVBoxLayout * versionLayout = new QVBoxLayout();
+
+	QHBoxLayout * subVersionLayout = new QHBoxLayout();
 	m_partVersion = new QLabel();
-	m_partVersion->setObjectName("infoViewPartTitle");
-	hboxLayout->addWidget(m_partVersion);
+	m_partVersion->setObjectName("infoViewPartVersion");
+	m_partVersion->setToolTip(tr("Part version number"));
+	subVersionLayout->addWidget(m_partVersion, 0, Qt::AlignLeft);
+	subVersionLayout->addStretch(1);
+	versionLayout->addLayout(subVersionLayout);
+	
+	m_lockCheckbox = new QCheckBox(tr("Locked"));
+	m_lockCheckbox->setObjectName("infoViewLockCheckbox");
+	m_lockCheckbox->setToolTip(tr("Change the locked state of the part in this view.  A locked part can't be moved"));
+	connect(m_lockCheckbox, SIGNAL(clicked(bool)), this, SLOT(changeLock(bool)));
+	versionLayout->addWidget(m_lockCheckbox);
+
+	hboxLayout->addLayout(versionLayout);
 	
 	hboxLayout->addSpacerItem(new QSpacerItem(IconSpace, 1, QSizePolicy::Expanding));
 	iconFrame->setLayout(hboxLayout);
@@ -292,7 +312,7 @@ void HtmlInfoView::hoverLeaveItem(InfoGraphicsView * infoGraphicsView, QGraphics
 	m_setContentTimer.start();
 }
 
-void HtmlInfoView::viewConnectorItemInfo(ConnectorItem * connectorItem) {
+void HtmlInfoView::viewConnectorItemInfo(QGraphicsSceneHoverEvent *, ConnectorItem * connectorItem) {
 
 	int count = connectorItem ? connectorItem->connectionsCount() : 0;
 	if (m_lastConnectorItem == connectorItem && m_lastConnectorItemCount == count) return;
@@ -301,20 +321,17 @@ void HtmlInfoView::viewConnectorItemInfo(ConnectorItem * connectorItem) {
 	m_lastConnectorItemCount = count;
 
 	Connector * connector = NULL;
-	ConnectorShared * connectorShared = NULL;
 	if (connectorItem) {
 		if (connectorItem->attachedTo() != m_lastItemBase) {
 			return;
 		}
 
 		connector = connectorItem->connector();
-		connectorShared = connector->connectorShared();
-	
 	}
 
 	if (m_connDescr) {
 		m_connDescr->setText(connectorItem ? tr("connected to %n item(s)", "", connectorItem->connectionsCount()) : "");
-		m_connName->setText(connectorShared ? connectorShared->name() : "");
+		m_connName->setText(connector ? connector->connectorSharedName() : "");
 		m_connType->setText(connector ? Connector::connectorNameFromType(connector->connectorType()) : "");
 	}
 
@@ -324,14 +341,14 @@ void HtmlInfoView::hoverEnterConnectorItem(InfoGraphicsView *igv, QGraphicsScene
 	Q_UNUSED(event)
 	Q_UNUSED(swappingEnabled)
 	Q_UNUSED(igv)
-	viewConnectorItemInfo(item);
+	viewConnectorItemInfo(event, item);
 }
 
 void HtmlInfoView::hoverLeaveConnectorItem(InfoGraphicsView *igv, QGraphicsSceneHoverEvent *event, ConnectorItem *connItem) {
 	Q_UNUSED(event);
 	Q_UNUSED(connItem);
 	Q_UNUSED(igv);
-	viewConnectorItemInfo(NULL);
+	viewConnectorItemInfo(event, NULL);
 }
 
 void HtmlInfoView::appendStuff(ItemBase* item, bool swappingEnabled) {
@@ -354,7 +371,7 @@ void HtmlInfoView::appendWireStuff(Wire* wire, bool swappingEnabled) {
 	QString nameString = tr("Wire");
 	if (swappingEnabled) {
 		if (wire->getRatsnest()) {
-			nameString = tr("Rat's nest wire");
+			nameString = tr("Ratsnest wire");
 		} 
 		else if(wire->getTrace()) {
 			nameString = tr("Trace wire %1").arg(autoroutable);
@@ -364,6 +381,7 @@ void HtmlInfoView::appendWireStuff(Wire* wire, bool swappingEnabled) {
 		 nameString = modelPart->description();
 	}
 	partTitle(nameString, modelPart->version(), modelPart->url());
+	m_lockCheckbox->setVisible(false);
 
 	setUpTitle(wire);
 	setUpIcons(wire->modelPart());
@@ -395,6 +413,8 @@ void HtmlInfoView::appendItemStuff(ItemBase * itemBase, ModelPart * modelPart, b
 		nameString = modelPart->description();
 	}
 	partTitle(nameString, modelPart->version(), modelPart->url());
+	m_lockCheckbox->setVisible(true);
+	m_lockCheckbox->setChecked(itemBase->moveLock());
 
 	displayProps(modelPart, itemBase, swappingEnabled);
 	addTags(modelPart);
@@ -466,10 +486,11 @@ void HtmlInfoView::setNullContent()
 {
 	setUpTitle(NULL);
 	partTitle("", "", "");
+	m_lockCheckbox->setVisible(false);
 	setUpIcons(NULL);
 	displayProps(NULL, NULL, false);
 	addTags(NULL);
-	viewConnectorItemInfo(NULL);
+	viewConnectorItemInfo(NULL, NULL);
 	m_connFrame->setVisible(false);
 	m_propFrame->setVisible(false);
 	m_proplabel->setVisible(false);
@@ -539,6 +560,8 @@ void HtmlInfoView::setUpTitle(ItemBase * itemBase)
 		m_titleEdit->setEnabled(false);
 		m_titleEdit->setText("");
 	}
+	// helps keep it left aligned?
+	m_titleEdit->setCursorPosition(0);
 
 }
 
@@ -637,7 +660,7 @@ void HtmlInfoView::displayProps(ModelPart * modelPart, ItemBase * itemBase, bool
 {
 	bool repeatPossible = (modelPart == m_lastPropsModelPart && itemBase == m_lastPropsItemBase && swappingEnabled == m_lastPropsSwappingEnabled);
 	if (repeatPossible && modelPart == NULL && itemBase == NULL) {
-		DebugDialog::debug("display props bail");
+		//DebugDialog::debug("display props bail");
 		return;
 	}
 
@@ -882,3 +905,11 @@ QHash<QString, QString> HtmlInfoView::getPartProperties(ModelPart * modelPart, I
 
 	return properties;
 }
+
+void HtmlInfoView::changeLock(bool lockState)
+{
+	if (m_currentItem == NULL) return;
+	if (m_currentItem->itemType() == ModelPart::Wire) return;
+
+	m_currentItem->setMoveLock(lockState);
+}
