@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5375 $:
+$Revision: 5944 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-08-08 21:38:25 +0200 (Mon, 08 Aug 2011) $
+$Date: 2012-04-06 16:06:09 +0200 (Fri, 06 Apr 2012) $
 
 ********************************************************************/
 
@@ -49,9 +49,20 @@ $Date: 2011-08-08 21:38:25 +0200 (Mon, 08 Aug 2011) $
 
 static const int rowHeight = 21;
 
+const double Hole::OffsetPixels = 2;
 QHash<QString, QString> Hole::HoleSizes;
 const QString Hole::AutorouteViaHoleSize = "autorouteViaHoleSize";
 const QString Hole::AutorouteViaRingThickness = "autorouteViaRingThickness";
+QString Hole::DefaultAutorouteViaHoleSize;
+QString Hole::DefaultAutorouteViaRingThickness;
+
+
+QString HoleSettings::currentUnits() {
+	if (mmRadioButton->isChecked()) return QObject::tr("mm");
+	return QObject::tr("in");
+}
+
+//////////////////////////////////////////////////
 
 Hole::Hole( ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdentifier, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel)
 	: PaletteItem(modelPart, viewIdentifier, viewGeometry, id, itemMenu, doLabel)
@@ -76,7 +87,7 @@ Hole::Hole( ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdent
 	m_holeSettings.holeDiameter = sizes.at(0);
 	m_holeSettings.ringThickness = sizes.at(1);
 
-	m_otherLayerRenderer = m_renderer = NULL;
+	m_otherLayerRenderer = m_extraRenderer = NULL;
 }
 
 Hole::~Hole() {
@@ -86,7 +97,8 @@ void Hole::initHoleSettings(HoleSettings & holeSettings)
 {
 	holeSettings.diameterEdit = holeSettings.thicknessEdit = NULL;
 	holeSettings.diameterValidator = holeSettings.thicknessValidator = NULL;
-	holeSettings.unitsComboBox = holeSettings.sizesComboBox = NULL;
+	holeSettings.inRadioButton = holeSettings.mmRadioButton = NULL;
+	holeSettings.sizesComboBox = NULL;
 	holeSettings.holeDiameterRange = Hole::holeDiameterRange;
 	holeSettings.ringThicknessRange = Hole::ringThicknessRange;
 }
@@ -119,8 +131,14 @@ void Hole::setUpHoleSizes() {
 		QString hs = ve.attribute("holesize");
 		QString name = ve.attribute("name");
 		if (ve.attribute("default").compare("yes") == 0) {
-			if (ringThickness.isEmpty()) settings.setValue(AutorouteViaRingThickness, rt);
-			if (holeSize.isEmpty()) settings.setValue(AutorouteViaHoleSize, hs);
+			if (ringThickness.isEmpty()) {
+				settings.setValue(AutorouteViaRingThickness, rt);
+				DefaultAutorouteViaRingThickness = rt;
+			}
+			if (holeSize.isEmpty()) {
+				settings.setValue(AutorouteViaHoleSize, hs);
+				DefaultAutorouteViaHoleSize = hs;
+			}
 		}
 		HoleSizes.insert(name, QString("%1,%2").arg(hs).arg(rt));
 		ve = ve.nextSiblingElement("via");
@@ -145,27 +163,27 @@ QString Hole::holeSize(HoleSettings & holeSettings) {
 	return QString("%1,%2").arg(holeSettings.holeDiameter).arg(holeSettings.ringThickness);
 }
 
-void Hole::setHoleSize(QString holeSize, bool force) {
-	if (setHoleSize(holeSize, force, m_holeSettings)) {
+void Hole::setHoleSize(QString newSize, bool force) {
+	//DebugDialog::debug(QString("old holesize %1").arg(viewIdentifier()) + holeSize(), sceneBoundingRect());
+	//foreach (QGraphicsItem * childItem, childItems()) {
+	//	DebugDialog::debug(QString("   child"), childItem->sceneBoundingRect());
+	//}
+
+	if (setHoleSize(newSize, force, m_holeSettings)) {
 		setBoth(m_holeSettings.holeDiameter, m_holeSettings.ringThickness);
-		modelPart()->setProp("hole size", holeSize);
+		modelPart()->setProp("hole size", newSize);
 
-		updateTooltip();
 		if (m_partLabel) m_partLabel->displayTextsIf();	
 	}
+	//DebugDialog::debug(QString("new holesize %1 ").arg(viewIdentifier()) + holeSize(), sceneBoundingRect());
+	//foreach (QGraphicsItem * childItem, childItems()) {
+	//	DebugDialog::debug(QString("   child"), childItem->sceneBoundingRect());
+	//}
 }
 
 bool Hole::setHoleSize(QString & holeSize, bool force, HoleSettings & holeSettings)
 {
-	QString hashedHoleSize = HoleSizes.value(holeSize);
-	QStringList sizes;
-	if (hashedHoleSize.isEmpty()) {
-		sizes = holeSize.split(",");
-	}
-	else {
-		sizes = hashedHoleSize.split(",");
-		holeSize = sizes[0] + "," + sizes[1];
-	}
+	QStringList sizes = getSizes(holeSize);
 	if (sizes.count() < 2) return false;
 
 	if (!force && (holeSettings.holeDiameter.compare(sizes.at(0)) == 0) && (holeSettings.ringThickness.compare(sizes.at(1)) == 0)) 
@@ -181,14 +199,39 @@ bool Hole::setHoleSize(QString & holeSize, bool force, HoleSettings & holeSettin
 	return true;
 }
 
+QRectF Hole::getRect(const QString & newSize) {
+	QString s = newSize;
+	QStringList sizes = getSizes(s);
+	if (sizes.count() < 2) return boundingRect();
+
+	double diameter = TextUtils::convertToInches(sizes.at(0));
+	double ringThickness = TextUtils::convertToInches(sizes.at(1));
+	double dim = (diameter + ringThickness + ringThickness) * FSvgRenderer::printerScale();
+	//DebugDialog::debug(QString("get rect %1 %2").arg(newSize).arg(dim));
+	return QRectF(0, 0, dim, dim);
+}
+
+QStringList Hole::getSizes(QString & holeSize)
+{
+	QStringList sizes;
+	QString hashedHoleSize = HoleSizes.value(holeSize);
+	if (hashedHoleSize.isEmpty()) {
+		sizes = holeSize.split(",");
+	}
+	else {
+		sizes = hashedHoleSize.split(",");
+		holeSize = sizes[0] + "," + sizes[1];
+	}
+	return sizes;
+}
+
 void Hole::setBoth(const QString & holeDiameter, const QString & ringThickness) {
 	if (this->m_viewIdentifier != ViewIdentifierClass::PCBView) return;
 
-	QStringList connectorIDs;
-	ItemBase * otherLayer = setBothSvg(holeDiameter, ringThickness, connectorIDs);
+	ItemBase * otherLayer = setBothSvg(holeDiameter, ringThickness);
 
 	// there's only one NonConnectorItem
-	foreach (SvgIdLayer * svgIdLayer, m_renderer->setUpNonConnectors()) {
+	foreach (SvgIdLayer * svgIdLayer, m_extraRenderer->setUpNonConnectors()) {
 		if (svgIdLayer == NULL) continue;
 
 		setBothNonConnectors(this, svgIdLayer);
@@ -200,19 +243,15 @@ void Hole::setBoth(const QString & holeDiameter, const QString & ringThickness)
 	}
 }
 
-ItemBase * Hole::setBothSvg(const QString & holeDiameter, const QString & ringThickness, const QStringList & connectorIDs) 
+ItemBase * Hole::setBothSvg(const QString & holeDiameter, const QString & ringThickness) 
 {
 	QString svg = makeSvg(holeDiameter, ringThickness, m_viewLayerID);
-	if (m_renderer == NULL) {
-		m_renderer = new FSvgRenderer(this);
-	}
+	loadExtraRenderer(svg.toUtf8(), false);
+	//DebugDialog::debug("both");
+	//DebugDialog::debug(svg);
 
 	QString setColor;
 	QStringList noIDs;
-	QByteArray result = m_renderer->loadSvg(svg.toLatin1(), m_filename, connectorIDs, noIDs,  noIDs, "", "", true);
-	if (!result.isEmpty()) {
-		setSharedRendererEx(m_renderer);
-	}
 
 	QString osvg;
 	ItemBase * otherLayer = NULL;
@@ -229,11 +268,16 @@ ItemBase * Hole::setBothSvg(const QString & holeDiameter, const QString & ringTh
 			m_otherLayerRenderer = new FSvgRenderer(this);
 		}
 
-		result = m_otherLayerRenderer->loadSvg(osvg.toLatin1(), m_filename, noIDs, noIDs,  noIDs, "", "", true);
-		if (!result.isEmpty()) {
+		//DebugDialog::debug(osvg);
+
+
+		bool result = m_otherLayerRenderer->loadSvgString(osvg);
+		if (result) {
 			qobject_cast<PaletteItemBase *>(otherLayer)->setSharedRendererEx(m_otherLayerRenderer);
 		}
 	}
+
+	//DebugDialog::debug("other");
 	
 	return otherLayer;
 }
@@ -243,6 +287,7 @@ void Hole::setBothNonConnectors(ItemBase * itemBase, SvgIdLayer * svgIdLayer) {
 		NonConnectorItem * nonConnectorItem = dynamic_cast<NonConnectorItem *>(child);
 		if (nonConnectorItem == NULL) continue;
 
+		//DebugDialog::debug(QString("hole set rect %1").arg(m_viewIdentifier), svgIdLayer->m_rect);
 		nonConnectorItem->setRect(svgIdLayer->m_rect);
 		nonConnectorItem->setRadius(svgIdLayer->m_radius, svgIdLayer->m_strokeWidth);
 		break;
@@ -252,12 +297,17 @@ void Hole::setBothNonConnectors(ItemBase * itemBase, SvgIdLayer * svgIdLayer) {
 
 QString Hole::makeSvg(const QString & holeDiameter, const QString & ringThickness, ViewLayer::ViewLayerID viewLayerID) 
 {
-	double hd = TextUtils::convertToInches(holeDiameter) * GraphicsUtils::StandardFritzingDPI;
-	double rt = TextUtils::convertToInches(ringThickness) * GraphicsUtils::StandardFritzingDPI;
+	double offsetDPI = OffsetPixels / FSvgRenderer::printerScale();
+	double hd = TextUtils::convertToInches(holeDiameter);
+	double rt = TextUtils::convertToInches(ringThickness);
 
-	double wInches = (hd + rt + rt) / GraphicsUtils::StandardFritzingDPI;
+	double wInches = hd + rt + rt + offsetDPI + offsetDPI;
 	QString svg = TextUtils::makeSVGHeader(1, GraphicsUtils::StandardFritzingDPI, wInches, wInches);
 
+	hd *= GraphicsUtils::StandardFritzingDPI;
+	rt *= GraphicsUtils::StandardFritzingDPI;
+	offsetDPI *= GraphicsUtils::StandardFritzingDPI;
+
 	QString setColor;
 	if (viewLayerID == ViewLayer::Copper0) {
 		setColor = ViewLayer::Copper0Color;
@@ -270,24 +320,26 @@ QString Hole::makeSvg(const QString & holeDiameter, const QString & ringThicknes
 	
 	QString id = makeID();
 	if (hd == 0) {
-		svg += QString("<circle cx='%1' cy='%1' r='%1' fill='%2' id='%3' />")
+		svg += QString("<circle cx='%1' cy='%1' r='%2' fill='%3' id='%4' />")
+					.arg(rt + offsetDPI)
 					.arg(rt)
 					.arg(setColor)
 					.arg(id);
 	}
 	else {
 		svg += QString("<circle fill='none' cx='%1' cy='%1' r='%2' stroke-width='%3' stroke='%4' id='%5' />")
-			.arg((hd / 2) + rt)
+			.arg((hd / 2) + rt + offsetDPI)
 			.arg((hd / 2) + (rt / 2))
 			.arg(rt)
 			.arg(setColor)
 			.arg(id);
 		svg += QString("<circle drill='0' fill='black' cx='%1' cy='%1' r='%2' stroke-width='0'  />")   // set the drill attribute to 0 for gerber translation
-			.arg((hd / 2) + rt)
+			.arg((hd / 2) + rt + offsetDPI)
 			.arg(hd / 2);
 	}
   		
 	svg += "</g></svg>";
+	//DebugDialog::debug("hole svg " + svg);
 	return svg;
 }
 
@@ -362,8 +414,9 @@ bool Hole::collectExtraInfo(QWidget * parent, const QString & family, const QStr
 		returnValue = m_modelPart->prop("hole size").toString();
 		QWidget * frame = createHoleSettings(parent, m_holeSettings, swappingEnabled, returnValue);
 
-		connect(m_holeSettings.sizesComboBox, SIGNAL(currentIndexChanged(const QString &)), this, SLOT(changeHoleSize(const QString &)));
-		connect(m_holeSettings.unitsComboBox, SIGNAL(currentIndexChanged(const QString &)), this, SLOT(changeUnits(const QString &)));
+		connect(m_holeSettings.sizesComboBox, SIGNAL(currentIndexChanged(const QString &)), this, SLOT(changeHoleSize(const QString &)));	
+		connect(m_holeSettings.mmRadioButton, SIGNAL(toggled(bool)), this, SLOT(changeUnits(bool)));
+		connect(m_holeSettings.inRadioButton, SIGNAL(toggled(bool)), this, SLOT(changeUnits(bool)));
 		connect(m_holeSettings.diameterEdit, SIGNAL(editingFinished()), this, SLOT(changeDiameter()));
 		connect(m_holeSettings.thicknessEdit, SIGNAL(editingFinished()), this, SLOT(changeThickness()));
 
@@ -378,7 +431,7 @@ void Hole::changeThickness()
 {
 	if (changeThickness(m_holeSettings, sender())) {
 		QLineEdit * edit = qobject_cast<QLineEdit *>(sender());
-		changeHoleSize(m_holeSettings.holeDiameter + "," + edit->text() + m_holeSettings.unitsComboBox->currentText());
+		changeHoleSize(m_holeSettings.holeDiameter + "," + edit->text() + currentUnits());
 	}	
 }
 
@@ -398,7 +451,7 @@ void Hole::changeDiameter()
 {
 	if (changeDiameter(m_holeSettings, sender())) {
 		QLineEdit * edit = qobject_cast<QLineEdit *>(sender());
-		changeHoleSize(edit->text() + m_holeSettings.unitsComboBox->currentText() + "," + m_holeSettings.ringThickness);
+		changeHoleSize(edit->text() + currentUnits() + "," + m_holeSettings.ringThickness);
 	}
 }
 
@@ -414,9 +467,9 @@ bool Hole::changeDiameter(HoleSettings & holeSettings, QObject * sender)
 	return (newValue != oldValue);
 }
 
-void Hole::changeUnits(const QString & units) 
+void Hole::changeUnits(bool) 
 {
-	QString newVal = changeUnits(units, m_holeSettings);
+	QString newVal = changeUnits(currentUnits(), m_holeSettings);
 	modelPart()->setProp("hole size", newVal);
 }
 
@@ -447,9 +500,9 @@ void Hole::updateValidators(HoleSettings & holeSettings)
 {
 	if (holeSettings.diameterValidator == NULL) return;
 	if (holeSettings.thicknessValidator == NULL) return;
-	if (holeSettings.unitsComboBox == NULL) return;
+	if (holeSettings.mmRadioButton == NULL) return;
 
-	QString units = holeSettings.unitsComboBox->currentText();
+	QString units = holeSettings.currentUnits();
 	QPointF hdRange = holeSettings.holeDiameterRange(holeSettings.ringThickness);
 	QPointF rtRange = holeSettings.ringThicknessRange(holeSettings.holeDiameter);
 
@@ -461,13 +514,13 @@ void Hole::updateValidators(HoleSettings & holeSettings)
 
 QPointF Hole::ringThicknessRange(const QString & holeDiameter) {
 	double hd = TextUtils::convertToInches(holeDiameter);
-	QPointF p(hd > 0 ? 0 : .001, 1.0);
+	QPointF p(hd > 0 ? 0 : .001, 10.0);
 	return p;
 }
 
 QPointF Hole::holeDiameterRange(const QString & ringThickness) {
 	double rt = TextUtils::convertToInches(ringThickness);
-	QPointF p(rt > 0 ? 0 : .001, 1.0);
+	QPointF p(rt > 0 ? 0 : .001, 10.0);
 	return p;
 }
 
@@ -475,20 +528,22 @@ QPointF Hole::holeDiameterRange(const QString & ringThickness) {
 void Hole::changeHoleSize(const QString & newSize) {
 	InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
 	if (infoGraphicsView != NULL) {
-		infoGraphicsView->setProp(this, "hole size", tr("hole size"), this->holeSize(), newSize, true);
+        QRectF holeRect = getRect(holeSize());
+        QRectF newHoleRect = getRect(newSize);
+        infoGraphicsView->setHoleSize(this, "hole size", tr("hole size"), holeSize(), newSize, holeRect, newHoleRect, true);
 	}
 }
 
 void Hole::updateEditTexts(HoleSettings & holeSettings) {
 	if (holeSettings.diameterEdit == NULL) return;
 	if (holeSettings.thicknessEdit == NULL) return;
-	if (holeSettings.unitsComboBox == NULL) return;
+	if (holeSettings.mmRadioButton == NULL) return;
 
 	double hd = TextUtils::convertToInches(holeSettings.holeDiameter);
 	double rt = TextUtils::convertToInches(holeSettings.ringThickness);
 
 	QString newVal;
-	if (holeSettings.unitsComboBox->currentText() == "in") {
+	if (holeSettings.currentUnits() == "in") {
 		newVal = QString("%1,%2").arg(hd).arg(rt);
 	}
 	else {
@@ -573,15 +628,24 @@ QWidget * Hole::createHoleSettings(QWidget * parent, HoleSettings & holeSettings
 	QGridLayout * gridLayout = new QGridLayout(subFrame);
 	gridLayout->setMargin(0);
 
-	holeSettings.unitsComboBox = new QComboBox(subFrame);
-    holeSettings.unitsComboBox->setMaximumWidth(60);
-	holeSettings.unitsComboBox->setMinimumHeight(rowHeight);
-	holeSettings.unitsComboBox->setMaximumHeight(rowHeight);
-	holeSettings.unitsComboBox->setEditable(false);
-	holeSettings.unitsComboBox->addItem("mm");
-	holeSettings.unitsComboBox->addItem("in");
-	gridLayout->addWidget(holeSettings.unitsComboBox, 0, 2, 2, 1);
-	holeSettings.unitsComboBox->setObjectName("infoViewComboBox");
+	QGroupBox * rbFrame = new QGroupBox("", subFrame);
+	rbFrame->setObjectName("infoViewGroupBox");
+	QVBoxLayout * vbl = new QVBoxLayout(rbFrame);
+	vbl->setMargin(0);
+
+	holeSettings.inRadioButton = new QRadioButton(tr("in"), subFrame);
+	gridLayout->addWidget(holeSettings.inRadioButton, 0, 2);
+	holeSettings.inRadioButton->setObjectName("infoViewRadioButton");
+
+	holeSettings.mmRadioButton = new QRadioButton(tr("mm"), subFrame);
+	gridLayout->addWidget(holeSettings.mmRadioButton, 1, 2);
+	holeSettings.inRadioButton->setObjectName("infoViewRadioButton");
+
+	vbl->addWidget(holeSettings.inRadioButton);
+	vbl->addWidget(holeSettings.mmRadioButton);
+	rbFrame->setLayout(vbl);
+
+	gridLayout->addWidget(rbFrame, 0, 2, 2, 1, Qt::AlignVCenter);
 
 	holeSettings.diameterEdit = new QLineEdit(subFrame);
 	holeSettings.diameterEdit->setMinimumHeight(rowHeight);
@@ -619,12 +683,19 @@ QWidget * Hole::createHoleSettings(QWidget * parent, HoleSettings & holeSettings
 
 	holeSettings.sizesComboBox->addItems(HoleSizes.keys());
 	holeSettings.sizesComboBox->setEnabled(swappingEnabled);
-	holeSettings.unitsComboBox->setEnabled(swappingEnabled);
-	holeSettings.unitsComboBox->setCurrentIndex(currentHoleSize.contains("in") ? 1 : 0);
-
+	holeSettings.mmRadioButton->setEnabled(swappingEnabled);
+	holeSettings.inRadioButton->setEnabled(swappingEnabled);
 	holeSettings.diameterEdit->setEnabled(swappingEnabled);
 	holeSettings.thicknessEdit->setEnabled(swappingEnabled);
 
+
+	if (currentHoleSize.contains("mm")) {
+		holeSettings.mmRadioButton->setChecked(true);
+	}
+	else {
+		holeSettings.inRadioButton->setChecked(true);
+	}
+
 	updateEditTexts(holeSettings);
 	updateValidators(holeSettings);
 	updateSizes(holeSettings);
@@ -643,3 +714,8 @@ bool Hole::rotation45Allowed() {
 bool Hole::canFindConnectorsUnder() {
 	return false;
 }
+
+QString Hole::currentUnits() {
+	return m_holeSettings.currentUnits();
+}
+
