@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5410 $:
+$Revision: 5952 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-08-17 23:12:23 +0200 (Wed, 17 Aug 2011) $
+$Date: 2012-04-08 00:47:10 +0200 (Sun, 08 Apr 2012) $
 
 ********************************************************************/
 
@@ -42,6 +42,7 @@ $Date: 2011-08-17 23:12:23 +0200 (Wed, 17 Aug 2011) $
 #include "../utils/folderutils.h"
 #include "../utils/textutils.h"
 #include "../utils/graphicsutils.h"
+#include "../utils/cursormaster.h"
 #include "../utils/familypropertycombobox.h"
 #include "../referencemodel/referencemodel.h"
 
@@ -66,6 +67,8 @@ bool numberValueLessThan(QString v1, QString v2)
 	return NumberMatcherValues.value(v1, 0) <= NumberMatcherValues.value(v2, 0);
 }
 
+static QSvgRenderer MoveLockRenderer;
+
 /////////////////////////////////
 
 class NameTriple {
@@ -142,6 +145,8 @@ ItemBase::ItemBase( ModelPart* modelPart, ViewIdentifierClass::ViewIdentifier vi
 	//DebugDialog::debug(QString("itembase %1 %2").arg(id).arg((long) static_cast<QGraphicsItem *>(this), 0, 16));
 	m_hasRubberBandLeg = m_moveLock = m_hoverEnterSpaceBarWasPressed = m_spaceBarWasPressed = false;
 
+	m_moveLockItem = NULL;
+
 	m_everVisible = true;
 
 	m_rightClickedConnector = NULL;
@@ -159,7 +164,7 @@ ItemBase::ItemBase( ModelPart* modelPart, ViewIdentifierClass::ViewIdentifier vi
 	m_sticky = false;
 	m_canFlipHorizontal = m_canFlipVertical = false;
 
-	setCursor(*ConnectorItem::MoveCursor);
+	setCursor(*CursorMaster::MoveCursor);
 
    	m_viewGeometry.set(viewGeometry);
 	setAcceptHoverEvents ( true );
@@ -201,17 +206,6 @@ void ItemBase::setTooltip() {
 	}
 }
 
-void ItemBase::setConnectorTooltips() {
-	foreach (ConnectorItem * connectorItem, cachedConnectorItems()) {
-        QString tt = QString("<b>%1</b><br />%2" + ITEMBASE_FONT_PREFIX + "%3" + ITEMBASE_FONT_SUFFIX)
-                .arg(connectorItem->connectorSharedName())
-                .arg(connectorItem->connector()->connectorShared()->description())
-				.arg(toolTip());
-
-		connectorItem->setBaseTooltip(tt);
-	}
-}
-
 void ItemBase::removeTooltip() {
 	this->setToolTip(___emptyString___);
 }
@@ -267,8 +261,6 @@ void ItemBase::initNames() {
 		NumberMatcher.setPattern(QString("(([0-9]+(\\.[0-9]*)?)|\\.[0-9]+)([\\s]*([") + TextUtils::PowerPrefixesString + "]))?");
 	}
 
-	ConnectorItem::initCursors();
-
 	if (TranslatedPropertyNames.count() == 0) {
 		TranslatedPropertyNames.insert("family", tr("family"));
 		TranslatedPropertyNames.insert("type", tr("type"));
@@ -331,6 +323,9 @@ void ItemBase::initNames() {
 void ItemBase::saveInstance(QXmlStreamWriter & streamWriter) {
 	streamWriter.writeStartElement(ViewIdentifierClass::viewIdentifierXmlName(m_viewIdentifier));
 	streamWriter.writeAttribute("layer", ViewLayer::viewLayerXmlNameFromID(m_viewLayerID));
+	if (m_moveLock) {
+		streamWriter.writeAttribute("locked", "true");
+	}
 	this->saveGeometry();
 	writeGeometry(streamWriter);
 	if (m_partLabel) {
@@ -339,7 +334,7 @@ void ItemBase::saveInstance(QXmlStreamWriter & streamWriter) {
 
 	bool saveConnectorItems = false;
 	foreach (ConnectorItem * connectorItem, cachedConnectorItems()) {
-		if (connectorItem->connectionsCount() > 0 || connectorItem->hasRubberBandLeg()) {
+		if (connectorItem->connectionsCount() > 0 || connectorItem->hasRubberBandLeg() || connectorItem->isGroundFillSeed()) {
 			saveConnectorItems = true;
 			break;
 		}
@@ -610,10 +605,6 @@ void ItemBase::hoverLeaveEvent ( QGraphicsSceneHoverEvent * event ) {
 	}
 }
 
-void ItemBase::hoverMoveEvent ( QGraphicsSceneHoverEvent *  ) {
-	//DebugDialog::debug(QString("hover move %1 %2").arg(instanceTitle()).arg(QTime::currentTime().msec()));
-}
-
 void ItemBase::updateConnections() {
 }
 
@@ -638,6 +629,8 @@ const QHash<QString, QPointer<Bus> > & ItemBase::buses() {
 }
 
 void ItemBase::busConnectorItems(class Bus * bus, QList<class ConnectorItem *> & items) {
+	if (bus == NULL) return;
+
 	foreach (Connector * connector, bus->connectors()) {
 		QList< QPointer<ConnectorItem> > allConnectorItems = connector->viewItems();
 		foreach (ConnectorItem * connectorItem, allConnectorItems) {
@@ -686,20 +679,6 @@ void ItemBase::paint(QPainter *painter, const QStyleOptionGraphicsItem *option,
 		layerKinChief()->paintSelected(painter, option, widget);
     }	
 
-	if (m_moveLock) {
-		painter->save();
-		QPen pen;
-		pen.setColor(QColor(0,0,0));
-		pen.setWidth(2);
-		painter->setPen(pen);
-		painter->drawArc(2, 1, 6, 4, 180 * 16, -180 * 16);
-		painter->drawLine(2, 4, 2, 5);
-		painter->drawLine(8, 4, 8, 5);
-		painter->fillRect(0, 5, 10, 7, QColor(0,0,0));
-		
-		painter->restore();
-	}
-
 	if (m_inactive) {
 		painter->restore();
 	}
@@ -978,7 +957,7 @@ bool ItemBase::canFlip(Qt::Orientations orientations) {
 }
 
 bool ItemBase::canFlipHorizontal() {
-	return m_canFlipHorizontal;
+	return m_canFlipHorizontal && !m_moveLock;
 }
 
 void ItemBase::setCanFlipHorizontal(bool cf) {
@@ -986,7 +965,7 @@ void ItemBase::setCanFlipHorizontal(bool cf) {
 }
 
 bool ItemBase::canFlipVertical() {
-	return m_canFlipVertical;
+	return m_canFlipVertical && !m_moveLock;
 }
 
 void ItemBase::setCanFlipVertical(bool cf) {
@@ -994,11 +973,11 @@ void ItemBase::setCanFlipVertical(bool cf) {
 }
 
 bool ItemBase::rotationAllowed() {
-	return true;
+	return !m_moveLock;
 }
 
 bool ItemBase::rotation45Allowed() {
-	return true;
+	return !m_moveLock;
 }
 
 void ItemBase::clearModelPart() {
@@ -1041,10 +1020,11 @@ void ItemBase::clearPartLabel() {
 void ItemBase::restorePartLabel(QDomElement & labelGeometry, ViewLayer::ViewLayerID viewLayerID)
 {
 	if (m_partLabel) {
+		m_partLabel->setPlainText(instanceTitle());
 		if (!labelGeometry.isNull()) {
 			m_partLabel->restoreLabel(labelGeometry, viewLayerID);
+			//m_partLabel->setPlainText(instanceTitle());
 		}
-		m_partLabel->setPlainText(instanceTitle());
 	}
 }
 
@@ -1109,16 +1089,6 @@ QVariant ItemBase::itemChange(QGraphicsItem::GraphicsItemChange change, const QV
 	return QGraphicsSvgItem::itemChange(change, value);
 }
 
-QString ItemBase::toolTip2() {
-	// because QGraphicsItem::toolTip() isn't virtual
-	QString tt = toolTip();
-	if (!tt.isEmpty()) return tt;
-
-	// because the tooltip may not have been initialized for this view
-	setTooltip();
-	return toolTip();
-}
-
 void ItemBase::cleanup() {
 }
 
@@ -1131,6 +1101,7 @@ ItemBase * ItemBase::layerKinChief() {
 }
 
 void ItemBase::rotateItem(double degrees) {
+	//DebugDialog::debug(QString("\trotating item %1 %2").arg(degrees).arg((long) this, 0, 16));
 	transformItem(QTransform().rotate(degrees));
 }
 
@@ -1151,6 +1122,8 @@ void ItemBase::flipItem(Qt::Orientations orientation) {
 }
 
 void ItemBase::transformItem(const QTransform & currTransf) {
+	//DebugDialog::debug(QString("\ttransform item %1").arg((long) this, 0, 16));
+
 	if (m_hasRubberBandLeg) {
 		prepareGeometryChange();
 	}
@@ -1163,6 +1136,8 @@ void ItemBase::transformItem(const QTransform & currTransf) {
 	if (!m_hasRubberBandLeg) {
 		updateConnections();
 	}
+	//QTransform t = this->transform();
+	//DebugDialog::debug(QString("matrix m11:%1 m12:%2 m21:%3 m22:%4").arg(t.m11()).arg(t.m12()).arg(t.m21()).arg(t.m22()));
 	update();
 }
 
@@ -1270,8 +1245,8 @@ FSvgRenderer * ItemBase::setUpImage(ModelPart * modelPart, ViewIdentifierClass::
 					case ViewIdentifierClass::BreadboardView:
 						modelPartShared->connectorIDs(viewIdentifier, viewLayerID, connectorIDs, terminalIDs, legIDs);
 						break;
-                                        default:
-                                                break;
+                    default:
+                        break;
 				}
 
 				bool gotOne = false;
@@ -1523,8 +1498,16 @@ bool ItemBase::hasCustomSVG() {
 }
 
 void ItemBase::setProp(const QString & prop, const QString & value) {
-	Q_UNUSED(prop);
-	Q_UNUSED(value);
+	if (!m_modelPart) return;
+
+	m_modelPart->setProp(prop, value);
+}
+
+QString ItemBase::prop(const QString & p)
+{
+	if (m_modelPart == NULL) return "";
+
+	return m_modelPart->prop(p).toString();
 }
 
 bool ItemBase::isObsolete() {
@@ -1654,7 +1637,7 @@ ViewLayer::ViewLayerID ItemBase::partLabelViewLayerID() {
 QString ItemBase::makePartLabelSvg(bool blackOnly, double dpi, double printerScale) {
 	if (m_partLabel == NULL) return "";
 	if (!m_partLabel->initialized()) return "";
-	return m_partLabel->makeSvg(blackOnly, dpi, printerScale);
+	return m_partLabel->makeSvg(blackOnly, dpi, printerScale, true);
 }
 
 QPointF ItemBase::partLabelScenePos() {
@@ -1672,11 +1655,11 @@ QRectF ItemBase::partLabelSceneBoundingRect() {
 bool ItemBase::getFlipDoc(ModelPart * modelPart, const QString & filename, ViewLayer::ViewLayerID viewLayerID, ViewLayer::ViewLayerSpec viewLayerSpec, QDomDocument & flipDoc)
 {
 	if (viewLayerSpec == ViewLayer::ThroughHoleThroughTop_OneLayer) {
-		if ((viewLayerID == ViewLayer::Copper0) && modelPart->flippedSMD() && (viewLayerSpec == ViewLayer::ThroughHoleThroughTop_OneLayer)) {
+		if ((viewLayerID == ViewLayer::Copper0) && modelPart->flippedSMD()) {
 			SvgFlattener::flipSMDSvg(filename, "", flipDoc, ViewLayer::viewLayerXmlNameFromID(ViewLayer::Copper1), ViewLayer::viewLayerXmlNameFromID(ViewLayer::Copper0), FSvgRenderer::printerScale());
 			return true;
 		}
-		else if ((viewLayerID == ViewLayer::Silkscreen0) && modelPart->flippedSMD() && (viewLayerSpec == ViewLayer::ThroughHoleThroughTop_OneLayer)) {
+		else if ((viewLayerID == ViewLayer::Silkscreen0) && modelPart->flippedSMD()) {
 			SvgFlattener::flipSMDSvg(filename, "", flipDoc, ViewLayer::viewLayerXmlNameFromID(ViewLayer::Silkscreen1), ViewLayer::viewLayerXmlNameFromID(ViewLayer::Silkscreen0), FSvgRenderer::printerScale());
 			return true;
 		}
@@ -1726,7 +1709,33 @@ bool ItemBase::moveLock() {
 
 void ItemBase::setMoveLock(bool moveLock) 
 {
+	if (moveLock == m_moveLock) return;
+
 	m_moveLock = moveLock;
+	if (moveLock) {
+		if (!MoveLockRenderer.isValid()) {
+			QString fn(":resources/images/part_lock.svg");
+			bool success = MoveLockRenderer.load(fn);
+			DebugDialog::debug(QString("movelock load success %1").arg(success));
+		}
+
+		m_moveLockItem = new QGraphicsSvgItem();
+		m_moveLockItem->setAcceptHoverEvents(false);
+		m_moveLockItem->setAcceptedMouseButtons(Qt::NoButton);
+		m_moveLockItem->setSharedRenderer(&MoveLockRenderer);
+		m_moveLockItem->setPos(0,0);
+		m_moveLockItem->setZValue(-99999);
+		m_moveLockItem->setParentItem(this);
+		m_moveLockItem->setVisible(true);
+
+	}
+	else {
+		if (m_moveLockItem) {
+			delete m_moveLockItem;
+			m_moveLockItem = NULL;
+		}
+	}
+
 	update();
 }
 
@@ -1858,5 +1867,12 @@ QPainterPath ItemBase::hoverShape() const
 
 const QCursor * ItemBase::getCursor(Qt::KeyboardModifiers)
 {
-	return ConnectorItem::MoveCursor;
+	return CursorMaster::MoveCursor;
 }
+
+PartLabel * ItemBase::partLabel() {
+	return m_partLabel;
+}
+
+void ItemBase::doneLoading() {
+}
\ No newline at end of file
