@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5375 $:
+$Revision: 5950 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-08-08 21:38:25 +0200 (Mon, 08 Aug 2011) $
+$Date: 2012-04-07 19:26:22 +0200 (Sat, 07 Apr 2012) $
 
 ********************************************************************/
 
@@ -33,7 +33,7 @@ $Date: 2011-08-08 21:38:25 +0200 (Mon, 08 Aug 2011) $
 #include "../svg/svgfilesplitter.h"
 #include "moduleidnames.h"
 #include "../svg/groundplanegenerator.h"
-
+#include "../utils/cursormaster.h"
 
 #include <QHBoxLayout>
 #include <QVBoxLayout>
@@ -45,6 +45,7 @@ $Date: 2011-08-08 21:38:25 +0200 (Mon, 08 Aug 2011) $
 #include <QMessageBox>
 #include <QImage>
 #include <QLineEdit>
+#include <QApplication>
 
 static QStringList ImageNames;
 static QStringList NewImageNames;
@@ -60,6 +61,7 @@ LogoItem::LogoItem( ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier v
 		ImageNames << "Made with Fritzing" << "Fritzing icon" << "OHANDA logo" << "OSHW logo";
 	}
 
+	m_inLogoEntry = QTime::currentTime().addSecs(-10);
 	m_fileNameComboBox = NULL;
 	m_aspectRatioCheck = NULL;
 	m_keepAspectRatio = true;
@@ -82,18 +84,13 @@ void LogoItem::addedToScene(bool temporary)
 		m_aspectRatio.setWidth(this->boundingRect().width());
 		m_aspectRatio.setHeight(this->boundingRect().height());
 		m_originalFilename = filename();
-		QString shape = modelPart()->prop("shape").toString();
-		if (!shape.isEmpty()) {
-					
+		QString shape = prop("shape");
+		if (!shape.isEmpty()) {					
 			m_aspectRatio = modelPart()->prop("aspectratio").toSizeF();
-			if (m_renderer == NULL) {
-				m_renderer = new FSvgRenderer(this);
+			if (loadExtraRenderer(shape.toUtf8(), false)) {
 			}
-
-			bool result = m_renderer->fastLoad(shape.toUtf8());
-			if (result) {
-				setSharedRendererEx(m_renderer);
-				positionGrips();
+			else {
+				DebugDialog::debug("bad shape in " + m_originalFilename + " " + shape);
 			}
 		}
 		else {
@@ -115,7 +112,7 @@ void LogoItem::addedToScene(bool temporary)
 QString LogoItem::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, double dpi)
 {
 	if (viewLayerID == layer() ) {
-		QString svg = modelPart()->prop("shape").toString();
+		QString svg = prop("shape");
 		if (!svg.isEmpty()) {
 			QString xmlName = ViewLayer::viewLayerXmlNameFromID(viewLayerID);
 			SvgFileSplitter splitter;
@@ -123,11 +120,44 @@ QString LogoItem::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QString,
 			if (!result) {
 				return "";
 			}
+
+			double scaleX = 1;
+			double scaleY = 1;
+			if (m_hasLogo) {
+				QDomDocument doc = splitter.domDocument();
+				QDomElement root = doc.documentElement();
+				QDomElement g = root.firstChildElement("g");
+				QDomElement text = g.firstChildElement("text");
+
+				// TODO: this is really a hack and resizing should change a scale factor rather than the <text> coordinates
+				// but it's not clear how to deal with existing sketches
+	
+				QString viewBox = root.attribute("viewBox");
+				double w = TextUtils::convertToInches(root.attribute("width"));
+				double h = TextUtils::convertToInches(root.attribute("height"));
+				QStringList coords = viewBox.split(" ", QString::SkipEmptyParts);
+				double sx = w / coords.at(2).toDouble();
+				double sy = h / coords.at(3).toDouble();
+				if (qAbs(sx - sy) > .001) {
+					// change vertical dpi to match horizontal dpi
+					// y coordinate is really intended in relation to font size so leave it be
+					scaleY = sy / sx;
+					root.setAttribute("viewBox", QString("0 0 %1 %2").arg(coords.at(2)).arg(h / sx));
+				}
+			}
+
 			result = splitter.normalize(dpi, xmlName, blackOnly);
 			if (!result) {
 				return "";
 			}
-			return splitter.elementString(xmlName);
+
+			QString string = splitter.elementString(xmlName);
+
+			if (scaleY == 1) return string;
+
+			QTransform t;
+			t.scale(scaleX, scaleY);
+			return TextUtils::svgTransform(string, t, false, "");
 		}
 	}
 
@@ -141,14 +171,14 @@ bool LogoItem::collectExtraInfo(QWidget * parent, const QString & family, const
 			returnProp = tr("logo");
 			returnValue = m_logo;
 
-			QLineEdit * e1 = new QLineEdit(parent);
-			e1->setObjectName("infoViewLineEdit");
+			QLineEdit * edit = new QLineEdit(parent);
+			edit->setObjectName("infoViewLineEdit");
 
-			e1->setText(m_logo);
-			e1->setEnabled(swappingEnabled);
-			connect(e1, SIGNAL(editingFinished()), this, SLOT(logoEntry()));
+			edit->setText(m_logo);
+			edit->setEnabled(swappingEnabled);
+			connect(edit, SIGNAL(editingFinished()), this, SLOT(logoEntry()));
 
-			returnWidget = e1;
+			returnWidget = edit;
 			return true;
 		}
 	}
@@ -190,96 +220,8 @@ bool LogoItem::collectExtraInfo(QWidget * parent, const QString & family, const
 	}
 
 	if (prop.compare("shape", Qt::CaseInsensitive) == 0) {
+		returnWidget = setUpDimEntry(true, returnWidget);
 		returnProp = tr("shape");
-		double w = qRound(m_modelPart->prop("width").toDouble() * 10) / 10.0;	// truncate to 1 decimal point
-		double h = qRound(m_modelPart->prop("height").toDouble() * 10) / 10.0;  // truncate to 1 decimal point
-
-		QVBoxLayout * vboxLayout = NULL;
-		QFrame * frame = NULL;
-
-		frame = new QFrame();
-		frame->setObjectName("infoViewPartFrame");
-		vboxLayout = new QVBoxLayout();
-		vboxLayout->setAlignment(Qt::AlignLeft);
-		vboxLayout->setSpacing(1);
-		vboxLayout->setContentsMargins(0, 3, 0, 0);
-
-		QFrame * subframe1 = new QFrame();
-		QHBoxLayout * hboxLayout1 = new QHBoxLayout();
-		hboxLayout1->setAlignment(Qt::AlignLeft);
-		hboxLayout1->setContentsMargins(0, 0, 0, 0);
-		hboxLayout1->setSpacing(2);
-
-		QFrame * subframe2 = new QFrame();
-		QHBoxLayout * hboxLayout2 = new QHBoxLayout();
-		hboxLayout2->setAlignment(Qt::AlignLeft);
-		hboxLayout2->setContentsMargins(0, 0, 0, 0);
-		hboxLayout2->setSpacing(2);
-
-		QFrame * subframe3 = new QFrame();
-		QHBoxLayout * hboxLayout3 = new QHBoxLayout();
-		hboxLayout3->setAlignment(Qt::AlignLeft);
-		hboxLayout3->setContentsMargins(0, 0, 0, 0);
-		hboxLayout3->setSpacing(0);
-
-		QLabel * l1 = new QLabel(tr("width(mm)"));	
-		l1->setMargin(0);
-		l1->setObjectName("infoViewLabel");
-		QLineEdit * e1 = new QLineEdit();
-		QDoubleValidator * validator = new QDoubleValidator(e1);
-		validator->setRange(0.1, 999.9, 1);
-		validator->setNotation(QDoubleValidator::StandardNotation);
-		e1->setObjectName("infoViewLineEdit");
-		e1->setValidator(validator);
-		e1->setMaxLength(5);
-		e1->setText(QString::number(w));
-
-		QLabel * l2 = new QLabel(tr("height(mm)"));
-		l2->setMargin(0);
-		l2->setObjectName("infoViewLabel");
-		QLineEdit * e2 = new QLineEdit();
-		validator = new QDoubleValidator(e1);
-		validator->setRange(0.1, 999.9, 1);
-		validator->setNotation(QDoubleValidator::StandardNotation);
-		e2->setObjectName("infoViewLineEdit");
-		e2->setValidator(validator);
-		e2->setMaxLength(5);
-		e2->setText(QString::number(h));
-
-		QLabel * l3 = new QLabel(tr("keep in proportion"));
-		l3->setMargin(0);
-		l3->setObjectName("infoViewLabel");
-		QCheckBox * checkBox = new QCheckBox();
-		checkBox->setChecked(m_keepAspectRatio);
-		checkBox->setObjectName("infoViewCheckBox");
-
-		hboxLayout1->addWidget(l1);
-		hboxLayout1->addWidget(e1);
-		hboxLayout2->addWidget(l2);
-		hboxLayout2->addWidget(e2);
-		hboxLayout3->addWidget(l3);
-		hboxLayout3->addWidget(checkBox);
-
-		subframe1->setLayout(hboxLayout1);
-		subframe2->setLayout(hboxLayout2);
-		subframe3->setLayout(hboxLayout3);
-
-		connect(e1, SIGNAL(editingFinished()), this, SLOT(widthEntry()));
-		connect(e2, SIGNAL(editingFinished()), this, SLOT(heightEntry()));
-		connect(checkBox, SIGNAL(toggled(bool)), this, SLOT(keepAspectRatio(bool)));
-
-		m_widthEditor = e1;
-		m_heightEditor = e2;
-		m_aspectRatioCheck = checkBox;
-
-		vboxLayout->addWidget(subframe1);
-		vboxLayout->addWidget(subframe2);
-		vboxLayout->addWidget(subframe3);
-		frame->setLayout(vboxLayout);
-
-		returnWidget = frame;
-
-		returnProp = tr("size");
 		return true;
 	}
 
@@ -287,10 +229,6 @@ bool LogoItem::collectExtraInfo(QWidget * parent, const QString & family, const
 
 }
 
-bool LogoItem::hasGrips() {
-	return true;
-}
-
 void LogoItem::prepLoadImage() {
 	QList<QByteArray> supportedImageFormats = QImageReader::supportedImageFormats();
 	QString imagesStr = tr("Images");
@@ -318,20 +256,16 @@ void LogoItem::prepLoadImageAux(const QString & fileName, bool addName)
 {
 	InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
 	if (infoGraphicsView != NULL) {
-		infoGraphicsView->loadLogoImage(this->id(), modelPart()->prop("shape").toString(), m_aspectRatio, modelPart()->prop("lastfilename").toString(), fileName, addName);
+		infoGraphicsView->loadLogoImage(this->id(), prop("shape"), m_aspectRatio, prop("lastfilename"), fileName, addName);
 	}
 }
 
 void LogoItem::reloadImage(const QString & svg, const QSizeF & aspectRatio, const QString & fileName, bool addName) 
 {
-	if (m_renderer == NULL) {
-		m_renderer = new FSvgRenderer(this);
-	}
-	bool result = m_renderer->fastLoad(svg.toUtf8());
+	bool result = loadExtraRenderer(svg.toUtf8(), false);
 	if (result) {
-		setSharedRendererEx(m_renderer);
 		if (aspectRatio == QSizeF(0, 0)) {
-			QRectF r = m_renderer->viewBoxF();
+			QRectF r = m_extraRenderer->viewBoxF();
 			m_aspectRatio.setWidth(r.width());
 			m_aspectRatio.setHeight(r.height());
 		}
@@ -353,13 +287,16 @@ void LogoItem::reloadImage(const QString & svg, const QSizeF & aspectRatio, cons
 				m_fileNameComboBox->blockSignals(wasBlocked);
 			}
 		}
-		positionGrips();
 		m_logo = "";
+
+		LayerHash layerHash;
+		resizeMM(GraphicsUtils::pixels2mm(m_aspectRatio.width(), FSvgRenderer::printerScale()),
+				 GraphicsUtils::pixels2mm(m_aspectRatio.height(), FSvgRenderer::printerScale()),
+				 layerHash);
 	}
 	else {
 		// restore previous (not sure whether this is necessary)
-		m_renderer->fastLoad(modelPart()->prop("shape").toString().toUtf8());
-		setSharedRendererEx(m_renderer);
+		loadExtraRenderer(prop("shape").toUtf8(), false);
 		unableToLoad(fileName);
 	}
 }
@@ -447,13 +384,15 @@ void LogoItem::loadImage(const QString & fileName, bool addName)
 			return;
 		}
 
-		if (image.format() != QImage::Format_RGB32) {
-			image = image.convertToFormat(QImage::Format_RGB32);
+		if (image.format() != QImage::Format_RGB32 && image.format() != QImage::Format_ARGB32) {
+			image = image.convertToFormat(QImage::Format_Mono);
 		}
 
 		GroundPlaneGenerator gpg;
+		gpg.setLayerName(layerName());
+		gpg.setMinRunSize(1, 1);
 		double res = image.dotsPerMeterX() / GraphicsUtils::InchesPerMeter;
-		gpg.scanImage(image, image.width(), image.height(), 1, res, colorString(), layerName(), false, 1, false, QSizeF(0, 0), 0);
+		gpg.scanImage(image, image.width(), image.height(), 1, res, colorString(), false, false, QSizeF(0, 0), 0, QPointF(0, 0));
 		QStringList newSvgs = gpg.newSVGs();
 		if (newSvgs.count() < 1) {
 			QMessageBox::information(
@@ -471,10 +410,6 @@ void LogoItem::loadImage(const QString & fileName, bool addName)
 		svg = TextUtils::mergeSvgFinish(doc);
 	}
 
-	if (m_renderer == NULL) {
-		m_renderer = new FSvgRenderer(this);
-	}
-
 	reloadImage(svg, QSizeF(0, 0), fileName, addName);
 }
 
@@ -485,24 +420,21 @@ void LogoItem::resizeMM(double mmW, double mmH, const LayerHash & viewLayers) {
 		return;
 	}
 
+	DebugDialog::debug(QString("resize mm %1 %2").arg(mmW).arg(mmH));
+
 	QRectF r = this->boundingRect();
 	if (qAbs(GraphicsUtils::pixels2mm(r.width(), FSvgRenderer::printerScale()) - mmW) < .001 &&
 		qAbs(GraphicsUtils::pixels2mm(r.height(), FSvgRenderer::printerScale()) - mmH) < .001) 
 	{
-		positionGrips();
 		return;
 	}
 
-	if (m_renderer == NULL) {
-		m_renderer = new FSvgRenderer(this);
-	}
-
 	double inW = GraphicsUtils::mm2mils(mmW) / 1000;
 	double inH = GraphicsUtils::mm2mils(mmH) / 1000;
 
 	// TODO: deal with aspect ratio
 
-	QString svg = modelPart()->prop("shape").toString();
+	QString svg = prop("shape");
 	if (svg.isEmpty()) return;
 
 	QString errorStr;
@@ -528,22 +460,14 @@ void LogoItem::resizeMM(double mmW, double mmH, const LayerHash & viewLayers) {
 
 	svg = TextUtils::removeXMLEntities(domDocument.toString());			
 
-	bool result = m_renderer->fastLoad(svg.toUtf8());
+	bool result = loadExtraRenderer(svg.toUtf8(), false);
 	if (result) {
-		setSharedRendererEx(m_renderer);
 		modelPart()->setProp("shape", svg);
 		modelPart()->setProp("width", mmW);
 		modelPart()->setProp("height", mmH);
-		positionGrips();
 	}
 
-	if (m_widthEditor) {
-		m_widthEditor->setText(QString::number(qRound(mmW * 10) / 10.0));
-	}
-
-	if (m_heightEditor) {
-		m_heightEditor->setText(QString::number(qRound(mmH * 10) / 10.0));
-	}
+	setWidthAndHeight(qRound(mmW * 10) / 10.0, qRound(mmH * 10) / 10.0);
 }
 
 void LogoItem::setProp(const QString & prop, const QString & value) {
@@ -556,7 +480,6 @@ void LogoItem::setProp(const QString & prop, const QString & value) {
 }
 
 void LogoItem::setLogo(QString logo, bool force) {
-
 	if (!force && m_logo.compare(logo) == 0) return;
 
 	switch (this->m_viewIdentifier) {
@@ -574,32 +497,44 @@ void LogoItem::setLogo(QString logo, bool force) {
 
 	if (svg.isEmpty()) return;
 
-	svg = hackSvg(svg, logo);
+	QSizeF oldSize = m_size;
+	QXmlStreamReader streamReader(svg);
+	QSizeF oldSvgSize = m_extraRenderer ? m_extraRenderer->viewBoxF().size() : QSizeF(0, 0);
+	
+	DebugDialog::debug(QString("size %1 %2, %3 %4").arg(m_size.width()).arg(m_size.height()).arg(oldSvgSize.width()).arg(oldSvgSize.height()));
 
-	rerender(svg);
+	svg = hackSvg(svg, logo);
+	QXmlStreamReader newStreamReader(svg);
+
+	bool ok = rerender(svg);
 
 	m_logo = logo;
 	modelPart()->setProp("logo", logo);
 	modelPart()->setProp("shape", svg);
-	positionGrips();
-
-	updateTooltip();
+	if (ok && !force) {
+		QSizeF newSvgSize = m_extraRenderer->viewBoxF().size();
+		QSizeF newSize = newSvgSize * oldSize.height() / oldSvgSize.height();
+		DebugDialog::debug(QString("size %1 %2, %3 %4").arg(m_size.width()).arg(m_size.height()).arg(newSize.width()).arg(newSize.height()));
+		
+		// set the new text to approximately the same height as the original
+		// if the text is non-proportional that will be lost
+		LayerHash layerHash;
+		resizeMM(GraphicsUtils::pixels2mm(newSize.width(), FSvgRenderer::printerScale()),
+				 GraphicsUtils::pixels2mm(newSize.height(), FSvgRenderer::printerScale()),
+				 layerHash);
+		//DebugDialog::debug(QString("size %1 %2").arg(m_size.width()).arg(m_size.height()));
+	}
 }
 
-void LogoItem::rerender(const QString & svg)
+bool LogoItem::rerender(const QString & svg)
 {
-	if (m_renderer == NULL) {
-		m_renderer = new FSvgRenderer(this);
-	}
-	//DebugDialog::debug(svg);
-
-	bool result = m_renderer->fastLoad(svg.toUtf8());
+	bool result = loadExtraRenderer(svg.toUtf8(), false);
 	if (result) {
-		setSharedRendererEx(m_renderer);
-		QRectF r = m_renderer->viewBoxF();
+		QRectF r = m_extraRenderer->viewBoxF();
 		m_aspectRatio.setWidth(r.width());
 		m_aspectRatio.setHeight(r.height());
 	}
+	return result;
 }
 
 QString LogoItem::getProperty(const QString & key) {
@@ -628,6 +563,10 @@ void LogoItem::logoEntry() {
 
 	if (edit->text().compare(this->logo()) == 0) return;
 
+	// m_inLogoEntry is a hack because a focus out event was being triggered on m_widthEntry when a user hit the return key in logoentry
+	// this triggers a call to widthEntry() which causes all kinds of havoc.  (bug in version 0.7.1 and possibly earlier)
+	m_inLogoEntry = QTime::currentTime().addSecs(1);
+
 	InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
 	if (infoGraphicsView != NULL) {
 		infoGraphicsView->setProp(this, "logo", tr("logo"), this->logo(), edit->text(), true);
@@ -687,10 +626,15 @@ QString LogoItem::hackSvg(const QString & svg, const QString & logo)
 }
 
 void LogoItem::widthEntry() {
+	if (QTime::currentTime() < m_inLogoEntry) return;
+
 	QLineEdit * edit = qobject_cast<QLineEdit *>(sender());
 	if (edit == NULL) return;
 
 	double w = edit->text().toDouble();
+	double oldW = m_modelPart->prop("width").toDouble();
+	if (w == oldW) return;
+
 	double h = m_modelPart->prop("height").toDouble();
 	if (m_keepAspectRatio) {
 		h = w * m_aspectRatio.height() / m_aspectRatio.width();
@@ -703,11 +647,21 @@ void LogoItem::widthEntry() {
 }
 
 void LogoItem::heightEntry() {
+	if (QTime::currentTime() < m_inLogoEntry) return;
+
 	QLineEdit * edit = qobject_cast<QLineEdit *>(sender());
 	if (edit == NULL) return;
 
-	double w = m_modelPart->prop("width").toDouble();
 	double h = edit->text().toDouble();
+	double oldH =  m_modelPart->prop("height").toDouble();
+	if (h == oldH) return;
+
+	setHeight(h);
+}
+
+void LogoItem::setHeight(double h)
+{
+	double w = m_modelPart->prop("width").toDouble();
 	if (m_keepAspectRatio) {
 		w = h * m_aspectRatio.width() / m_aspectRatio.height();
 	}
@@ -750,7 +704,7 @@ void LogoItem::setFileNameItems() {
 
 	int ix = 0;
 	foreach (QString name, getImageNames()) {
-		if (modelPart()->prop("lastfilename").toString().contains(name)) {
+		if (prop("lastfilename").contains(name)) {
 			m_fileNameComboBox->setCurrentIndex(ix);
 			return;
 		}
@@ -758,7 +712,7 @@ void LogoItem::setFileNameItems() {
 	}
 
 	foreach (QString name, getNewImageNames()) {
-		if (modelPart()->prop("lastfilename").toString().contains(name)) {
+		if (prop("lastfilename").contains(name)) {
 			m_fileNameComboBox->setCurrentIndex(ix);
 			return;
 		}
@@ -795,6 +749,40 @@ QStringList & LogoItem::getNewImageNames() {
 	return NewImageNames;
 }
 
+double LogoItem::minWidth() {
+	return ResizableBoard::CornerHandleSize * 2;
+}
+
+double LogoItem::minHeight() {
+	return ResizableBoard::CornerHandleSize * 2;
+}
+
+bool LogoItem::freeRotationAllowed(Qt::KeyboardModifiers modifiers) {
+
+	if ((modifiers & altOrMetaModifier()) == 0) return false;
+	if (!isSelected()) return false;
+
+	return true;
+}
+
+ResizableBoard::Corner LogoItem::findCorner(QPointF scenePos, Qt::KeyboardModifiers modifiers) {
+	ResizableBoard::Corner corner = ResizableBoard::findCorner(scenePos, modifiers);
+	if (corner == ResizableBoard::NO_CORNER) return corner;
+
+	if (modifiers & altOrMetaModifier()) {
+		// free rotate
+		setCursor(*CursorMaster::RotateCursor);
+		return ResizableBoard::NO_CORNER;
+	}
+
+	return corner;
+}
+
+void LogoItem::paintHover(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
+{
+	PaletteItem::paintHover(painter, option, widget);
+}
+
 ///////////////////////////////////////////////////////////////////////
 
 CopperLogoItem::CopperLogoItem( ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdentifier, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel)
@@ -805,7 +793,7 @@ CopperLogoItem::CopperLogoItem( ModelPart * modelPart, ViewIdentifierClass::View
 	}
 
 	if (Copper0ImageNames.count() == 0) {
-		Copper0ImageNames << "Fritzing icon copper1";
+		Copper0ImageNames << "Fritzing icon copper0";
 	}
 
 	m_hasLogo = (modelPart->moduleID().endsWith(ModuleIDNames::LogoTextModuleIDName));
@@ -820,24 +808,24 @@ CopperLogoItem::~CopperLogoItem() {
 }
 
 ViewLayer::ViewLayerID CopperLogoItem::layer() {
-	return modelPart()->properties().value("layer").contains("0") ? ViewLayer::Copper0 :  ViewLayer::Copper1;
+	return isCopper0() ? ViewLayer::Copper0 :  ViewLayer::Copper1;
 }
 
 QString CopperLogoItem::colorString() {
-	return modelPart()->properties().value("layer").contains("0") ? ViewLayer::Copper0Color :  ViewLayer::Copper1Color;
+	return isCopper0() ? ViewLayer::Copper0Color :  ViewLayer::Copper1Color;
 }
 
 QStringList & CopperLogoItem::getImageNames() {
-	return modelPart()->properties().value("layer").contains("0") ? Copper0ImageNames :  Copper1ImageNames;
+	return isCopper0() ? Copper0ImageNames :  Copper1ImageNames;
 }
 
 QStringList & CopperLogoItem::getNewImageNames() {
-	return modelPart()->properties().value("layer").contains("0") ? NewCopper0ImageNames :  NewCopper1ImageNames;
+	return isCopper0() ? NewCopper0ImageNames :  NewCopper1ImageNames;
 }
 
 QString CopperLogoItem::hackSvg(const QString & svg, const QString & logo) {
 	QString newSvg = LogoItem::hackSvg(svg, logo);
-	if (!modelPart()->properties().value("layer").contains("0")) return newSvg;
+	if (!isCopper0()) return newSvg;
 
 	return flipSvg(newSvg);
 }
@@ -860,7 +848,7 @@ QString CopperLogoItem::flipSvg(const QString & svg)
 
 void CopperLogoItem::reloadImage(const QString & svg, const QSizeF & aspectRatio, const QString & fileName, bool addName) 
 {
-	if (modelPart()->properties().value("layer").contains("0")) {
+	if (isCopper0()) {
 		if (!svg.contains("_flipped_")) {
 			LogoItem::reloadImage(flipSvg(svg), aspectRatio, fileName, addName);
 			return;
@@ -869,3 +857,8 @@ void CopperLogoItem::reloadImage(const QString & svg, const QSizeF & aspectRatio
 
 	LogoItem::reloadImage(svg, aspectRatio, fileName, addName);
 }
+
+bool CopperLogoItem::isCopper0() {
+	return modelPart()->properties().value("layer").contains("0");
+}
+
