@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5375 $:
+$Revision: 5857 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-08-08 21:38:25 +0200 (Mon, 08 Aug 2011) $
+$Date: 2012-02-12 18:16:07 +0100 (Sun, 12 Feb 2012) $
 
 ********************************************************************/
 
@@ -32,6 +32,8 @@ $Date: 2011-08-08 21:38:25 +0200 (Mon, 08 Aug 2011) $
 #include "../utils/textutils.h"
 #include "../layerattributes.h"
 #include "partlabel.h"
+#include "../connectors/connectoritem.h"
+
 
 #include <QDomNodeList>
 #include <QDomDocument>
@@ -46,7 +48,7 @@ static const int MinSipPins = 1;
 static const int MaxSipPins = 64;
 static const int MinDipPins = 4;
 static const int MaxDipPins = 64;
-int MysteryPart::NoExcusePins = 0;
+
 
 // TODO
 //	save into parts bin
@@ -65,8 +67,6 @@ MysteryPart::MysteryPart( ModelPart * modelPart, ViewIdentifierClass::ViewIdenti
 		m_spacing = modelPart->properties().value("spacing", "300mil");
 		modelPart->setProp("spacing", m_spacing);
 	}
-
-	m_renderer = NULL;
 }
 
 MysteryPart::~MysteryPart() {
@@ -131,7 +131,6 @@ void MysteryPart::setSpacing(QString spacing, bool force) {
 	m_spacing = spacing;
 	modelPart()->setProp("spacing", spacing);
 
-	updateTooltip();
     if (m_partLabel) m_partLabel->displayTextsIf();
 
 }
@@ -140,32 +139,25 @@ void MysteryPart::setChipLabel(QString chipLabel, bool force) {
 
 	if (!force && m_chipLabel.compare(chipLabel) == 0) return;
 
+	m_chipLabel = chipLabel;
+
 	QString svg;
 	switch (this->m_viewIdentifier) {
 		case ViewIdentifierClass::BreadboardView:
+			svg = makeSvg(chipLabel, true);
+			break;
 		case ViewIdentifierClass::SchematicView:
-			svg = makeSvg(chipLabel);
+			svg = makeSvg(chipLabel, false);
+			svg = retrieveSchematicSvg(svg);
 			break;
 		default:
 			break;
 	}
 
-	if (!svg.isEmpty()) {
-		if (m_renderer == NULL) {
-			m_renderer = new FSvgRenderer(this);
-		}
-		//DebugDialog::debug(svg);
+	loadExtraRenderer(svg, false);
 
-		bool result = m_renderer->fastLoad(svg.toUtf8());
-		if (result) {
-			setSharedRendererEx(m_renderer);
-		}
-	}
-
-	m_chipLabel = chipLabel;
 	modelPart()->setProp("chip label", chipLabel);
 
-	updateTooltip();
     if (m_partLabel) m_partLabel->displayTextsIf();
 
 }
@@ -175,9 +167,13 @@ QString MysteryPart::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QStri
 	QString svg = PaletteItem::retrieveSvg(viewLayerID, svgHash, blackOnly, dpi);
 	switch (viewLayerID) {
 		case ViewLayer::Breadboard:
-		case ViewLayer::Schematic:
 		case ViewLayer::Icon:
 			return TextUtils::replaceTextElement(svg, "label", m_chipLabel);
+
+		case ViewLayer::Schematic:
+			svg = retrieveSchematicSvg(svg);
+			return TextUtils::removeSVGHeader(svg);
+
 		default:
 			break;
 	}
@@ -185,13 +181,28 @@ QString MysteryPart::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QStri
 	return svg; 
 }
 
-QString MysteryPart::makeSvg(const QString & chipLabel) {
+QString MysteryPart::retrieveSchematicSvg(QString & svg) {
+
+	bool hasLocal = false;
+	QStringList labels = getPinLabels(hasLocal);
+		
+	if (hasLocal) {
+		svg = makeSchematicSvg(labels, false);
+	}
+
+	return TextUtils::replaceTextElement(svg, "label", m_chipLabel);
+}
+
+
+QString MysteryPart::makeSvg(const QString & chipLabel, bool replace) {
 	QString path = filename();
 	QFile file(filename());
 	QString svg;
 	if (file.open(QFile::ReadOnly)) {
 		svg = file.readAll();
 		file.close();
+		if (!replace) return svg;
+
 		return TextUtils::replaceTextElement(svg, "label", chipLabel);
 	}
 
@@ -280,7 +291,7 @@ void MysteryPart::addedToScene(bool temporary)
 		setSpacing(m_spacing, true);
 	}
 
-    return PaletteItem::addedToScene(temporary);
+    PaletteItem::addedToScene(temporary);
 }
 
 const QString & MysteryPart::title() {
@@ -364,12 +375,12 @@ ItemBase::PluralType MysteryPart::isPlural() {
 
 QString MysteryPart::genSipFZP(const QString & moduleid)
 {
-	return PaletteItem::genFZP(moduleid, "mystery_part_sipFzpTemplate", MinSipPins, MaxSipPins, 1);
+	return PaletteItem::genFZP(moduleid, "mystery_part_sipFzpTemplate", MinSipPins, MaxSipPins, 1, false);
 }
 
 QString MysteryPart::genDipFZP(const QString & moduleid)
 {
-	return PaletteItem::genFZP(moduleid, "mystery_part_dipFzpTemplate", MinDipPins, MaxDipPins, 2);
+	return PaletteItem::genFZP(moduleid, "mystery_part_dipFzpTemplate", MinDipPins, MaxDipPins, 2, false);
 }
 
 QString MysteryPart::genModuleID(QMap<QString, QString> & currPropsMap)
@@ -400,26 +411,53 @@ QString MysteryPart::makeSchematicSvg(const QString & expectedFileName)
 	}
 
 	int pins = pieces.at(2).toInt();
-	int increment = 300;
-	double totalHeight = (pins * increment) + 330;
+	QStringList labels;
+	for (int i = 0; i < pins; i++) {
+		labels << QString::number(i + 1);
+	}
+
+	return makeSchematicSvg(labels, sip);
+}
+
+QString MysteryPart::makeSchematicSvg(const QStringList & labels, bool sip) 
+{	
+	int increment = GraphicsUtils::StandardSchematicSeparationMils;   // 7.5mm;
 	int border = 30;
+	double totalHeight = (labels.count() * increment) + increment + border;
 	int textOffset = 50;
 	int repeatTextOffset = 50;
 	int fontSize = 255;
+	int labelFontSize = 130;
+	int defaultLabelWidth = 30;
 	QString labelText = "?";
+	double textMax = defaultLabelWidth;
+	QFont font("Droid Sans", labelFontSize * 72 / GraphicsUtils::StandardFritzingDPI, QFont::Normal);
+	QFontMetricsF fm(font);
+	for (int i = 0; i < labels.count(); i++) {
+		double w = fm.width(labels.at(i));
+		if (w > textMax) textMax = w;
+	}
+	textMax = textMax * GraphicsUtils::StandardFritzingDPI / 72;
+
+	int totalWidth = (5 * increment) + border;
+	int innerWidth = 4 * increment;
+	if (textMax > defaultLabelWidth) {
+		totalWidth += (textMax - defaultLabelWidth);
+		innerWidth += (textMax - defaultLabelWidth);
+	}
 
 	QString header("<?xml version='1.0' encoding='UTF-8' standalone='no'?>\n"
 					"<svg xmlns:svg='http://www.w3.org/2000/svg' \n"
 					"xmlns='http://www.w3.org/2000/svg' \n"
 					"version='1.2' baseProfile='tiny' \n"
-					"width='1.53in' height='%1in' viewBox='0 0 1530 %2'>\n"
+					"width='%7in' height='%1in' viewBox='0 0 %8 %2'>\n"
 					"<g id='schematic'>\n"
-					"<rect x='315' y='15' fill='none' width='1200' height='%3' stroke='#000000' stroke-linejoin='round' stroke-linecap='round' stroke-width='30' />\n"
-					"<text id='label' x='915' y='%4' fill='#000000' stroke='none' font-family='DroidSans' text-anchor='middle' font-size='%5' >%6</text>\n");
+					"<rect x='315' y='15' fill='none' width='%9' height='%3' stroke='#000000' stroke-linejoin='round' stroke-linecap='round' stroke-width='30' />\n"
+					"<text id='label' x='%11' y='%4' fill='#000000' stroke='none' font-family='DroidSans' text-anchor='middle' font-size='%5' >%6</text>\n");
 
 	if (!sip) {
-		header +=	"<circle fill='#000000' cx='1330' cy='200' r='150' stroke-width='0' />\n"
-					"<text x='1330' fill='#FFFFFF' y='305' font-family='DroidSans' text-anchor='middle' font-weight='bold' stroke-width='0' font-size='275' >?</text>\n";
+		header +=	"<circle fill='#000000' cx='%10' cy='200' r='150' stroke-width='0' />\n"
+					"<text x='%10' fill='#FFFFFF' y='305' font-family='DroidSans' text-anchor='middle' font-weight='bold' stroke-width='0' font-size='275' >?</text>\n";
 	}
 	else {
 		labelText = "IC";
@@ -428,26 +466,33 @@ QString MysteryPart::makeSchematicSvg(const QString & expectedFileName)
 	}
 
 	QString svg = header
-		.arg(totalHeight / 1000)
+		.arg(totalHeight / GraphicsUtils::StandardFritzingDPI)
 		.arg(totalHeight)
 		.arg(totalHeight - border)
 		.arg((totalHeight / 2) + textOffset)
 		.arg(fontSize)
-		.arg(labelText);
+		.arg(labelText)
+		.arg(totalWidth / 1000.0)
+		.arg(totalWidth)
+		.arg(innerWidth)
+		.arg(totalWidth - 200)
+		.arg(increment + textMax + ((totalWidth - increment - textMax) / 2.0))
+		;
 
 
 	QString repeat("<line fill='none' stroke='#000000' stroke-linejoin='round' stroke-linecap='round' stroke-width='30' x1='15' y1='%1' x2='300' y2='%1'  />\n"
 					"<rect x='0' y='%2' fill='none' width='300' height='30' id='connector%3pin' stroke-width='0' />\n"
 					"<rect x='0' y='%2' fill='none' width='30' height='30' id='connector%3terminal' stroke-width='0' />\n"
-					"<text id='label%3' x='390' y='%4' font-family='DroidSans' stroke='none' fill='#000000' text-anchor='start' font-size='130' >%5</text>\n");
+					"<text id='label%3' x='390' y='%4' font-family='DroidSans' stroke='none' fill='#000000' text-anchor='start' font-size='%6' >%5</text>\n");
   
-	for (int i = 0; i < pins; i++) {
+	for (int i = 0; i < labels.count(); i++) {
 		svg += repeat
-			.arg(315 + (i * increment))
-			.arg(300 + (i * increment))
+			.arg(increment + (border / 2) + (i * increment))
+			.arg(increment + (i * increment))
 			.arg(i)
-			.arg(300 + repeatTextOffset + (i * increment))
-			.arg(i + 1);
+			.arg(increment + repeatTextOffset + (i * increment))
+			.arg(labels.at(i))
+			.arg(labelFontSize);
 	}
 
 	svg += "</g>\n";
@@ -495,7 +540,7 @@ QString MysteryPart::makeBreadboardDipSvg(const QString & expectedFileName)
 					"</svg>\n");
 
 
-	header = TextUtils::incrementTemplateString(header, 1, spacing - increment, TextUtils::incMultiplyPinFunction, TextUtils::noCopyPinFunction);
+	header = TextUtils::incrementTemplateString(header, 1, spacing - increment, TextUtils::incMultiplyPinFunction, TextUtils::noCopyPinFunction, NULL);
 	header = header.arg(TextUtils::getViewBoxCoord(header, 3) / 100.0);
 	if (spacing == 10) {
 		header.replace("{{6.0}}", "8.0");
@@ -511,15 +556,17 @@ QString MysteryPart::makeBreadboardDipSvg(const QString & expectedFileName)
 	header.replace("{", "[");
 	header.replace("}", "]");
 
-	QString svg = TextUtils::incrementTemplateString(header, 1, increment * ((pins - 4) / 2), TextUtils::incMultiplyPinFunction, TextUtils::noCopyPinFunction);
+	QString svg = TextUtils::incrementTemplateString(header, 1, increment * ((pins - 4) / 2), TextUtils::incMultiplyPinFunction, TextUtils::noCopyPinFunction, NULL);
 
-	repeatB = TextUtils::incrementTemplateString(repeatB, 1, spacing - increment, TextUtils::incMultiplyPinFunction, TextUtils::noCopyPinFunction);
+	repeatB = TextUtils::incrementTemplateString(repeatB, 1, spacing - increment, TextUtils::incMultiplyPinFunction, TextUtils::noCopyPinFunction, NULL);
 	repeatB.replace("{", "[");
 	repeatB.replace("}", "]");
 
-	NoExcusePins = pins;
-	QString repeatTs = TextUtils::incrementTemplateString(repeatT, pins / 2, increment, TextUtils::standardMultiplyPinFunction, negCopyPinFunction);
-	QString repeatBs = TextUtils::incrementTemplateString(repeatB, pins / 2, increment, TextUtils::standardMultiplyPinFunction, TextUtils::standardCopyPinFunction);
+	int userData[2];
+	userData[0] = pins;
+	userData[1] = 1;
+	QString repeatTs = TextUtils::incrementTemplateString(repeatT, pins / 2, increment, TextUtils::standardMultiplyPinFunction, TextUtils::negIncCopyPinFunction, userData);
+	QString repeatBs = TextUtils::incrementTemplateString(repeatB, pins / 2, increment, TextUtils::standardMultiplyPinFunction, TextUtils::standardCopyPinFunction, NULL);
 
 	return svg.arg(TextUtils::getViewBoxCoord(svg, 2) / 100).arg(repeatTs).arg(repeatBs);
 }
@@ -545,23 +592,27 @@ QString MysteryPart::makeBreadboardSipSvg(const QString & expectedFileName)
 					"</g>\n"
 					"</svg>\n");
 
-	QString svg = TextUtils::incrementTemplateString(header, 1, increment * (pins - 1), TextUtils::incMultiplyPinFunction, TextUtils::noCopyPinFunction);
+	QString svg = TextUtils::incrementTemplateString(header, 1, increment * (pins - 1), TextUtils::incMultiplyPinFunction, TextUtils::noCopyPinFunction, NULL);
 
 	QString repeat("<rect id='connector%1terminal' stroke='none' stroke-width='0' x='[1.87]' y='25.586' fill='#8C8C8C' width='2.3' height='2.0'/>\n"
 					"<rect id='connector%1pin' stroke='none' stroke-width='0' x='[1.87]' y='23.336' fill='#8C8C8C' width='2.3' height='4.25'/>\n");
 
-	QString repeats = TextUtils::incrementTemplateString(repeat, pins, increment, TextUtils::standardMultiplyPinFunction, TextUtils::standardCopyPinFunction);
+	QString repeats = TextUtils::incrementTemplateString(repeat, pins, increment, TextUtils::standardMultiplyPinFunction, TextUtils::standardCopyPinFunction, NULL);
 
 	return svg.arg(TextUtils::getViewBoxCoord(svg, 2) / 100).arg(repeats);
 }
 
-QString MysteryPart::negCopyPinFunction(int pin, const QString & argString) 
-{ 
-	// TODO: pass an argument to these copyPin functions
-	return argString.arg(NoExcusePins - (pin + 1)); 
-}
+bool MysteryPart::changePinLabels(bool singleRow, bool sip) {
+	Q_UNUSED(singleRow);
 
-QString MysteryPart::incCopyPinFunction(int pin, const QString & argString) 
-{ 
-	return argString.arg(pin + 1); 
-}
+	if (m_viewIdentifier != ViewIdentifierClass::SchematicView) return true;
+
+	bool hasLocal = false;
+	QStringList labels = getPinLabels(hasLocal);
+	if (labels.count() == 0) return true;
+
+	QString svg = MysteryPart::makeSchematicSvg(labels, sip);
+	loadExtraRenderer(svg, false);
+
+	return true;
+}
\ No newline at end of file
