@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5308 $:
+$Revision: 5874 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-07-30 21:09:56 +0200 (Sat, 30 Jul 2011) $
+$Date: 2012-02-20 14:06:43 +0100 (Mon, 20 Feb 2012) $
 
 ********************************************************************/
 
@@ -33,7 +33,11 @@ $Date: 2011-07-30 21:09:56 +0200 (Sat, 30 Jul 2011) $
 #include "partlabel.h"
 #include "../commands.h"
 #include "../connectors/connectoritem.h"
+#include "../connectors/svgidlayer.h"
 #include "../layerattributes.h"
+#include "../dialogs/pinlabeldialog.h"
+#include "../utils/folderutils.h"
+#include "../utils/textutils.h"
 
 #include <QGraphicsSceneMouseEvent>
 #include <QSvgRenderer>
@@ -42,10 +46,62 @@ $Date: 2011-07-30 21:09:56 +0200 (Sat, 30 Jul 2011) $
 #include <QDomElement>
 #include <QDir>
 #include <QMessageBox>
+#include <QPushButton>
+#include <QVBoxLayout>
+#include <QRegExp>
+#include <limits>
+
+/////////////////////////////////////////////////
+
+static bool ByIDParseSuccessful = true;
+static QRegExp IntegerFinder("\\d+");
+
+
+int findNumber(const QString & string) {
+	int ix = string.indexOf(IntegerFinder);
+	if (ix < 0) {
+		return -1;
+	}
+
+	int result = IntegerFinder.cap(0).toInt();
+	int length = IntegerFinder.cap(0).length();
+
+	int jx = string.lastIndexOf(IntegerFinder);
+	if (jx >= ix + length) {
+		return -1;
+	}
+
+	return result;
+}
+
+bool byID(ConnectorItem * c1, ConnectorItem * c2)
+{
+	int i1 = findNumber(c1->connectorSharedID());
+	if (i1 < 0) {
+		ByIDParseSuccessful = false;
+		return true;
+	}
+	int i2 = findNumber(c2->connectorSharedID());
+	if (i2 < 0) {
+		ByIDParseSuccessful = false;
+		return true;
+	}
+
+	if (i2 == i1 && c1 != c2) {
+		// should not be two connectors with the same number
+		ByIDParseSuccessful = false;
+		return true;
+	}
+
+	return i1 <= i2;
+}
+
+/////////////////////////////////////////////////
 
 PaletteItem::PaletteItem( ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdentifier, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu, bool doLabel)
 	: PaletteItemBase(modelPart, viewIdentifier, viewGeometry, id, itemMenu)
 {
+	m_extraRenderer = NULL;
 	if(doLabel) {
 		m_partLabel = new PartLabel(this, NULL);
 		m_partLabel->setVisible(false);
@@ -330,13 +386,6 @@ void PaletteItem::setInstanceTitle(const QString& title) {
 	}
 }
 
-void PaletteItem::updateTooltip() {
-	ItemBase::updateTooltip();
-	foreach (ItemBase * lkpi, m_layerKin) {
-		lkpi->updateTooltip();
-	}
-}
-
 QString PaletteItem::family() {
 	return modelPartShared()->family();
 }
@@ -461,22 +510,26 @@ void PaletteItem::resetKinImage(ItemBase * layerKin, InfoGraphicsView * infoGrap
 	qobject_cast<PaletteItemBase *>(layerKin)->setUpImage(modelPart(), layerKin->viewIdentifier(), infoGraphicsView->viewLayers(), layerKin->viewLayerID(), layerKin->viewLayerSpec(), true, layerAttributes, error);
 }
 
-QString PaletteItem::genFZP(const QString & moduleid, const QString & templateName, int minPins, int maxPins, int steps)
+QString PaletteItem::genFZP(const QString & moduleid, const QString & templateName, int minPins, int maxPins, int steps, bool smd)
 {
 	QString FzpTemplate = "";
 	QString ConnectorFzpTemplate = "";
 
-	if (FzpTemplate.isEmpty()) {
-		QFile file(QString(":/resources/templates/%1.txt").arg(templateName));
-		file.open(QFile::ReadOnly);
-		FzpTemplate = file.readAll();
-		file.close();
+
+	QFile file1(QString(":/resources/templates/%1.txt").arg(templateName));
+	file1.open(QFile::ReadOnly);
+	FzpTemplate = file1.readAll();
+	file1.close();
+	if (smd) {
+		FzpTemplate.replace("<layer layerId=\"copper0\"/>", "");
 	}
-	if (ConnectorFzpTemplate.isEmpty()) {
-		QFile file(":/resources/templates/generic_sip_connectorFzpTemplate.txt");
-		file.open(QFile::ReadOnly);
-		ConnectorFzpTemplate = file.readAll();
-		file.close();
+
+	QFile file2(":/resources/templates/generic_sip_connectorFzpTemplate.txt");
+	file2.open(QFile::ReadOnly);
+	ConnectorFzpTemplate = file2.readAll();
+	file2.close();
+	if (smd) {
+		ConnectorFzpTemplate.replace("<p layer=\"copper0\" svgId=\"connector%1pin\"/>", "");
 	}
 
 	QStringList ss = moduleid.split("_");
@@ -501,3 +554,231 @@ QString PaletteItem::genFZP(const QString & moduleid, const QString & templateNa
 
 	return FzpTemplate.arg(count).arg(middle);
 }
+
+bool PaletteItem::collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget)
+{
+	if (prop.compare("editable pin labels", Qt::CaseInsensitive) == 0 && value.compare("true") == 0) {
+		returnProp = "";
+		returnValue = value;
+
+		QPushButton * button = new QPushButton(tr("Edit Pin Labels"));
+		button->setObjectName("infoViewButton");
+		connect(button, SIGNAL(pressed()), this, SLOT(openPinLabelDialog()));
+		button->setEnabled(swappingEnabled);
+
+		returnWidget = button;
+
+		return true;
+	}
+
+	return PaletteItemBase::collectExtraInfo(parent, family, prop, value, swappingEnabled, returnProp, returnValue, returnWidget);
+}
+
+void PaletteItem::openPinLabelDialog() {
+	InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
+	if (infoGraphicsView == NULL) {
+		QMessageBox::warning(
+			NULL,
+			tr("Fritzing"),
+			tr("Unable to proceed; unable to find top level view.")
+		);		
+		return;	
+	}
+
+	QStringList labels;
+	QList<ConnectorItem *> sortedConnectorItems = sortConnectorItems();
+	if (sortedConnectorItems.count() == 0) {
+		QMessageBox::warning(
+			NULL,
+			tr("Fritzing"),
+			tr("Unable to proceed; part connectors do no have standard IDs.")
+		);
+		return;
+	}
+
+	foreach (ConnectorItem * connectorItem, sortedConnectorItems) {
+		labels.append(connectorItem->connectorSharedName());
+	}
+
+	QString chipLabel = modelPart()->prop("chip label").toString();
+	if (chipLabel.isEmpty()) {
+		chipLabel = instanceTitle();
+	}
+
+	bool singleRow = isSingleRow(sortedConnectorItems);
+	PinLabelDialog pinLabelDialog(labels, singleRow, chipLabel, modelPart()->isCore(), NULL);
+	int result = pinLabelDialog.exec();
+	if (result != QDialog::Accepted) return;
+
+	QStringList newLabels = pinLabelDialog.labels();
+	if (newLabels.count() != sortedConnectorItems.count()) {
+		QMessageBox::warning(
+			NULL,
+			tr("Fritzing"),
+			tr("Label mismatch.  Nothing was saved.")
+		);	
+		return;
+	}
+
+	infoGraphicsView->renamePins(this, labels, newLabels, singleRow);
+}
+
+void PaletteItem::renamePins(const QStringList & labels, bool singleRow)
+{
+	QList<ConnectorItem *> sortedConnectorItems = sortConnectorItems();
+	for (int i = 0; i < labels.count(); i++) {
+		ConnectorItem * connectorItem = sortedConnectorItems.at(i);
+		connectorItem->setConnectorLocalName(labels.at(i));
+	}
+
+	InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
+	infoGraphicsView->changePinLabels(this, singleRow);
+}
+
+bool PaletteItem::isSingleRow(QList<ConnectorItem *> & connectorItems) {
+	if (connectorItems.count() == 2) {
+		// no way to tell? so default to double
+		return false;
+	}
+	else if (connectorItems.count() % 2 == 0) {
+		QPointF p = connectorItems.at(0)->sceneAdjustedTerminalPoint(NULL);
+		double slope = 0;
+		for (int i = 1; i < connectorItems.count(); i++) { 
+			QPointF q = connectorItems.at(i)->sceneAdjustedTerminalPoint(NULL);
+			if (p == q) continue;
+			
+			double newSlope = q.x() == p.x() ? std::numeric_limits<double>::max() : (q.y()  - p.y()) / (q.x() - p.x());
+			if (i == 1) {
+				slope = newSlope;
+			}
+			else {
+				double d = qAbs(newSlope - slope);
+				if (d != 0 && d / qMax(qAbs(slope), qAbs(newSlope)) > 0.01) {
+					return false;
+				}
+			}
+		}
+	}
+
+	return true;
+}
+
+QList<ConnectorItem *> PaletteItem::sortConnectorItems() {
+	QList<ConnectorItem *> sortedConnectorItems(this->cachedConnectorItems());
+	ByIDParseSuccessful = true;
+	qSort(sortedConnectorItems.begin(), sortedConnectorItems.end(), byID);
+	if (!ByIDParseSuccessful || sortedConnectorItems.count() == 0) {		
+		sortedConnectorItems.clear();
+	}
+
+	return sortedConnectorItems;
+}
+
+bool PaletteItem::changePinLabels(bool singleRow, bool sip) {
+	Q_UNUSED(singleRow);
+	Q_UNUSED(sip);
+	if (m_viewIdentifier != ViewIdentifierClass::SchematicView) return true;
+
+	return false;
+}
+
+QStringList PaletteItem::getPinLabels(bool & hasLocal) {
+	hasLocal = false;
+	QStringList labels;
+	QList<ConnectorItem *> sortedConnectorItems = sortConnectorItems();
+	if (sortedConnectorItems.count() == 0) return labels;
+
+	foreach (ConnectorItem * connectorItem, sortedConnectorItems) {
+		labels.append(connectorItem->connectorSharedName());
+		if (!connectorItem->connector()->connectorLocalName().isEmpty()) {
+			hasLocal = true;
+		}
+	}
+
+	return labels;
+}
+
+bool PaletteItem::loadExtraRenderer(const QString & svg, bool fastLoad) {
+	if (!svg.isEmpty()) {
+		if (m_extraRenderer == NULL) {
+			m_extraRenderer = new FSvgRenderer(this);
+		}
+		//DebugDialog::debug(svg);
+
+		bool result = fastLoad ? m_extraRenderer->fastLoad(svg.toUtf8()) : m_extraRenderer->loadSvgString(svg);
+		if (result) {
+			prepareGeometryChange();
+			setSharedRendererEx(m_extraRenderer);
+		}
+
+		return result;
+	}
+
+	return false;
+}
+
+void PaletteItem::resetConnectors() {
+	if (m_viewIdentifier != ViewIdentifierClass::SchematicView) return;
+
+	QSizeF size = m_extraRenderer->defaultSizeF();   // pixels
+	QRectF viewBox = m_extraRenderer->viewBoxF();
+	foreach (ConnectorItem * connectorItem, cachedConnectorItems()) {
+		SvgIdLayer * svgIdLayer = connectorItem->connector()->fullPinInfo(m_viewIdentifier, m_viewLayerID);
+		if (svgIdLayer == NULL) continue;
+
+		QRectF bounds = this->m_extraRenderer->boundsOnElement(svgIdLayer->m_svgId);
+		QPointF p(bounds.left() * size.width() / viewBox.width(), bounds.top() * size.height() / viewBox.height());
+		QRectF r = connectorItem->rect();
+		r.moveTo(p.x(), p.y());
+		connectorItem->setRect(r);
+	}
+}
+
+
+void PaletteItem::resetConnectors(ItemBase * otherLayer,FSvgRenderer * otherLayerRenderer)
+{
+	// there's only one connector
+	foreach (Connector * connector, m_modelPart->connectors().values()) {
+		if (connector == NULL) continue;
+
+		connector->unprocess(m_viewIdentifier, m_viewLayerID);
+		SvgIdLayer * svgIdLayer = connector->fullPinInfo(m_viewIdentifier, m_viewLayerID);
+		if (svgIdLayer == NULL) continue;
+
+		bool result = m_extraRenderer->setUpConnector(svgIdLayer, false);
+		if (!result) continue;
+
+		resetConnector(this, svgIdLayer);
+	}
+
+	if (otherLayer) {
+		foreach (Connector * connector, m_modelPart->connectors().values()) {
+			if (connector == NULL) continue;
+
+			connector->unprocess(m_viewIdentifier, otherLayer->viewLayerID());
+			SvgIdLayer * svgIdLayer = connector->fullPinInfo(m_viewIdentifier, otherLayer->viewLayerID());
+			if (svgIdLayer == NULL) continue;
+
+			bool result = otherLayerRenderer->setUpConnector(svgIdLayer, false);
+			if (!result) continue;
+
+			resetConnector(otherLayer, svgIdLayer);
+		}
+	}
+
+
+}
+
+void PaletteItem::resetConnector(ItemBase * itemBase, SvgIdLayer * svgIdLayer) 
+{
+	foreach (ConnectorItem * connectorItem, itemBase->cachedConnectorItems()) {
+		//DebugDialog::debug(QString("via set rect %1").arg(itemBase->viewIdentifier()), svgIdLayer->m_rect);
+
+		connectorItem->setRect(svgIdLayer->m_rect);
+		connectorItem->setTerminalPoint(svgIdLayer->m_point);
+		connectorItem->setRadius(svgIdLayer->m_radius, svgIdLayer->m_strokeWidth);
+		connectorItem->attachedMoved();
+		break;
+	}
+}
+
