@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5417 $:
+$Revision: 5952 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-08-18 12:47:41 +0200 (Thu, 18 Aug 2011) $
+$Date: 2012-04-08 00:47:10 +0200 (Sun, 08 Apr 2012) $
 
 ********************************************************************/
 
@@ -38,6 +38,7 @@ $Date: 2011-08-18 12:47:41 +0200 (Thu, 18 Aug 2011) $
 #include "../utils/focusoutcombobox.h"
 #include "../utils/textutils.h"
 #include "../utils/graphicsutils.h"
+#include "../utils/cursormaster.h"
 
 #include <QBrush>
 #include <QPen>
@@ -49,11 +50,15 @@ $Date: 2011-08-18 12:47:41 +0200 (Thu, 18 Aug 2011) $
 #include <QEvent>
 #include <qmath.h>
 
+static QPointF RotationCenter;
+static QPointF RotationAxis;
+static QTransform OriginalTransform;
+
 
 PaletteItemBase::PaletteItemBase(ModelPart * modelPart, ViewIdentifierClass::ViewIdentifier viewIdentifier, const ViewGeometry & viewGeometry, long id, QMenu * itemMenu ) :
 	ItemBase(modelPart, viewIdentifier, viewGeometry, id, itemMenu)
 {
-	m_syncSelected = false;
+	m_inRotation = m_syncSelected = false;
 	m_offset.setX(0);
 	m_offset.setY(0);
  	m_blockItemSelectedChange = false;
@@ -222,6 +227,30 @@ void PaletteItemBase::mousePressEvent(PaletteItemBase * originalItem, QGraphicsS
 {
 	Q_UNUSED(originalItem);
 
+	m_inRotation = false;
+
+	QPointF corner;
+	if (freeRotationAllowed(event->modifiers()) && inRotationLocation(event->scenePos(), event->modifiers(), corner)) {
+		this->saveGeometry();
+		m_inRotation = true;
+		RotationCenter = mapToScene(this->boundingRectWithoutLegs().center());
+		RotationAxis = event->scenePos(); 
+		OriginalTransform = this->transform();
+		/*
+		DebugDialog::debug(QString("%11:in rotation:%1,%2 a:%3,%4 t:%5,%6,%7,%8 %9,%10")
+			.arg(RotationCenter.x()).arg(RotationCenter.y())
+			.arg(RotationAxis.x()).arg(RotationAxis.y())
+			.arg(OriginalTransform.m11()).arg(OriginalTransform.m12()).arg(OriginalTransform.m21()).arg(OriginalTransform.m22())
+			.arg(OriginalTransform.dx()).arg(OriginalTransform.dy())
+			.arg((long) this, 0, 16)
+			);
+			*/
+		this->debugInfo("in rotation");
+		InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
+		if (infoGraphicsView) infoGraphicsView->setAnyInRotation();
+		return;
+	}
+
 	ItemBase::mousePressEvent(event);
 	if (canFindConnectorsUnder()) {
 		foreach (ConnectorItem * connectorItem, cachedConnectorItems()) {
@@ -230,6 +259,73 @@ void PaletteItemBase::mousePressEvent(PaletteItemBase * originalItem, QGraphicsS
 	}
 }
 
+void PaletteItemBase::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
+{
+	if (!m_inRotation) {
+		ItemBase::mouseMoveEvent(event);
+		return;
+	}
+
+	// TODO: doesn't account for scaling
+	// see http://www.gamedev.net/topic/441695-transform-matrix-decomposition/
+	double originalAngle = atan2(OriginalTransform.m12(), OriginalTransform.m11()) * 180 / M_PI;
+	double a1 = atan2(RotationAxis.y() - RotationCenter.y(), RotationAxis.x() - RotationCenter.x());
+	double a2 = atan2(event->scenePos().y() - RotationCenter.y(), event->scenePos().x() - RotationCenter.x());
+
+	double deltaAngle = (a2 - a1) * 180 / M_PI;
+	//DebugDialog::debug(QString("original:%1 delta:%2").arg(originalAngle).arg(deltaAngle));
+	switch (m_viewIdentifier) {
+		case ViewIdentifierClass::BreadboardView:
+		case ViewIdentifierClass::PCBView:
+			{
+				double nearest = qRound((originalAngle + deltaAngle) / 45) * 45;
+				if (qAbs(originalAngle + deltaAngle - nearest) < 6) {
+					deltaAngle = nearest - originalAngle;
+					//DebugDialog::debug(QString("\tdelta angle %1").arg(deltaAngle));
+				}
+			}
+			break;
+		case ViewIdentifierClass::SchematicView:
+			{
+				double nearest = qRound((originalAngle + deltaAngle) / 90) * 90;
+				deltaAngle = nearest - originalAngle;
+			}
+			break;
+		default:
+			return;
+	}
+
+	ItemBase * chief = layerKinChief();
+	// restore viewGeometry to original angle
+	chief->getViewGeometry().setTransform(OriginalTransform);
+	foreach (ItemBase * itemBase, chief->layerKin()) {
+		itemBase->getViewGeometry().setTransform(OriginalTransform);
+	}
+
+	//DebugDialog::debug(QString("rotating item %1 da:%2 oa:%3 %4").arg(QTime::currentTime().toString("HH:mm:ss.zzz")).arg(deltaAngle).arg(originalAngle).arg((long) this, 0, 16));
+	chief->rotateItem(deltaAngle);
+}
+
+void PaletteItemBase::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
+{
+	if (!m_inRotation) {
+		ItemBase::mouseReleaseEvent(event);
+		return;
+	}
+
+	m_inRotation = false;
+	InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
+	if (infoGraphicsView) {
+		// TODO: doesn't account for scaling
+		// see: http://www.gamedev.net/topic/441695-transform-matrix-decomposition/
+		double originalAngle = atan2(OriginalTransform.m12(), OriginalTransform.m11()) * 180 / M_PI;
+		double currentAngle = atan2(transform().m12(), transform().m11()) * 180 / M_PI;
+		rotateItem(originalAngle - currentAngle);		// put it back; undo command will redo it
+		saveGeometry();
+		infoGraphicsView->triggerRotate(this->layerKinChief(), currentAngle - originalAngle);
+	}
+}
+
 bool PaletteItemBase::canFindConnectorsUnder() {
 	return true;
 }
@@ -238,19 +334,18 @@ void PaletteItemBase::findConnectorsUnder() {
 	if (!canFindConnectorsUnder()) return;
 
 	foreach (ConnectorItem * connectorItem, cachedConnectorItems()) {
-		if (connectorItem->connector()->connectorType() == Connector::Female) {
-			// also helps speed things up
-			continue;
+		switch (connectorItem->connector()->connectorType()) {
+			case Connector::Female:
+			case Connector::Pad:
+				continue;
+			default:
+				break;
 		}
 
 		connectorItem->findConnectorUnder(true, false, ConnectorItem::emptyConnectorItemList, false, NULL);
 	}
 }
 
-void PaletteItemBase::mouseReleaseEvent(QGraphicsSceneMouseEvent *event) {
-	ItemBase::mouseReleaseEvent(event);
-}
-
 bool PaletteItemBase::collectFemaleConnectees(QSet<ItemBase *> & items) {
 	bool hasMale = false;
 	foreach (ConnectorItem * item, cachedConnectorItems()) {
@@ -307,7 +402,6 @@ bool PaletteItemBase::setUpImage(ModelPart * modelPart, ViewIdentifierClass::Vie
 
 	if (doConnectors) {
 		setUpConnectors(renderer, modelPart->ignoreTerminalPoints());
-		setConnectorTooltips();
 	}
 
 	if (!m_viewGeometry.transform().isIdentity()) {
@@ -318,9 +412,14 @@ bool PaletteItemBase::setUpImage(ModelPart * modelPart, ViewIdentifierClass::Vie
 	return true;
 }
 
-void PaletteItemBase::setSharedRendererEx(FSvgRenderer * renderer) {
-	setSharedRenderer(renderer);
-	m_size = renderer->defaultSizeF();
+void PaletteItemBase::setSharedRendererEx(FSvgRenderer * newRenderer) {
+	if (newRenderer != renderer()) {
+		setSharedRenderer(newRenderer);
+	}
+	else {
+		update();
+	}
+	m_size = newRenderer->defaultSizeF();
 }
 
 void PaletteItemBase::setUpConnectors(FSvgRenderer * renderer, bool ignoreTerminalPoints) {
@@ -397,6 +496,16 @@ void PaletteItemBase::connectedMoved(ConnectorItem * from, ConnectorItem * to) {
 	// female connectors really only operate in breadboard view
 	if (m_viewIdentifier != ViewIdentifierClass::BreadboardView) return;
 
+	QTransform t = this->transform();
+	if (t.isRotating()) {
+		// map a square and see if it's a diamond
+		QRectF test(0, 0, 10, 10);
+		QRectF result = t.mapRect(test);
+		if (result.width() != test.width() || result.height() != test.height()) {
+			return;
+		}
+	}
+
 	QPointF fromTerminalPoint = from->sceneAdjustedTerminalPoint(to);
 	QPointF toTerminalPoint = to->sceneAdjustedTerminalPoint(from);
 
@@ -417,7 +526,7 @@ void PaletteItemBase::hoverEnterEvent ( QGraphicsSceneHoverEvent * event ) {
 	if (event->isAccepted()) {
 		if (hasRubberBandLeg()) {
 			//DebugDialog::debug("---pab set override cursor");
-			QApplication::setOverrideCursor(cursor());
+			CursorMaster::instance()->addCursor(this, cursor());
 
 			bool connected = false;
 			foreach (ConnectorItem * connectorItem, cachedConnectorItems()) {
@@ -426,39 +535,42 @@ void PaletteItemBase::hoverEnterEvent ( QGraphicsSceneHoverEvent * event ) {
 					break;
 				}
 			}
-			if (connected) {
-				QApplication::instance()->installEventFilter(this);
-			}
 		}
+
+		checkFreeRotation(event->modifiers(), event->scenePos());
 	}
+
+}
+
+void PaletteItemBase::hoverMoveEvent ( QGraphicsSceneHoverEvent * event ) {
+	checkFreeRotation(event->modifiers(), event->scenePos());
 }
 
 void PaletteItemBase::hoverLeaveEvent ( QGraphicsSceneHoverEvent * event ) {
 	if (hasRubberBandLeg()) {
-		QApplication::instance()->removeEventFilter(this);
 		//DebugDialog::debug("------pab restore override cursor");
-		QApplication::restoreOverrideCursor();
 	}
 
+	CursorMaster::instance()->removeCursor(this);
+
 	ItemBase::hoverLeaveEvent(event);
 }
 
-bool PaletteItemBase::eventFilter(QObject * object, QEvent * event)
+void PaletteItemBase::cursorKeyEvent(Qt::KeyboardModifiers modifiers)
 {
-	Q_UNUSED(object);
+	InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
+	QPoint p = infoGraphicsView->mapFromGlobal(QCursor::pos());
+	checkFreeRotation(modifiers, infoGraphicsView->mapToScene(p));
 	if (hasRubberBandLeg()) {
-		if (event->type() == QEvent::KeyPress || event->type() == QEvent::KeyRelease) {
-			QKeyEvent *keyEvent = static_cast<QKeyEvent*>(event);
-			if (keyEvent->modifiers() & altOrMetaModifier()) {
-				QApplication::changeOverrideCursor(*ConnectorItem::RubberbandCursor);
-			}
-			else {
-				QApplication::changeOverrideCursor(*ConnectorItem::MoveCursor);
-			}
+		QCursor cursor;
+		if (modifiers & altOrMetaModifier()) {
+			cursor = *CursorMaster::RubberbandCursor;
 		}
+		else {
+			cursor = *CursorMaster::MoveCursor;
+		}
+		CursorMaster::instance()->addCursor(this, cursor);
 	}
-
-	return false;
 }
 
 void PaletteItemBase::contextMenuEvent(QGraphicsSceneContextMenuEvent *event) {
@@ -569,7 +681,6 @@ void PaletteItemBase::setProp(const QString & prop, const QString & value)
 	ItemBase::setProp(prop, value);
 }
 
-
 void PaletteItemBase::partPropertyEntry() {
 	QLineEdit * lineEdit = qobject_cast<QLineEdit *>(sender());
 	if (lineEdit == NULL) return;
@@ -587,15 +698,65 @@ const QCursor * PaletteItemBase::getCursor(Qt::KeyboardModifiers modifiers)
 		if ((modifiers & altOrMetaModifier())) {
 			foreach (ConnectorItem * connectorItem, cachedConnectorItems()) {
 				if (connectorItem->connectionsCount() > 0) {
-					return ConnectorItem::RubberbandCursor;
+					return CursorMaster::RubberbandCursor;
 				}
 			}
 		}
 	}
 
-	return ConnectorItem::MoveCursor;
+	//DebugDialog::debug("returning move cursor");
+	return CursorMaster::MoveCursor;
 }
 
+bool PaletteItemBase::freeRotationAllowed(Qt::KeyboardModifiers modifiers) {
+	Q_UNUSED(modifiers);
+	//if ((modifiers & altOrMetaModifier()) == 0) return false;
+	if (!isSelected()) return false;
+	if (this->moveLock()) return false;
+
+	return rotation45Allowed();
+}
+
+bool PaletteItemBase::inRotationLocation(QPointF scenePos, Qt::KeyboardModifiers modifiers, QPointF & returnPoint)
+{
+	if (!freeRotationAllowed(modifiers)) return false;
+	if (m_viewIdentifier == ViewIdentifierClass::SchematicView) return false;
+
+	QRectF r = this->boundingRectWithoutLegs();
+	QPolygonF polygon;
+	polygon.append(mapToScene(r.topLeft()));
+	polygon.append(mapToScene(r.topRight()));
+	polygon.append(mapToScene(r.bottomRight()));
+	polygon.append(mapToScene(r.bottomLeft()));
+	foreach (QPointF p, polygon) {
+		double dsqd = GraphicsUtils::distanceSqd(p, scenePos);
+		if (dsqd < 9) {
+			returnPoint = p;
+			return true;
+		}
+	}
+
+	return false;
+}
+
+bool PaletteItemBase::inRotation() {
+	return m_inRotation;
+}
+
+void PaletteItemBase::checkFreeRotation(Qt::KeyboardModifiers modifiers, QPointF scenePos)
+{
+	if (!freeRotationAllowed(modifiers)) return;
+
+	QPointF returnPoint;
+	bool inCorner = inRotationLocation(scenePos, modifiers, returnPoint);
+
+	if (inCorner) {
+		CursorMaster::instance()->addCursor(this, *CursorMaster::RotateCursor);
+	}
+	else {
+		CursorMaster::instance()->addCursor(this, cursor());
+	}
+}
 
 /*
 
