@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5417 $:
+$Revision: 5832 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-08-18 12:47:41 +0200 (Thu, 18 Aug 2011) $
+$Date: 2012-02-03 16:20:27 +0100 (Fri, 03 Feb 2012) $
 
 ********************************************************************/
 
@@ -39,7 +39,6 @@ $Date: 2011-08-18 12:47:41 +0200 (Thu, 18 Aug 2011) $
 #include <QMenu>
 #include <QApplication>
 #include <QInputDialog>
-#include <QFontMetricsF>
 #include <QStringList>
 #include <QFont>
 #include <QSvgRenderer>
@@ -384,6 +383,7 @@ void PartLabel::restoreLabel(QDomElement & labelGeometry, ViewLayer::ViewLayerID
 	//setBrush(QBrush(c));
 
 	setUpText();
+	m_initialized = true;
 	double fs = labelGeometry.attribute("fontSize").toDouble(&ok);
 	if (!ok) {
 		InfoGraphicsView *infographics = InfoGraphicsView::getInfoGraphicsView(this);
@@ -408,6 +408,8 @@ void PartLabel::restoreLabel(QDomElement & labelGeometry, ViewLayer::ViewLayerID
 		if (m_owner->hasPartNumberProperty()) m_displayKeys.append(ModelPartShared::PartNumberPropertyName);
 	}
 
+	displayTexts();
+
 	QTransform t;
 	if (GraphicsUtils::loadTransform(labelGeometry.firstChildElement("transform"), t)) {
 		setTransform(t);
@@ -547,6 +549,7 @@ void PartLabel::rotateFlipLabel(double degrees, Qt::Orientations orientation) {
 			yScale = 1;
 		}
 		else return;
+
 		transformLabel(QTransform().scale(xScale,yScale));
 	}
 }
@@ -824,9 +827,16 @@ QString mapToSVGStyle(QFont::Style style) {
 	}
 }
 
-QString PartLabel::makeSvg(bool blackOnly, double dpi, double printerScale) {
+QString PartLabel::makeSvg(bool blackOnly, double dpi, double printerScale, bool includeTransform) {
 	double w, h;
-	return makeSvgAux(blackOnly, dpi, printerScale, w, h);
+	QString svg = makeSvgAux(blackOnly, dpi, printerScale, w, h);
+	if (includeTransform) {
+		QTransform t = transform();
+		if (!t.isIdentity()) {
+			svg = TextUtils::svgTransform(svg, t, false, QString());
+		}
+	}
+	return svg;
 }
 
 
@@ -850,20 +860,20 @@ QString PartLabel::makeSvgAux(bool blackOnly, double dpi, double printerScale, d
 	w = 0;
 	QStringList texts = m_displayText.split("\n");
 	foreach (QString t, texts) {
+		QString t1 = TextUtils::convertExtendedChars(TextUtils::escapeAnd(t));
 		svg += QString("<text x='0' y='%1'>%2</text>")
 			.arg(y * dpi / printerScale)
-			.arg(TextUtils::stripNonValidXMLCharacters(TextUtils::escapeAnd(t)));
+			.arg(t1);
 		y += pixels;
 		w = qMax(w, t.length() * pixels * 0.75);
 		//DebugDialog::debug(QString("\t%1, %2").arg(w).arg(y));
 	}
 
 	svg += "</g>";
-    QTransform t = transform();
 
 	h = y - (pixels / 2);
 
-	QFontInfo fontInfo(m_font);
+	//QFontInfo fontInfo(m_font);
 	//DebugDialog::debug(QString("%1 match:%2 ps:%3 sty:%4 w:%5")
 	//	.arg(fontInfo.family())
 	//	.arg(fontInfo.exactMatch())
@@ -872,23 +882,25 @@ QString PartLabel::makeSvgAux(bool blackOnly, double dpi, double printerScale, d
 	//	.arg(fontInfo.weight())
 	//	);
 
+	return svg;
+
 	//DebugDialog::debug(QString("final:%1 %2 %3").arg(w).arg(h).arg(m_font.toString()));
-    return TextUtils::svgTransform(svg, t, false, QString());
+    
 }
 
 void PartLabel::resetSvg()
 {
+
 	double w, h;
-	QString innerSvg = makeSvgAux(false, 1000, FSvgRenderer::printerScale(), w, h);
+	QString innerSvg = makeSvgAux(false, GraphicsUtils::StandardFritzingDPI, FSvgRenderer::printerScale(), w, h);
 	if (innerSvg.isEmpty()) return;
 
-	QString svg = TextUtils::makeSVGHeader(FSvgRenderer::printerScale(), 1000, w, h) + innerSvg + "\n</svg>";
+	QString svg = TextUtils::makeSVGHeader(FSvgRenderer::printerScale(), GraphicsUtils::StandardFritzingDPI, w, h) + innerSvg + "\n</svg>";
 
 	if (m_renderer == NULL) {
 		m_renderer = new QSvgRenderer(this);
 	}
 
-
 	// using renderer()->load() doesn't seem to work, so keep a separate shared renderer as a workaround
 	bool loaded = m_renderer->load(svg.toUtf8());
 	if (loaded) {
