@@ -18,15 +18,16 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5401 $:
+$Revision: 5653 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-08-16 19:25:49 +0200 (Tue, 16 Aug 2011) $
+$Date: 2011-12-01 17:19:24 +0100 (Thu, 01 Dec 2011) $
 
 ********************************************************************/
 
 #include "stripboard.h"
 #include "../utils/graphicsutils.h"
 #include "../utils/textutils.h"
+#include "../svg/gerbergenerator.h"
 #include "../fsvgrenderer.h"
 #include "../sketch/infographicsview.h"
 #include "moduleidnames.h"
@@ -284,7 +285,7 @@ Stripboard::Stripboard( ModelPart * modelPart, ViewIdentifierClass::ViewIdentifi
 	foreach (Connector * connector, modelPart->connectors().values()) {
 		ConnectorShared * connectorShared = connector->connectorShared();
 		int cx, cy;
-		getXY(cx, cy, connectorShared->name());
+		getXY(cx, cy, connector->connectorSharedName());
 		BusShared * busShared = m_buses.at(cy);
 		busShared->addConnectorShared(connectorShared);
 	}
@@ -299,7 +300,7 @@ QString Stripboard::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QStrin
 {
 	QString svg = Perfboard::retrieveSvg(viewLayerID, svgHash, blackOnly, dpi);
 	if (svg.isEmpty()) return svg;
-	if (!svg.contains("boardoutline")) return svg;
+	if (!svg.contains(GerberGenerator::MagicBoardOutlineID)) return svg;
 
 	/*
 	QFile file(filename());
@@ -433,7 +434,7 @@ void Stripboard::addedToScene(bool temporary)
 		}
 	}
 
-	QString config = modelPart()->prop("buses").toString();
+	QString config = prop("buses");
 	if (config.isEmpty()) return;
 
 	QStringList removed = config.split(" ", QString::SkipEmptyParts);
