@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5417 $:
+$Revision: 5895 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-08-18 12:47:41 +0200 (Thu, 18 Aug 2011) $
+$Date: 2012-03-06 01:43:16 +0100 (Tue, 06 Mar 2012) $
 
 ********************************************************************/
 
@@ -87,12 +87,12 @@ later:
 #include "../utils/textutils.h"
 #include "../utils/bezier.h"
 #include "../utils/bezierdisplay.h"
+#include "../utils/cursormaster.h"
+#include "../utils/ratsnestcolors.h"
 #include "../layerattributes.h"
 
 #include <stdlib.h>
 
-QHash<QString, QString> Wire::colors;
-QHash<QString, QString> Wire::shadowColors;
 QHash<QString, QString> Wire::colorTrans;
 QStringList Wire::colorNames;
 QHash<int, QString> Wire::widthTrans;
@@ -115,22 +115,6 @@ bool alphaLessThan(QColor * c1, QColor * c2)
 
 /////////////////////////////////////////////////////////////
 
-
-WireMenu::WireMenu(const QString & title, QWidget * parent) : QMenu(title, parent) 
-{
-	m_wire = NULL;
-}
-
-void WireMenu::setWire(Wire * w) {
-	m_wire = w;
-}
-
-Wire * WireMenu::wire() {
-	return m_wire;
-}
-
-/////////////////////////////////////////////////////////////
-
 WireAction::WireAction(QAction * action) : QAction(action) {
 	m_wire = NULL;
 	this->setText(action->text());
@@ -189,7 +173,6 @@ FSvgRenderer * Wire::setUp(ViewLayer::ViewLayerID viewLayerID, const LayerHash &
 	FSvgRenderer * svgRenderer = setUpConnectors(m_modelPart, m_viewIdentifier);
 	if (svgRenderer != NULL) {
 		initEnds(m_viewGeometry, svgRenderer->viewBox(), infoGraphicsView);
-		setConnectorTooltips();
 	}
 	setZValue(this->z());
 
@@ -314,6 +297,7 @@ void Wire::paintBody(QPainter * painter, const QStyleOptionGraphicsItem * option
 		painter->restore();
 	}
 	   
+	// DebugDialog::debug(QString("pen width %1 %2").arg(m_pen.widthF()).arg(m_viewIdentifier));
 	painter->setPen(m_pen);
 	if (painterPath.isEmpty()) {
 		painter->drawLine(getPaintLine());	
@@ -388,12 +372,18 @@ void Wire::mouseDoubleClickEvent(QGraphicsSceneMouseEvent *event) {
 	emit wireSplitSignal(this, event->scenePos(), this->pos(), this->line());
 }
 
+void Wire::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
+{
+	InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
+	if (infoGraphicsView != NULL) {
+		infoGraphicsView->setActiveWire(this);
+	}
+
+	ItemBase::contextMenuEvent(event);
+}
+
 void Wire::mousePressEvent(QGraphicsSceneMouseEvent *event)
 {
-	WireMenu * wireMenu = qobject_cast<WireMenu *>(m_itemMenu);
-	if (wireMenu) {
-		wireMenu->setWire(this);
-	}
 	ItemBase::mousePressEvent(event);
 }
 
@@ -476,7 +466,6 @@ void Wire::mouseMoveConnectorEvent(ConnectorItem * connectorItem, QGraphicsScene
 	mouseMoveEventAux(this->mapFromItem(connectorItem, event->pos()), event->modifiers());
 }
 
-
 void Wire::mouseMoveEvent(QGraphicsSceneMouseEvent *event) {
 	mouseMoveEventAux(event->pos(), event->modifiers());
 }
@@ -603,18 +592,28 @@ void Wire::mouseMoveEventAux(QPointF eventPos, Qt::KeyboardModifiers modifiers)
 
 void Wire::setConnector0Rect() {
 	QRectF rect = m_connector0->rect();
-	rect.moveTo(0 - (rect.width()  / 2.0)  ,
+	rect.moveTo(0 - (rect.width()  / 2.0),
 				0 - (rect.height()  / 2.0) );
-	//DebugDialog::debug(QString("set connector rect %1 %2").arg(rect.width()).arg(rect.height()));
 	m_connector0->setRect(rect);
+
+//	QPointF p = m_connector0->mapToScene(m_connector0->rect().center());
+//	m_connector0->debugInfo(QString("c0:%1 %2").arg(p.x()).arg(p.y()));
+//	p = m_connector1->mapToScene(m_connector1->rect().center());
+//	m_connector1->debugInfo(QString("c1:%1 %2").arg(p.x()).arg(p.y()));
+
 }
 
 
 void Wire::setConnector1Rect() {
 	QRectF rect = m_connector1->rect();
-	rect.moveTo(this->line().dx() - (rect.width()  / 2.0)  ,
+	rect.moveTo(this->line().dx() - (rect.width()  / 2.0),
 				this->line().dy() - (rect.height()  / 2.0) );
 	m_connector1->setRect(rect);
+
+//	QPointF p = m_connector0->mapToScene(m_connector0->rect().center());
+//	m_connector0->debugInfo(QString("c0:%1 %2").arg(p.x()).arg(p.y()));
+//	p = m_connector1->mapToScene(m_connector1->rect().center());
+//	m_connector1->debugInfo(QString("c1:%1 %2").arg(p.x()).arg(p.y()));
 }
 
 void Wire::mouseReleaseEvent(QGraphicsSceneMouseEvent *event) {
@@ -681,7 +680,7 @@ void Wire::writeGeometry(QXmlStreamWriter & streamWriter) {
 	streamWriter.writeEndElement();
 }
 
-void Wire::setExtras(QDomElement & element, InfoGraphicsView * infoGraphicsView)
+void Wire::setExtras(const QDomElement & element, InfoGraphicsView * infoGraphicsView)
 {
 	if (element.isNull()) return;
 
@@ -712,7 +711,7 @@ void Wire::setExtras(QDomElement & element, InfoGraphicsView * infoGraphicsView)
 
 }
 
-void Wire::setColorFromElement(QDomElement & element) {
+void Wire::setColorFromElement(const QDomElement & element) {
 	QString colorString = element.attribute("color");
 	if (colorString.isNull() || colorString.isEmpty()) return;
 
@@ -737,17 +736,15 @@ void Wire::hoverLeaveConnectorItem(QGraphicsSceneHoverEvent * event, ConnectorIt
 
 void Wire::hoverEnterEvent ( QGraphicsSceneHoverEvent * event ) {
 	ItemBase::hoverEnterEvent(event);
-	QApplication::instance()->installEventFilter(this);
-	QApplication::setOverrideCursor(cursor());
+	CursorMaster::instance()->addCursor(this, cursor());
 	//DebugDialog::debug("---wire set override cursor");
 	updateCursor(event->modifiers());
 }
 
 void Wire::hoverLeaveEvent ( QGraphicsSceneHoverEvent * event ) {
-	QApplication::instance()->removeEventFilter(this);
 	ItemBase::hoverLeaveEvent(event);
 	//DebugDialog::debug("------wire restore override cursor");
-	QApplication::restoreOverrideCursor();
+	CursorMaster::instance()->removeCursor(this);
 }
 
 
@@ -785,8 +782,6 @@ void Wire::mouseDoubleClickConnectorEvent(ConnectorItem * connectorItem) {
 
 	if (chained == 1) {
 		// near as I can tell, this is to eliminate the overrides from the connectorItem and then from the wire itself
-		QApplication::restoreOverrideCursor();
-		QApplication::restoreOverrideCursor();
 		emit wireJoinSignal(this, connectorItem);
 	}
 }
@@ -931,7 +926,7 @@ FSvgRenderer * Wire::setUpConnectors(ModelPart * modelPart, ViewIdentifierClass:
 		return NULL;
 	}
 
-	foreach (Connector * connector, m_modelPart->connectors().values()) {
+	foreach (Connector * connector, modelPart->connectors().values()) {
 		if (connector == NULL) continue;
 
 		SvgIdLayer * svgIdLayer = connector->fullPinInfo(viewIdentifier, m_viewLayerID);
@@ -982,6 +977,7 @@ ConnectorItem * Wire::connector1() {
 
 void Wire::findConnectorsUnder() {
 	foreach (ConnectorItem * connectorItem, cachedConnectorItems()) {
+		if (connectorItem->connectionsCount() > 0) continue;  // only check free ends
 		connectorItem->findConnectorUnder(true, false, ConnectorItem::emptyConnectorItemList, false, NULL);
 	}
 }
@@ -1032,7 +1028,16 @@ void Wire::collectWiresAux(QList<Wire *> & wires, ConnectorItem * start) {
 
 bool Wire::stickyEnabled()
 {
-	return (connector0()->connectionsCount() <= 0) && (connector1()->connectionsCount() <= 0);
+	QList<Wire *> wires;
+	QList<ConnectorItem *> ends;
+	this->collectChained(wires, ends);
+	foreach (ConnectorItem * connector, ends) {
+		if (connector->connectionsCount() > 0) {
+			return false;
+		}
+	}
+
+	return true;
 }
 
 bool Wire::getTrace() {
@@ -1118,16 +1123,9 @@ void Wire::setColorString(QString colorName, double op) {
 	// sets a color using the name (.e. "red")
 	// note: colorName is associated with a Fritzing color, not a Qt color
 
-	QString colorString = colors.value(colorName, "");
+	QString colorString = RatsnestColors::wireColor(m_viewIdentifier, colorName);     
 	if (colorString.isEmpty()) {
 		colorString = colorName;
-
-		foreach (QString c, colors.keys()) {
-			if (colors.value(c).compare(colorName, Qt::CaseInsensitive) == 0) {
-				colorName = c;
-				break;
-			}
-		}
 	}
 
 	QColor c;
@@ -1135,12 +1133,12 @@ void Wire::setColorString(QString colorName, double op) {
 	setColor(c, op);
 	m_colorName = colorName;
 
-	colorString = shadowColors.value(colorName, "");
-	if (colorString.isEmpty()) {
-		colorString = colorName;
+	QString shadowColorString = RatsnestColors::shadowColor(m_viewIdentifier, colorName);
+	if (shadowColorString.isEmpty()) {
+		shadowColorString = colorString;
 	}
 
-	c.setNamedColor(colorString);
+	c.setNamedColor(shadowColorString);
 	setShadowColor(c);
 }
 
@@ -1157,7 +1155,7 @@ QString Wire::colorString() {
 }
 
 void Wire::initNames() {
-	if (colors.count() > 0) return;
+	if (colorNames.count() > 0) return;
 
 	widths << 16 << 24 << 32 << 48;
 	widthTrans.insert(widths[0], tr("thin (16 mil)"));
@@ -1180,7 +1178,6 @@ void Wire::initNames() {
 	colorNames.append(tr("orange"));
     colorNames.append(tr("brown"));
     colorNames.append(tr("purple"));
-    colorNames.append(tr("schematic black"));
 
 	// need this hash table to translate from user's language to internal color name
     colorTrans.insert(tr("blue"), "blue");
@@ -1193,43 +1190,6 @@ void Wire::initNames() {
 	colorTrans.insert(tr("orange"), "orange");
 	colorTrans.insert(tr("brown"), "brown");
     colorTrans.insert(tr("purple"), "purple");
-    colorTrans.insert(tr("schematic black"), "schematic black");
-
-    colors.insert("blue",	"#418dd9");
-	colors.insert("red",	"#cc1414");
-    colors.insert("black",	"#404040");
-	colors.insert("yellow", "#ffe24d");
-	colors.insert("green",	"#47cc79");
-	colors.insert("grey",	"#999999");
-	colors.insert("white",	"#ffffff");
-	colors.insert("orange", "#ff7033");
-	colors.insert("jumper", ViewLayer::JumperColor);
-	colors.insert("trace",  ViewLayer::Copper0Color);    
-	colors.insert("trace1",  ViewLayer::Copper1Color);    
-	//colors.insert("unrouted", "#000000");
-	//colors.insert("blackblack", "#000000");
-	colors.insert("schematicGrey", "#9d9d9d");
-    colors.insert("purple", "#ab58a2");
-	colors.insert("brown", "#8c3b00");
-	colors.insert("schematic black", "#000000");
-
-    shadowColors.insert("blue",		"#1b5bb3");
-	shadowColors.insert("red",		"#8c0000");
-    shadowColors.insert("black",	"#000000");
-	shadowColors.insert("yellow",	"#e6ab00");
-	shadowColors.insert("green",	"#00a63d");
-	shadowColors.insert("grey",		"#666666");
-	shadowColors.insert("white",	"#999999");
-	shadowColors.insert("orange",	"#d95821");
-    shadowColors.insert("jumper",	"#2d6563");
-	shadowColors.insert("trace",	"#d69b00");
-	shadowColors.insert("trace1",   "#d69b00");    
-	//shadowColors.insert("unrouted", "#000000");
-    shadowColors.insert("purple",	"#7a3a73");
-    shadowColors.insert("brown",	"#6c2710");
-	shadowColors.insert("schematicGrey", "#1d1d1d");
-	//shadowColors.insert("blackblack", "#000000");
-	shadowColors.insert("schematic black", "#000000");
 }
 
 bool Wire::hasFlag(ViewGeometry::WireFlag flag)
@@ -1237,6 +1197,10 @@ bool Wire::hasFlag(ViewGeometry::WireFlag flag)
 	return m_viewGeometry.hasFlag(flag);
 }
 
+bool Wire::isTraceType(ViewGeometry::WireFlag flag) {
+	return hasFlag(flag);
+}
+
 bool Wire::hasAnyFlag(ViewGeometry::WireFlags flags)
 {
 	return m_viewGeometry.hasAnyFlag(flags);
@@ -1253,13 +1217,6 @@ Wire * Wire::findTraced(ViewGeometry::WireFlags flags, QList<ConnectorItem *>  &
 	return ConnectorItem::directlyWiredTo(ends[0], ends[1], flags);
 }
 
-QRgb Wire::getRgb(const QString & name) {
-	QString str = colors.value(name);
-	QColor c;
-	c.setNamedColor(str);
-	return c.rgb();
-}
-
 void Wire::setWireFlags(ViewGeometry::WireFlags wireFlags) {
 	m_viewGeometry.setWireFlags(wireFlags);
 }
@@ -1345,23 +1302,34 @@ QVariant Wire::itemChange(GraphicsItemChange change, const QVariant &value)
 void Wire::cleanup() {
 }
 
-void Wire::getConnectedColor(ConnectorItem * connectorItem, QBrush * &brush, QPen * &pen, double & opacity, double & negativePenWidth, bool & negativeOffsetRect) {
-
+void Wire::getConnectedColor(ConnectorItem * connectorItem, QBrush * &brush, QPen * &pen, double & opacity, double & negativePenWidth, bool & negativeOffsetRect) 
+{
 	connectorItem->setBigDot(false);
 	int count = 0;
 	bool bendpoint = true;
+	InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
+	if (infoGraphicsView == NULL) {
+		ItemBase::getConnectedColor(connectorItem, brush, pen, opacity, negativePenWidth, negativeOffsetRect);
+		return;
+	}
+
 	foreach (ConnectorItem * toConnectorItem, connectorItem->connectedToItems()) {
-		if (toConnectorItem->attachedToItemType() != ModelPart::Wire) {
+		if (toConnectorItem->attachedToItemType() == ModelPart::Wire) {
+			Wire * w = qobject_cast<Wire *>(toConnectorItem->attachedTo());
+			if (w->isTraceType(infoGraphicsView->getTraceFlag())) {
+				count++;
+			}
+		}
+		else {
 			// for drawing a big dot on the end of a part connector in schematic view if the part is connected to more than one trace
 			bendpoint = false;
 			if (toConnectorItem->connectionsCount() > 1) {	
-				InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
-				if (infoGraphicsView != NULL && infoGraphicsView->hasBigDots()) {
+				if (infoGraphicsView->hasBigDots()) {
 					int c = 0;
 					foreach (ConnectorItem * totoConnectorItem, toConnectorItem->connectedToItems()) {
 						if (totoConnectorItem->attachedToItemType() == ModelPart::Wire) {
 							Wire * w = qobject_cast<Wire *>(totoConnectorItem->attachedTo());
-							if (w && w->getTrace()) {
+							if (w && w->isTraceType(ViewGeometry::SchematicTraceFlag) && w->isTraceType(infoGraphicsView->getTraceFlag())) {
 								c++;
 							}
 						}
@@ -1373,11 +1341,9 @@ void Wire::getConnectedColor(ConnectorItem * connectorItem, QBrush * &brush, QPe
 				}
 			}
 
-			ItemBase::getConnectedColor(connectorItem, brush, pen, opacity, negativePenWidth, negativeOffsetRect);
-			return;
+			count = 0;
+			break;
 		}
-
-		count++;
 	}
 
 	if (count == 0) {
@@ -1467,11 +1433,14 @@ bool Wire::collectExtraInfo(QWidget * parent, const QString & family, const QStr
 			QString englishCurrColor = colorString();
 			foreach(QString transColorName, Wire::colorNames) {
 				QString englishColorName = Wire::colorTrans.value(transColorName);
-				comboBox->addItem(transColorName, QVariant(englishColorName));
-				if (englishColorName.compare(englishCurrColor, Qt::CaseInsensitive) == 0) {
-					comboBox->setCurrentIndex(ix);
+				bool ok = (this->m_viewIdentifier != ViewIdentifierClass::SchematicView || englishColorName.compare("white", Qt::CaseInsensitive) != 0);
+				if (ok) {
+					comboBox->addItem(transColorName, QVariant(englishColorName));
+					if (englishColorName.compare(englishCurrColor, Qt::CaseInsensitive) == 0) {
+						comboBox->setCurrentIndex(ix);
+					}
+					ix++;
 				}
-				ix++;
 			}
 
 			connect(comboBox, SIGNAL(currentIndexChanged(const QString &)), this, SLOT(colorEntry(const QString &)));
@@ -1703,23 +1672,17 @@ bool Wire::hasShadow() {
 	return m_pen.widthF() != m_shadowPen.widthF();
 }
 
-bool Wire::eventFilter(QObject * object, QEvent * event)
+void Wire::cursorKeyEvent(Qt::KeyboardModifiers modifiers)
 {
-	Q_UNUSED(object);
-	if (!(m_dragEnd || m_dragCurve)) {
-		if (event->type() == QEvent::KeyPress || event->type() == QEvent::KeyRelease) {
-			InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);;
-			if (infoGraphicsView) {
-				QPoint p = infoGraphicsView->mapFromGlobal(QCursor::pos());
-				QPointF r = infoGraphicsView->mapToScene(p);
-				QKeyEvent *keyEvent = static_cast<QKeyEvent*>(event);
-				// DebugDialog::debug(QString("got key event %1").arg(keyEvent->modifiers()));
-				updateCursor(keyEvent->modifiers());
-			}
-		}
-	}
+	if (m_dragEnd || m_dragCurve) return;
 
-	return false;
+	InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);;
+	if (infoGraphicsView) {
+		QPoint p = infoGraphicsView->mapFromGlobal(QCursor::pos());
+		QPointF r = infoGraphicsView->mapToScene(p);
+		// DebugDialog::debug(QString("got key event %1").arg(keyEvent->modifiers()));
+		updateCursor(modifiers);
+	}
 }
 
 void Wire::updateCursor(Qt::KeyboardModifiers modifiers)
@@ -1752,17 +1715,17 @@ void Wire::updateCursor(Qt::KeyboardModifiers modifiers)
 		
 	if (segment) {
 		// dragging a segment of wire between bounded by two other wires
-		QApplication::changeOverrideCursor(*ConnectorItem::RubberbandCursor);
+		CursorMaster::instance()->addCursor(this, *CursorMaster::RubberbandCursor);
 	}
 	else if (totalConnections == 0) {
 		// only in breadboard view
-		QApplication::changeOverrideCursor(*ConnectorItem::MoveCursor);
+		CursorMaster::instance()->addCursor(this, *CursorMaster::MoveCursor);
 	}
 	else if (infoGraphicsView != NULL && infoGraphicsView->curvyWiresIndicated(modifiers)) {
-		QApplication::changeOverrideCursor(*ConnectorItem::MakeCurveCursor);
+		CursorMaster::instance()->addCursor(this, *CursorMaster::MakeCurveCursor);
 	}
 	else {
-		QApplication::changeOverrideCursor(*ConnectorItem::NewBendpointCursor);
+		CursorMaster::instance()->addCursor(this, *CursorMaster::NewBendpointCursor);
 	}
 }
 
