@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5417 $:
+$Revision: 5961 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-08-18 12:47:41 +0200 (Thu, 18 Aug 2011) $
+$Date: 2012-04-09 02:13:59 +0200 (Mon, 09 Apr 2012) $
 
 ********************************************************************/
 
@@ -82,12 +82,12 @@ class MainWindow : public FritzingWindow
 {
     Q_OBJECT
 public:
-    MainWindow(class PaletteModel *, ReferenceModel *refModel);
+    MainWindow(class PaletteModel *, ReferenceModel *refModel, QWidget * parent);
     MainWindow(QFile & fileToLoad);
 	~MainWindow();
 
-    void load(const QString & fileName, bool setAsLastOpened, bool addToRecent, const QString & displayName);
-	bool loadWhich(const QString & fileName, bool setAsLastOpened, bool addToRecent, bool dontAsk);
+    void mainLoad(const QString & fileName, const QString & displayName);
+	bool loadWhich(const QString & fileName, bool setAsLastOpened, bool addToRecent, const QString & displayName);
 	void notClosableForAWhile();
 	QAction *raiseWindowAction();
 	QSizeGrip *sizeGrip();
@@ -98,12 +98,11 @@ public:
 	class PartsEditorMainWindow* getPartsEditor(ModelPart *modelPart, long id, ItemBase * fromItem, class PartsBinPaletteWidget* requester);
 	ModelPart *loadPartFromFile(const QString& newPartPath, bool connectorsChanged=false);
 	void addDefaultParts();
-	void init();
+    void init(class PaletteModel *, ReferenceModel *refModel, bool lockFiles);
 	void showFileProgressDialog(const QString & path);
 	void setFileProgressPath(const QString & path);
 	void clearFileProgressDialog();
 	class FileProgressDialog * fileProgressDialog();
-	void fileProgressDialogSetBinLoadingCount(int);
 
 	const QString &selectedModuleID();
 
@@ -115,19 +114,28 @@ public:
 	// if we consider a part as the smallest ("atomic") entity inside
 	// fritzing, then this functions may help with the bundle tasks
 	// on the complex entities: sketches, bins, modules (?)
-	void saveBundledNonAtomicEntity(QString &filename, const QString &extension, Bundler *bundler, const QList<ModelPart*> &partsToSave);
+	void saveBundledNonAtomicEntity(QString &filename, const QString &extension, Bundler *bundler, const QList<ModelPart*> &partsToSave, bool askForFilename, const QString & destFolderPath, bool saveModel, bool deleteLeftovers);
 	void loadBundledNonAtomicEntity(const QString &filename, Bundler *bundler, bool addToBin, bool dontAsk);
+	void saveAsShareable(const QString & path, bool saveModel);
+
 	
-	void setCurrentFile(const QString &fileName, bool addToRecent, bool recovered, const QString & backupName);
-	void setRecovered(bool);
+	void setCurrentFile(const QString &fileName, bool addToRecent, bool setAsLastOpened);
 	void setReportMissingModules(bool);
 	QList<SketchWidget *> sketchWidgets();
 	void setCloseSilently(bool);
 	void exportToGerber(const QString & outputDir);
+	class PCBSketchWidget * pcbView();
+	void noBackup();
+	void swapSelectedAux(ItemBase * itemBase, const QString & moduleID);
+	bool saveAsAux(const QString & fileName);
+	void swapObsolete(bool displayFeedback);
+	void selectAllObsolete(bool displayFeedback);
+	void hideTempBin();
+	const QString & fritzingVersion();
 
 public:
 	static void initNames();
-	static MainWindow * newMainWindow(PaletteModel * paletteModel, ReferenceModel *refModel, const QString & displayPath, bool showProgress);
+	static MainWindow * newMainWindow(PaletteModel * paletteModel, ReferenceModel *refModel, const QString & displayPath, bool showProgress, bool lockFiles);
 	static void setAutosavePeriod(int);
 	static void setAutosaveEnabled(bool);
 
@@ -147,9 +155,21 @@ public slots:
 	void acceptAlienFiles();
 	void statusMessage(QString message, int timeout);
 	void warnSMD(const QString & moduleID);
+    void showPCBView();
+	void groundFill();
+	void removeGroundFill(bool force = false);
+	void copperFill();
+	void setOneGroundFillSeed();
+	void setGroundFillSeeds();	
+	void clearGroundFillSeeds();
+	void changeBoardLayers(int layers, bool doEmit);
+	void swapOne(ItemBase * itemBase, const QString & moduleID);
+	void selectAllObsolete();
+	void swapObsolete();
+	void dropTempSlot(ModelPart *, QWidget *);
 
 protected slots:
-	void load();
+	void mainLoad();
 	void openRecentOrExampleFile();
     void print();
     void doExport();
@@ -170,9 +190,9 @@ protected slots:
     void actualSize();
 	void hundredPercentSize();
     void alignToGrid();
+    void showGrid();
     void showBreadboardView();
     void showSchematicView();
-    void showPCBView();
 	void showPartsBinIconView();
 	void showPartsBinListView();
     void updateEditMenu();
@@ -239,6 +259,7 @@ protected slots:
 	void activeLayerTop();
 	void activeLayerBottom();
 	void activeLayerBoth();
+	void toggleActiveLayer();
 	void createTrace();
 	void excludeFromAutoroute();
 	void selectAllTraces();
@@ -248,11 +269,9 @@ protected slots:
 	void selectAllJumperItems();
 	void selectAllVias();
 
-	void saveBundledSketch();
 	void shareOnline();
 	void saveBundledPart(const QString &moduleId=___emptyString___);
-	void saveBundledAux(ModelPart *mp, const QDir &destFolder);
-	void loadBundledSketch(const QString &fileName, bool dontAsk);
+	QStringList saveBundledAux(ModelPart *mp, const QDir &destFolder);
 	void loadBundledPart();
 
 	void binSaved(bool hasAlienParts);
@@ -269,8 +288,6 @@ protected slots:
 	void reportBug();
 	void enableDebug();
 	void tidyWires();
-	void groundFill();
-	void removeGroundFill();
 	void changeWireColor(bool checked);
 
 	void startSaveInstancesSlot(const QString & fileName, ModelPart *, QXmlStreamWriter &);
@@ -279,9 +296,6 @@ protected slots:
 	void exportNormalizedSVG();
 	void exportNormalizedFlattenedSVG();
 
-	void selectAllObsolete();
-	void swapObsolete();
-
 	void launchExternalProcess();
 	bool externalProcess(QString & name, QString & path, QStringList & args);
 	void processError(QProcess::ProcessError processError);
@@ -299,8 +313,7 @@ protected slots:
 	void updateLayerMenuSlot();
 	bool save();
 	bool saveAs();
-	void changeBoardLayers(int layers, bool doEmit);
-    void backupSketch();
+	void backupSketch();
 	void undoStackCleanChanged(bool isClean);
 	void autosaveNeeded(int index = 0);
 	void firstTimeHelpHidden();
@@ -310,16 +323,19 @@ protected slots:
 	void updateNet();
 	void selectMoveLock();
 	void moveLock();
-	void searchPartsBin();
 	void showNavigator();
 	void autorouterSettings();
 	void warnSMDReally();
 	void boardDeletedSlot();
 	void cursorLocationSlot(double, double);
 	void locationLabelClicked();
+	void swapSelectedMap(const QString & family, const QString & prop, QMap<QString, QString> & currPropsMap, ItemBase *);
 	void swapSelectedDelay(const QString & family, const QString & prop, QMap<QString, QString> & currPropsMap, ItemBase *);
 	void swapSelectedTimeout();
 	void filenameIfSlot(QString & filename);
+	void openURL();
+	void setActiveWire(Wire *);
+	void setActiveConnectorItem(ConnectorItem *);
 
 protected:
 	void initSketchWidget(SketchWidget *);
@@ -329,12 +345,7 @@ protected:
     void createExportActions();
     void createOrderFabAct();
     void createOpenExampleMenu();
-    void populateMenuFromXMLFile(
-    		QMenu *parentMenu, QStringList &actionsTracker,
-    		const QString &folderPath, const QString &indexFileName/*,
-    		const QString &rootNode, const QString &indexNode,
-    		const QString &submenuNode*/
-    );
+    void populateMenuFromXMLFile(QMenu *parentMenu, QStringList &actionsTracker, const QString &folderPath, const QString &indexFileName);
     QHash<QString, struct SketchDescriptor *> indexAvailableElements(QDomElement &domElem, const QString &srcPrefix, QStringList & actionsTracker);
     void populateMenuWithIndex(const QHash<QString, struct SketchDescriptor *> &, QMenu * parentMenu, QDomElement &domElem);
     void populateMenuFromFolderContent(QMenu *parentMenu, const QString &path);
@@ -350,7 +361,6 @@ protected:
 	void connectPairs();
 	void connectPair(SketchWidget * signaller, SketchWidget * slotter);
 	void closeEvent(QCloseEvent * event);
-	bool saveAsAux(const QString & fileName);
 	void saveAsAuxAux(const QString & fileName);
 	void printAux(QPrinter &printer, bool removeBackground, bool paginate);
 	void exportAux(QString fileName, QImage::Format format, bool removeBackground);
@@ -372,17 +382,14 @@ protected:
 	QString getExtensionString();
 	QStringList getExtensions();
 	const QString defaultSaveFolder();
-	const QString & fileName();
 	bool undoStackIsEmpty();
 
 	void createTraceMenuActions();
 	void hideShowTraceMenu();
 
-	QList<ModelPart*> moveToPartsFolder(QDir &unzipDir, MainWindow* mw, bool addToBin=true);
-	bool loadBundledAux(QDir &unzipDir, QList<ModelPart*> mps);
-	bool preloadBundledAux(QDir &unzipDir, bool dontAsk);
-	void copyToSvgFolder(const QFileInfo& file, const QString &destFolder = "contrib");
-	ModelPart* copyToPartsFolder(const QFileInfo& file, bool addToBin=true, const QString &destFolder="contrib");
+	QList<ModelPart*> moveToPartsFolder(QDir &unzipDir, MainWindow* mw, bool addToBin, bool addToAlien, const QString & prefixFolder, const QString &destFolder);
+	void copyToSvgFolder(const QFileInfo& file, bool addToAlien, const QString & prefixFolder, const QString &destFolder);
+	ModelPart* copyToPartsFolder(const QFileInfo& file, bool addToBin, bool addToAlien, const QString & prefixFolder, const QString &destFolder);
 
 	void closeIfEmptySketch(MainWindow* mw);
 	bool whatToDoWithAlienFiles();
@@ -423,9 +430,8 @@ protected:
 	bool alreadyOpen(const QString & fileName);
 	bool loadCustomBoardShape();
 	void svgMissingLayer(const QString & layername, const QString & path);
-	void swapSelectedAux(ItemBase * itemBase, const QString & moduleID);
 	long swapSelectedAuxAux(ItemBase * itemBase, const QString & moduleID, ViewLayer::ViewLayerSpec viewLayerSpec, QUndoCommand * parentCommand);
-	bool swapSpecial(QMap<QString, QString> & currPropsMap);
+	bool swapSpecial(const QString & prop, QMap<QString, QString> & currPropsMap);
 
 	void enableAddBendpointAct(QGraphicsItem *);
 	class FileProgressDialog * exportProgress();
@@ -438,16 +444,18 @@ protected:
 	bool wannaRestart();
 
 	void updateActiveLayerButtons();
+	int activeLayerIndex();
 	bool hasLinkedProgramFiles(const QString & filename, QStringList & linkedProgramFiles);
 	void pasteAux(bool pasteInPlace);
 
 	void routingStatusLabelMouse(QMouseEvent*, bool show);
 	class Wire * retrieveWire();
-	void updatePartsBinMenu(QMenu * partsBinMenu, QMenu * binMenu, int skip);
+	class ConnectorItem * retrieveConnectorItem();
 	QString getBomProps(ItemBase *);
-	void swapSelectedMap(const QString & family, const QString & prop, QMap<QString, QString> & currPropsMap, ItemBase *);
 	ModelPart * findReplacedby(ModelPart * originalModelPart);
-
+	void groundFillAux(bool fillGroundTraces);
+	void connectStartSave(bool connect);
+	void loadBundledSketch(const QString &fileName, bool addToRecent, bool setAsLastOpened);
 
 protected:
 	static void removeActionsStartingAt(QMenu *menu, int start=0);
@@ -511,12 +519,10 @@ protected:
 	QMenu *m_openRecentFileMenu;
     QAction *m_openRecentFileActs[MaxRecentFiles];
 	QMenu *m_openExampleMenu;
-	QMenu * m_partsBinFileMenu;
 	QAction *m_saveAct;
 	QAction *m_saveAsAct;
 	QAction *m_pageSetupAct;
 	QAction *m_printAct;
-	QAction *m_saveAsBundledAct;
 	QAction *m_shareOnlineAct;
 
 	QAction * m_launchExternalProcessAct;
@@ -571,8 +577,6 @@ protected:
 	QAction *m_exportNormalizedFlattenedSvgAction;
     QAction *m_openInPartsEditorAct;
     QMenu *m_addToBinMenu;
-	QMenu * m_partsBinPartMenu;
-	QAction *m_partsBinSearchAct;
 
 
 	QMenu *m_rotateMenu;
@@ -608,6 +612,7 @@ protected:
     QAction *m_actualSizeAct;
     QAction *m_100PercentSizeAct;
     QAction *m_alignToGridAct;
+    QAction *m_showGridAct;
     QAction *m_showBreadboardAct;
     QAction *m_showSchematicAct;
     QAction *m_showPCBAct;
@@ -629,19 +634,20 @@ protected:
 	// Trace Menu
 	QMenu *m_pcbTraceMenu;
 	QMenu *m_schematicTraceMenu;
+	QMenu *m_breadboardTraceMenu;
 	QAction *m_autorouteAct;
 	QAction *m_orderFabAct;
 	QAction *m_activeLayerTopAct;
 	QAction *m_activeLayerBottomAct;
 	QAction *m_activeLayerBothAct;
-	QAction *m_createTraceAct;
 	class WireAction *m_createTraceWireAct;
-	class WireAction *m_updateNetAct;			// when we're confident, we can remove this
+	class WireAction *m_createWireWireAct;
 	QAction *m_createJumperAct;
 	QAction *m_changeTraceLayerAct;
 	QAction *m_excludeFromAutorouteAct;
 	class WireAction *m_excludeFromAutorouteWireAct;
 	QAction *m_selectAllTracesAct;
+	QAction *m_selectAllWiresAct;
 	QAction *m_selectAllCopperFillAct;
 	QAction *m_updateRoutingStatusAct;
 	QAction *m_selectAllExcludedTracesAct;
@@ -649,6 +655,10 @@ protected:
 	QAction *m_selectAllViasAct;
 	QAction *m_groundFillAct;
 	QAction *m_removeGroundFillAct;
+	QAction *m_copperFillAct;
+	class ConnectorItemAction *m_setOneGroundFillSeedAct;
+	QAction *m_setGroundFillSeedsAct;
+	QAction *m_clearGroundFillSeedsAct;
 	QAction *m_designRulesCheckAct;
 	QAction *m_autorouterSettingsAct;
 	QAction *m_tidyWiresAct;
@@ -714,13 +724,16 @@ protected:
 	QTimer m_autosaveTimer;
 	bool m_autosaveNeeded;
 	bool m_backingUp;
-	bool m_recovered;
 	QString m_bundledSketchName;
 	RoutingStatus m_routingStatus;
 	bool m_smdOneSideWarningGiven;
 	bool m_orderFabEnabled;		
-	SwapTimer m_swapTimer;
 	bool m_closeSilently;
+	QString m_fzzFolder;
+	QHash<QString, struct LockedFile *> m_fzzFiles;
+	SwapTimer m_swapTimer;
+	QPointer<Wire> m_activeWire;
+	QPointer<ConnectorItem> m_activeConnectorItem;
 
 public:
 	static int RestartNeeded;
@@ -728,6 +741,7 @@ public:
 	static bool AutosaveEnabled;
 	static QString BackupFolder;
 
+
 protected:
 	static const QString UntitledSketchName;
 	static int UntitledSketchIndex;
