@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5417 $:
+$Revision: 5942 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-08-18 12:47:41 +0200 (Thu, 18 Aug 2011) $
+$Date: 2012-04-04 13:17:45 +0200 (Wed, 04 Apr 2012) $
 
 ********************************************************************/
 
@@ -90,6 +90,19 @@ bool sortPartList(ItemBase * b1, ItemBase * b2){
     return b1->instanceTitle().toLower() < b2->instanceTitle().toLower();
 }
 
+void massageOutput(QString & svg, bool doMask, bool doSilk, QString & maskTop, QString & maskBottom, const QString & fileName, int dpi)
+{
+	if (doMask) {
+		if (fileName.contains("bottom")) maskBottom = svg; else maskTop = svg;
+		svg = TextUtils::expandAndFill(svg, "black", GerberGenerator::MaskClearanceMils * 2 * dpi / 1000);
+	}
+	else if (doSilk) {
+		QString use = (fileName.contains("bottom")) ? maskBottom : maskTop;
+		use = TextUtils::expandAndFill(use, "white", GerberGenerator::MaskClearanceMils * 2 * dpi / 1000);
+		svg = TextUtils::mergeSvg(svg, use, "", false);
+	}
+}
+
 /////////////////////////////////////////////////////////
 
 void MainWindow::initNames()
@@ -139,6 +152,11 @@ void MainWindow::exportEtchable() {
 
 void MainWindow::exportEtchable(bool wantPDF, bool wantSVG, bool flip)
 {
+	QList<ItemBase *> boards = m_pcbGraphicsView->findBoard();
+	if (boards.count() != 1) return;
+
+    ItemBase * board = boards.at(0);
+
 	RoutingStatus routingStatus;
 	m_pcbGraphicsView->updateRoutingStatus(NULL, routingStatus, true);
 	if (routingStatus.m_connectorsLeftToRoute > 0) {
@@ -160,56 +178,61 @@ void MainWindow::exportEtchable(bool wantPDF, bool wantSVG, bool flip)
 	QString extFmt = (wantPDF) ? fileExtFormats.value(pdfActionType) : fileExtFormats.value(svgActionType);
 	QString fileExt;
 
-	if (m_pcbGraphicsView->boardLayers() == 1) {
-		QString fileName = FolderUtils::getSaveFileName(this,
-			tr("Export PDF or SVG for etchable PCB production..."),
-			path+"/"+constructFileName("etch", suffix),
-			extFmt,
-			&fileExt
-		);
 
-		if (fileName.isEmpty()) {
-			return; //Cancel pressed
-		}
+	QString exportDir = QFileDialog::getExistingDirectory(this, tr("Choose a folder for exporting"),
+												defaultSaveFolder(),
+												QFileDialog::ShowDirsOnly
+												| QFileDialog::DontResolveSymlinks);
+	if (exportDir.isEmpty()) return;
 
-		if(!alreadyHasExtension(fileName, suffix)) {
-			fileName += suffix;
-		}
-
-		fileNames.append(fileName);
+	FolderUtils::setOpenSaveFolder(exportDir);
+	fileNames.append(exportDir + "/" + constructFileName("etch_copper_bottom", suffix));
+	fileNames.append(exportDir + "/" + constructFileName("etch_mask_bottom", suffix));
+	if (m_pcbGraphicsView->boardLayers() > 1) {
+		fileNames.append(exportDir + "/" + constructFileName("etch_copper_top", suffix));
+		fileNames.append(exportDir + "/" + constructFileName("etch_mask_top", suffix));
 	}
-	else {
-		QString exportDir = QFileDialog::getExistingDirectory(this, tr("Choose a folder for exporting"),
-												 defaultSaveFolder(),
-												 QFileDialog::ShowDirsOnly
-												 | QFileDialog::DontResolveSymlinks);
-		if (exportDir.isEmpty()) return;
-
-		FolderUtils::setOpenSaveFolder(exportDir);
-		fileNames.append(exportDir + "/" + constructFileName("etch_bottom", suffix));
-		fileNames.append(exportDir + "/" + constructFileName("etch_top", suffix));
-		fileExt = extFmt;
-	}
-
+	fileNames.append(exportDir + "/" + constructFileName("etch_silk_top", suffix));
+	fileExt = extFmt;
 	
 	FileProgressDialog * fileProgressDialog = exportProgress();
 
-    ItemBase * board = m_pcbGraphicsView->findBoard();
 
+	QString maskTop, maskBottom;
+	QList<ItemBase *> copperLogoItems;
 	for (int ix = 0; ix < fileNames.count(); ix++) {
+		bool doMask = false;
+		bool doSilk = false;
 		QString fileName = fileNames[ix];
 		LayerList viewLayerIDs;
-		if (ix == 0) {
+		if (fileName.contains("copper_bottom")) {
 			viewLayerIDs << ViewLayer::GroundPlane0 << ViewLayer::Copper0 << ViewLayer::Copper0Trace;
 		}
-		else {
+		else if (fileName.contains("mask_bottom")) {
+			doMask = true;
+			viewLayerIDs << ViewLayer::Copper0;
+		}
+		else if (fileName.contains("copper_top")) {
 			viewLayerIDs << ViewLayer::GroundPlane1 << ViewLayer::Copper1 << ViewLayer::Copper1Trace;
 		}
+		else if (fileName.contains("mask_top")) {
+			viewLayerIDs << ViewLayer::Copper1;
+			doMask = true;
+		}
+		else if (fileName.contains("silk_top")) {
+			viewLayerIDs << ViewLayer::Silkscreen1 << ViewLayer::Silkscreen1Label;
+			doSilk = true;
+		}
+
+		if (doMask) {
+			m_pcbGraphicsView->hideCopperLogoItems(copperLogoItems);
+		}
 
 		QSizeF imageSize;
 		if (wantSVG) {
 			bool empty;
-			QString svg = m_pcbGraphicsView->renderToSVG(FSvgRenderer::printerScale(), viewLayerIDs, viewLayerIDs, true, imageSize, board, GraphicsUtils::IllustratorDPI, false, false, false, empty);
+			QString svg = m_pcbGraphicsView->renderToSVG(FSvgRenderer::printerScale(), viewLayerIDs, true, imageSize, board, GraphicsUtils::IllustratorDPI, false, false, false, empty);
+			massageOutput(svg, doMask, doSilk, maskTop, maskBottom, fileName, GraphicsUtils::IllustratorDPI);		
 			svg = mergeBoardSvg(svg, board, GraphicsUtils::IllustratorDPI, imageSize, flip);
 			
 			QFile file(fileName);
@@ -225,7 +248,8 @@ void MainWindow::exportEtchable(bool wantPDF, bool wantSVG, bool flip)
 			printer.setOutputFileName(fileName);
 			int res = printer.resolution();
 			bool empty;
-			QString svg = m_pcbGraphicsView->renderToSVG(FSvgRenderer::printerScale(), viewLayerIDs, viewLayerIDs, true, imageSize, board, res, false, false, false, empty);
+			QString svg = m_pcbGraphicsView->renderToSVG(FSvgRenderer::printerScale(), viewLayerIDs, true, imageSize, board, res, false, false, false, empty);
+			massageOutput(svg, doMask, doSilk, maskTop, maskBottom, fileName, res);
 			svg = mergeBoardSvg(svg, board, res, imageSize, flip);
 			
 			// now convert to pdf
@@ -247,6 +271,10 @@ void MainWindow::exportEtchable(bool wantPDF, bool wantSVG, bool flip)
 
 			painter.end();
 		}
+		if (doMask) {
+			m_pcbGraphicsView->restoreCopperLogoItems(copperLogoItems);
+		}
+
 	}
 
 	m_statusBar->showMessage(tr("Sketch exported"), 2000);
@@ -329,7 +357,7 @@ QString MainWindow::mergeBoardSvg(QString & svg, ItemBase * board, int res, QSiz
 	//SvgFileSplitter::changeStrokeWidth(boardSvg, res / 360.0, true, byteArray);
 	imageSize = boardImageSize;
 
-	return TextUtils::mergeSvg(boardSvg /* QString(byteArray) */, svg, "", flip);
+	return TextUtils::convertExtendedChars(TextUtils::mergeSvg(boardSvg /* QString(byteArray) */, svg, "", flip));
 }
 
 QString MainWindow::getBoardSilkscreenSvg(ItemBase * board, int res, QSizeF & imageSize) {
@@ -345,7 +373,7 @@ QString MainWindow::getBoardSilkscreenSvg(ItemBase * board, int res, QSizeF & im
 	LayerList viewLayerIDs;
 	viewLayerIDs << ViewLayer::Silkscreen1;
 	bool empty;
-	QString svg = m_pcbGraphicsView->renderToSVG(FSvgRenderer::printerScale(), viewLayerIDs, viewLayerIDs, true, imageSize, board, res, true, false, false, empty);
+	QString svg = m_pcbGraphicsView->renderToSVG(FSvgRenderer::printerScale(), viewLayerIDs, true, imageSize, board, res, true, false, false, empty);
 	board->setSelected(false);
 	foreach (QGraphicsItem * item, items) {
 		item->setSelected(true);
@@ -451,8 +479,8 @@ void MainWindow::exportAux(QString fileName, QImage::Format format, bool removeB
 
 	QSize imgSize(width, height);
 	QImage image(imgSize,format);
-	image.setDotsPerMeterX(1200*254);
-	image.setDotsPerMeterY(1200*254);
+	image.setDotsPerMeterX(39.3700787 * FSvgRenderer::printerScale());
+	image.setDotsPerMeterY(39.3700787 * FSvgRenderer::printerScale());
 	QPainter painter;
 	QColor color;
 	if (removeBackground) {
@@ -627,11 +655,11 @@ bool MainWindow::saveAsAux(const QString & fileName) {
 	undoStackCleanChanged(true);
 
 	m_statusBar->showMessage(tr("Saved '%1'").arg(fileName), 2000);
-    setCurrentFile(fileName, true, false, "");
+    setCurrentFile(fileName, true, true);
 
-	if(m_restarting && m_fileName != ___emptyString___) {
+	if(m_restarting && m_fwFilename != ___emptyString___) {
 		QSettings settings;
-		settings.setValue("lastOpenSketch",m_fileName);
+		settings.setValue("lastOpenSketch",m_fwFilename);
 	}
 
    // mark the stack clean so we update the window dirty flag
@@ -647,13 +675,23 @@ bool MainWindow::saveAsAux(const QString & fileName) {
 
 void MainWindow::saveAsAuxAux(const QString & fileName) {
     QApplication::setOverrideCursor(Qt::WaitCursor);
-	connect(m_sketchModel->root(), SIGNAL(startSaveInstances(const QString &, ModelPart *, QXmlStreamWriter &)),
-		this, SLOT(startSaveInstancesSlot(const QString &, ModelPart *, QXmlStreamWriter &)), Qt::DirectConnection);
 
-	m_sketchModel->save(fileName, false);
+	connectStartSave(true);
 
-	disconnect(m_sketchModel->root(), SIGNAL(startSaveInstances(const QString &, ModelPart *, QXmlStreamWriter &)),
-			   this, SLOT(startSaveInstancesSlot(const QString &, ModelPart *, QXmlStreamWriter &)));
+	QDir dir(this->m_fzzFolder);
+	QStringList nameFilters("*" + FritzingSketchExtension);
+	QFileInfoList fileList = dir.entryInfoList(nameFilters, QDir::Files | QDir::NoSymLinks);
+	foreach (QFileInfo fileInfo, fileList) {
+		QFile file(fileInfo.absoluteFilePath());
+		file.remove();
+	}
+
+	QString fzName = dir.absoluteFilePath(QFileInfo(fileName).completeBaseName() + FritzingSketchExtension); 
+	m_sketchModel->save(fzName, false);
+
+	saveAsShareable(fileName, false);
+
+	connectStartSave(false);
 
 	QApplication::restoreOverrideCursor();
 }
@@ -671,11 +709,6 @@ void MainWindow::createExportActions() {
 	m_saveAsAct->setStatusTip(tr("Save the current sketch"));
 	connect(m_saveAsAct, SIGNAL(triggered()), this, SLOT(saveAs()));
 
-	m_saveAsBundledAct = new QAction(tr("Save As Shareable..."), this);
-	m_saveAsBundledAct->setShortcut(tr("Alt+Ctrl+S"));
-	m_saveAsBundledAct->setStatusTip(tr("Export current sketch and its non-core parts"));
-	connect(m_saveAsBundledAct, SIGNAL(triggered()), this, SLOT(saveBundledSketch()));
-
 	m_shareOnlineAct = new QAction(tr("Share online..."), this);
 	m_shareOnlineAct->setStatusTip(tr("Post a project to the Fritzing website"));
 	connect(m_shareOnlineAct, SIGNAL(triggered()), this, SLOT(shareOnline()));
@@ -847,7 +880,7 @@ void MainWindow::exportSvg(double res, bool selectedItems, bool flatten) {
 
 	QSizeF imageSize;
 	bool empty;
-	QString svg = m_currentGraphicsView->renderToSVG(FSvgRenderer::printerScale(), viewLayerIDs, viewLayerIDs, false, imageSize, NULL, res, selectedItems, flatten, false, empty);
+	QString svg = m_currentGraphicsView->renderToSVG(FSvgRenderer::printerScale(), viewLayerIDs, false, imageSize, NULL, res, selectedItems, flatten, false, empty);
 	if (svg.isEmpty()) {
 		// tell the user something reasonable
 		return;
@@ -861,7 +894,7 @@ void MainWindow::exportSvg(double res, bool selectedItems, bool flatten) {
 	file.open(QIODevice::WriteOnly);
 	QTextStream out(&file);
 	out.setCodec("UTF-8");
-	out << svg;
+	out << TextUtils::convertExtendedChars(svg);
 	file.close();
 	delete fileProgressDialog;
 
@@ -934,8 +967,7 @@ void MainWindow::exportBOM() {
 	foreach (ItemBase * itemBase, partList) {
 		if (itemBase->itemType() != ModelPart::Part) continue;
 
-        QString label = itemBase->instanceTitle();
-        QString desc = itemBase->title();
+        QString desc = getBomProps(itemBase);
 
         if(!shoppingList.contains(desc)) {
             shoppingList.insert(desc, 1);
@@ -960,13 +992,14 @@ void MainWindow::exportBOM() {
         it.next();
 		ItemBase * itemBase = descrs.value(it.key());
 
-		shoppingListString += bomRowTemplate.arg(it.value()).arg(it.key()).arg(getBomProps(itemBase));
+		// it.value() = count; it.key() = bomProps
+		shoppingListString += bomRowTemplate.arg(it.value()).arg(itemBase->title()).arg(it.key());
     }
 
 	QString bom = bomTemplate
 		.arg("Fritzing Bill of Materials")
-		.arg(QFileInfo(m_fileName).fileName())
-		.arg(m_fileName)
+		.arg(QFileInfo(m_fwFilename).fileName())
+		.arg(m_fwFilename)
 		.arg(QDateTime::currentDateTime().toString("dddd, MMMM d yyyy, hh:mm:ss"))
 		.arg(assemblyString)
 		.arg(shoppingListString)
@@ -1028,7 +1061,7 @@ void MainWindow::exportNetlist() {
 	doc.setContent(QString("<?xml version='1.0' encoding='UTF-8'?>\n") + TextUtils::CreatedWithFritzingXmlComment);
 	QDomElement netlist = doc.createElement("netlist");
 	doc.appendChild(netlist);
-	netlist.setAttribute("sketch", QFileInfo(m_fileName).fileName());
+	netlist.setAttribute("sketch", QFileInfo(m_fwFilename).fileName());
     netlist.setAttribute("date", QDateTime::currentDateTime().toString());
 
 	// TODO: filter out 'ignore' connectors
@@ -1128,7 +1161,7 @@ void MainWindow::exportNetlist() {
 }
 
 FileProgressDialog * MainWindow::exportProgress() {
-	return (new FileProgressDialog("Exporting...", 0, this));
+    return (new FileProgressDialog("Exporting...", 0, this));
 }
 
 void MainWindow::exportNormalizedSVG() {
@@ -1169,14 +1202,21 @@ void MainWindow::exportToGerber() {
 
     //NOTE: this assumes just one board per sketch
 
-    // grab the list of parts
-    ItemBase * board = m_pcbGraphicsView->findBoard();
+	QList<ItemBase *> boards = m_pcbGraphicsView->findBoard();
+
     // barf an error if there's no board
-    if (!board) {
+    if (boards.count() == 0) {
         QMessageBox::critical(this, tr("Fritzing"),
                    tr("Your sketch does not have a board yet!  Please add a PCB in order to export to Gerber."));
         return;
     }
+    if (boards.count() > 1) {
+        QMessageBox::critical(this, tr("Fritzing"),
+                   tr("Gerber export can not handle multiple boards."));
+        return;
+    }
+
+	ItemBase * board = boards.at(0);
 
     QString exportDir = QFileDialog::getExistingDirectory(this, tr("Choose a folder for exporting"),
                                              defaultSaveFolder(),
@@ -1185,11 +1225,27 @@ void MainWindow::exportToGerber() {
 
 	if (exportDir.isEmpty()) return;
 
+	FileProgressDialog * fileProgressDialog = exportProgress();
+
 	FolderUtils::setOpenSaveFolder(exportDir);
-	GerberGenerator::exportToGerber(m_fileName, exportDir, board, m_pcbGraphicsView, true);
+	GerberGenerator::exportToGerber(m_fwFilename, exportDir, board, m_pcbGraphicsView, true);
+	m_statusBar->showMessage(tr("Sketch exported to Gerber"), 2000);
+
+	delete fileProgressDialog;
 }
 
 void MainWindow::exportToGerber(const QString & exportDir) {
-	GerberGenerator::exportToGerber(m_fileName, exportDir, NULL, m_pcbGraphicsView, false);
+	GerberGenerator::exportToGerber(m_fwFilename, exportDir, NULL, m_pcbGraphicsView, false);
 }
 
+void MainWindow::connectStartSave(bool doConnect) {
+
+	if (doConnect) {
+		connect(m_sketchModel->root(), SIGNAL(startSaveInstances(const QString &, ModelPart *, QXmlStreamWriter &)),
+				this, SLOT(startSaveInstancesSlot(const QString &, ModelPart *, QXmlStreamWriter &)), Qt::DirectConnection);
+	}
+	else {
+		disconnect(m_sketchModel->root(), SIGNAL(startSaveInstances(const QString &, ModelPart *, QXmlStreamWriter &)),
+				this, SLOT(startSaveInstancesSlot(const QString &, ModelPart *, QXmlStreamWriter &)));
+	}
+}
