@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5143 $:
+$Revision: 5922 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-07-01 02:37:01 +0200 (Fri, 01 Jul 2011) $
+$Date: 2012-03-19 20:11:41 +0100 (Mon, 19 Mar 2012) $
 
 ********************************************************************/
 
@@ -29,14 +29,22 @@ $Date: 2011-07-01 02:37:01 +0200 (Fri, 01 Jul 2011) $
 #include "../items/partfactory.h"
 #include "../items/moduleidnames.h"
 #include "../version/version.h"
+#include "../viewgeometry.h"
 
 #include <QMessageBox>
 
+
+/////////////////////////////////////////////////
+
 ModelBase::ModelBase( bool makeRoot )
 {
 	m_reportMissingModules = true;
 	m_referenceModel = NULL;
-	m_root = (makeRoot) ? new ModelPart() : NULL;
+	m_root = NULL;
+	if (makeRoot) {
+		m_root = new ModelPart();
+		m_root->setModelPartShared(new ModelPartSharedRoot());
+	}
 }
 
 ModelBase::~ModelBase() {
@@ -45,7 +53,6 @@ ModelBase::~ModelBase() {
 	}
 }
 
-
 ModelPart * ModelBase::root() {
 	return m_root;
 }
@@ -96,8 +103,9 @@ bool ModelBase::load(const QString & fileName, ModelBase * refModel, QList<Model
     }
 
 	bool checkForRats = true;
-	QString fzVersion = root.attribute("fritzingVersion");
-	if (fzVersion.length() > 0) {
+	bool checkForTraces = true;
+	m_fritzingVersion = root.attribute("fritzingVersion");
+	if (m_fritzingVersion.length() > 0) {
 		// with version 0.4.3 ratsnests in fz files are obsolete
 		VersionThing versionThingRats;
 		versionThingRats.majorVersion = 0;
@@ -105,15 +113,40 @@ bool ModelBase::load(const QString & fileName, ModelBase * refModel, QList<Model
 		versionThingRats.minorSubVersion = 2;
 		versionThingRats.releaseModifier = "";
 		VersionThing versionThingFz;
-		Version::toVersionThing(fzVersion,versionThingFz);
+		Version::toVersionThing(m_fritzingVersion,versionThingFz);
 		checkForRats = !Version::greaterThan(versionThingRats, versionThingFz);
+		// with version 0.6.5 traces are copied to all views
+		versionThingRats.minorVersion = 6;
+		versionThingRats.minorSubVersion = 4;
+		checkForTraces = !Version::greaterThan(versionThingRats, versionThingFz);
 	}
 
+	ModelPartSharedRoot * modelPartSharedRoot = this->rootModelPartShared();
+
     QDomElement title = root.firstChildElement("title");
 	if (!title.isNull()) {
-		this->root()->modelPartShared()->setTitle(title.text());
+		if (modelPartSharedRoot) {
+			modelPartSharedRoot->setTitle(title.text());
+		}
 	}
 
+	QString iconFilename = root.attribute("icon");
+	if (iconFilename.isEmpty()) {
+		iconFilename = title.text() + ".png";
+	}
+
+	if (!iconFilename.isEmpty()) {
+		if (modelPartSharedRoot) {
+			modelPartSharedRoot->setIcon(iconFilename);
+		}
+	}
+
+	QString searchTerm = root.attribute("search");
+	if (!searchTerm.isEmpty() && modelPartSharedRoot != NULL) {
+		modelPartSharedRoot->setSearchTerm(searchTerm);
+	}
+
+
     QDomElement views = root.firstChildElement("views");
 	emit loadedViews(this, views);
 
@@ -131,7 +164,28 @@ bool ModelBase::load(const QString & fileName, ModelBase * refModel, QList<Model
    	}
 
 	emit loadingInstances(this, instances);
-	bool result = loadInstances(domDocument, instances, modelParts, checkForRats);
+
+	if (checkForRats) {
+		QDomElement instance = instances.firstChildElement("instance");
+   		while (!instance.isNull()) {
+			QDomElement nextInstance = instance.nextSiblingElement("instance");
+			if (isRatsnest(instance)) {
+				instances.removeChild(instance);
+			}
+			
+			instance = nextInstance;
+		}
+	}
+
+	if (checkForTraces) {
+		QDomElement instance = instances.firstChildElement("instance");
+   		while (!instance.isNull()) {
+			checkTraces(instance);
+			instance = instance.nextSiblingElement("instance");
+		}
+	}
+
+	bool result = loadInstances(domDocument, instances, modelParts);
 	emit loadedInstances(this, instances);
 	return result;
 }
@@ -140,18 +194,13 @@ ModelPart * ModelBase::fixObsoleteModuleID(QDomDocument & domDocument, QDomEleme
 	return PartFactory::fixObsoleteModuleID(domDocument, instance, moduleIDRef, m_referenceModel);
 }
 
-bool ModelBase::loadInstances(QDomDocument & domDocument, QDomElement & instances, QList<ModelPart *> & modelParts, bool checkForRats)
+bool ModelBase::loadInstances(QDomDocument & domDocument, QDomElement & instances, QList<ModelPart *> & modelParts)
 {
 	QHash<QString, QString> missingModules;
    	QDomElement instance = instances.firstChildElement("instance");
    	ModelPart* modelPart = NULL;
    	while (!instance.isNull()) {
 		emit loadingInstance(this, instance);
-		if (checkForRats && isRatsnest(instance)) {
-			// ratsnests in sketches are now obsolete
-			instance = instance.nextSiblingElement("instance");
-			continue;
-		}
 
    		// for now assume all parts are in the palette
    		QString moduleIDRef = instance.attribute("moduleIdRef");
@@ -191,6 +240,13 @@ bool ModelBase::loadInstances(QDomDocument & domDocument, QDomElement & instance
    			modelPart->setInstanceTitle(instanceTitle);
    		}
 
+		QDomElement localConnectors = instance.firstChildElement("localConnectors");
+		QDomElement localConnector = localConnectors.firstChildElement("localConnector");
+		while (!localConnector.isNull()) {
+			modelPart->setConnectorLocalName(localConnector.attribute("id"), localConnector.attribute("name"));
+			localConnector = localConnector.nextSiblingElement("localConnector");
+		}
+
    		QString instanceText = instance.firstChildElement("text").text();
    		if(!instanceText.isNull() && !instanceText.isEmpty()) {
    			modelPart->setInstanceText(instanceText);
@@ -284,15 +340,18 @@ bool ModelBase::addPart(ModelPart * modelPart, bool update) {
 
 
 void ModelBase::save(const QString & fileName, bool asPart) {
-	QDir dir(fileName);
-	dir.cdUp();
+	QFileInfo info(fileName);
+	QDir dir = info.absoluteDir();
 
-	QString temp = dir.absolutePath() + "/" + "part_temp.xml";
+	QString temp = dir.absoluteFilePath("temp.xml");
     QFile file1(temp);
     if (!file1.open(QFile::WriteOnly | QFile::Text)) {
         QMessageBox::warning(NULL, QObject::tr("Fritzing"),
-                             QObject::tr("Cannot write file temp file:\n%1.")
-                              .arg(file1.errorString()));
+                             QObject::tr("Cannot write file temp:\n%1\n%2\n%3.")
+							  .arg(temp)
+							  .arg(fileName)
+                              .arg(file1.errorString())
+							  );
         return;
     }
 
@@ -381,7 +440,7 @@ bool ModelBase::paste(ModelBase * refModel, QByteArray & data, QList<ModelPart *
 	//file.write(domDocument.toByteArray());
 	//file.close();
 
-	return loadInstances(domDocument, instances, modelParts, false);
+	return loadInstances(domDocument, instances, modelParts);
 }
 
 void ModelBase::renewModelIndexes(QDomElement & parentElement, const QString & childName, QHash<long, long> & oldToNew)
@@ -428,10 +487,6 @@ void ModelBase::renewModelIndexes(QDomElement & parentElement, const QString & c
 	}
 }
 
-bool ModelBase::isRatsnest(QDomElement & instance) {
-	return PartFactory::isRatsnest(instance);
-}
-
 void ModelBase::setReportMissingModules(bool b) {
 	m_reportMissingModules = b;
 }
@@ -441,8 +496,134 @@ bool ModelBase::genFZP(const QString & moduleID, ModelBase * refModel) {
 	if (path.isEmpty()) return false;
 
 	ModelPart* mp = refModel->addPart(path, true, true);
-	mp->setCore(true);
+	if (mp) mp->setCore(true);
 	return mp != NULL;
 }
 
+ModelPartSharedRoot * ModelBase::rootModelPartShared() {
+	if (m_root == NULL) return NULL;
 
+	return m_root->modelPartSharedRoot();
+}
+
+bool ModelBase::isRatsnest(QDomElement & instance) {
+	QString moduleIDRef = instance.attribute("moduleIdRef");
+	if (moduleIDRef.compare(ModuleIDNames::WireModuleIDName) != 0) return false;
+
+	QDomElement views = instance.firstChildElement("views");
+	if (views.isNull()) return false;
+
+	QDomElement view = views.firstChildElement();
+	while (!view.isNull()) {
+		QDomElement geometry = view.firstChildElement("geometry");
+		if (!geometry.isNull()) {
+			int flags = geometry.attribute("wireFlags").toInt();
+			if (flags & ViewGeometry::RatsnestFlag) {
+				return true;
+			}
+			if (flags & ViewGeometry::ObsoleteJumperFlag) {
+				return true;
+			}
+		}
+
+		view = view.nextSiblingElement();
+	}
+
+	return false;
+}
+
+
+void ModelBase::checkTraces(QDomElement & instance) {
+	QString moduleIDRef = instance.attribute("moduleIdRef");
+	if (moduleIDRef.compare(ModuleIDNames::WireModuleIDName) != 0) return;
+
+	QDomElement views = instance.firstChildElement("views");
+	if (views.isNull()) return;
+
+	QDomElement bbView = views.firstChildElement("breadboardView");
+	QDomElement schView = views.firstChildElement("schematicView");
+	QDomElement pcbView = views.firstChildElement("pcbView");
+
+	if (!bbView.isNull() && !schView.isNull() && !pcbView.isNull()) {
+		// if it's a breadboard wire; just make sure flag is correct
+
+		QList<QDomElement> elements;
+		elements << bbView << schView << pcbView;
+		foreach (QDomElement element, elements) {
+			QDomElement geometry = element.firstChildElement("geometry");
+			if (!geometry.isNull()) {
+				int flags = geometry.attribute("wireFlags").toInt();
+				if (flags & ViewGeometry::PCBTraceFlag) return;				// not a breadboard wire, bail out
+				if (flags & ViewGeometry::SchematicTraceFlag) return;		// not a breadboard wire, bail out
+
+				if ((flags & ViewGeometry::NormalFlag) == 0) {
+					flags |= ViewGeometry::NormalFlag;
+					geometry.setAttribute("wireFlags", QString::number(flags));
+				}
+			}
+		}
+
+
+		return;
+	}
+
+	if (!bbView.isNull()) {
+		QDomElement geometry = bbView.firstChildElement("geometry");
+		if (!geometry.isNull()) {
+			int flags = geometry.attribute("wireFlags").toInt();
+			if ((flags & ViewGeometry::NormalFlag) == 0) {
+				flags |= ViewGeometry::NormalFlag;
+				geometry.setAttribute("wireFlags", QString::number(flags));
+			}
+		}
+		schView = bbView.cloneNode(true).toElement();
+		pcbView = bbView.cloneNode(true).toElement();
+		schView.setTagName("schematicView");
+		pcbView.setTagName("pcbView");
+		views.appendChild(pcbView);
+		views.appendChild(schView);
+		return;
+	}
+
+	if (!schView.isNull()) {
+		QDomElement geometry = schView.firstChildElement("geometry");
+		if (!geometry.isNull()) {
+			int flags = geometry.attribute("wireFlags").toInt();
+			if (flags & ViewGeometry::PCBTraceFlag) {
+				flags ^= ViewGeometry::PCBTraceFlag;
+				flags |= ViewGeometry::SchematicTraceFlag;
+				geometry.setAttribute("wireFlags", QString::number(flags));
+			}
+		}
+		pcbView = schView.cloneNode(true).toElement();
+		bbView = schView.cloneNode(true).toElement();
+		pcbView.setTagName("pcbView");
+		bbView.setTagName("breadboardView");
+		views.appendChild(bbView);
+		views.appendChild(pcbView);
+		return;
+	}
+	
+	if (!pcbView.isNull()) {
+		schView = pcbView.cloneNode(true).toElement();
+		bbView = pcbView.cloneNode(true).toElement();
+		schView.setTagName("schematicView");
+		bbView.setTagName("breadboardView");
+		views.appendChild(bbView);
+		views.appendChild(schView);
+		return;
+	}
+
+	QDomElement iconView = views.firstChildElement("iconView");
+	if (!iconView.isNull()) return;
+
+	QString string;
+	QTextStream stream(&string);
+	instance.save(stream, 0);
+	stream.flush();
+	DebugDialog::debug(QString("no wire view elements in fz file %1").arg(string));	
+}
+
+const QString & ModelBase::fritzingVersion() {
+	return m_fritzingVersion;
+}
