@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5348 $:
+$Revision: 5945 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-08-05 11:53:44 +0200 (Fri, 05 Aug 2011) $
+$Date: 2012-04-07 00:49:26 +0200 (Sat, 07 Apr 2012) $
 
 ********************************************************************/
 
@@ -69,8 +69,7 @@ ModelPart::ModelPart(QDomDocument * domDocument, const QString & path, ItemType
 
 void ModelPart::commonInit(ItemType type) {
 	m_type = type;
-	m_core = false;
-	m_alien = false;
+	m_locationFlags = 0;
 	m_indexSynched = false;
 }
 
@@ -147,9 +146,7 @@ void ModelPart::copy(ModelPart * modelPart) {
 
 	m_type = modelPart->itemType();
 	m_modelPartShared = modelPart->modelPartShared();
-	m_core = modelPart->isCore();
-	m_contrib = modelPart->isContrib();
-	m_alien = modelPart->isAlien();
+	m_locationFlags = modelPart->m_locationFlags;
 }
 
 void ModelPart::copyNew(ModelPart * modelPart) {
@@ -168,6 +165,10 @@ ModelPartShared * ModelPart::modelPartShared() {
 	return m_modelPartShared;
 }
 
+ModelPartSharedRoot * ModelPart::modelPartSharedRoot() {
+	return qobject_cast<ModelPartSharedRoot *>(m_modelPartShared);
+}
+
 void ModelPart::setModelPartShared(ModelPartShared * modelPartShared) {
 	m_modelPartShared = modelPartShared;
 }
@@ -193,6 +194,15 @@ void ModelPart::saveInstances(const QString & fileName, QXmlStreamWriter & strea
 		streamWriter.writeStartDocument();
     	streamWriter.writeStartElement("module");
 		streamWriter.writeAttribute("fritzingVersion", Version::versionString());
+		ModelPartSharedRoot * root = modelPartSharedRoot();
+		if (root) {
+			if (!root->icon().isEmpty()) {
+				streamWriter.writeAttribute("icon", root->icon());
+			}
+			if (!root->searchTerm().isEmpty()) {
+				streamWriter.writeAttribute("search", root->searchTerm());
+			}
+		}
 		QString title = this->title();
 		if(!title.isNull() && !title.isEmpty()) {
 			streamWriter.writeTextElement("title",title);
@@ -243,6 +253,27 @@ void ModelPart::saveInstance(QXmlStreamWriter & streamWriter)
 		}
 	}
 
+	bool writeLocal = false;
+	foreach (Connector * connector, this->connectors()) {
+		if (!connector->connectorLocalName().isEmpty()) {
+			writeLocal = true;
+			break;
+		}
+	}
+
+	if (writeLocal) {
+		streamWriter.writeStartElement("localConnectors");
+		foreach (Connector * connector, this->connectors()) {
+			if (!connector->connectorLocalName().isEmpty()) {
+				streamWriter.writeStartElement("localConnector");
+				streamWriter.writeAttribute("id", connector->connectorSharedID());
+				streamWriter.writeAttribute("name", connector->connectorLocalName());
+				streamWriter.writeEndElement();
+			}
+		}
+		streamWriter.writeEndElement();
+	}
+
 	foreach (QByteArray byteArray, dynamicPropertyNames()) {
 		streamWriter.writeStartElement("property");
 		streamWriter.writeAttribute("name",  byteArray.data());
@@ -451,9 +482,15 @@ const QDomElement & ModelPart::instanceDomElement() {
 }
 
 const QString & ModelPart::title() {
+	if (!m_localTitle.isEmpty()) return m_localTitle;
+
 	if (m_modelPartShared != NULL) return m_modelPartShared->title();
 
-	return ___emptyString___;
+	return m_localTitle;
+}
+
+void ModelPart::setLocalTitle(const QString & localTitle) {
+	m_localTitle = localTitle;
 }
 
 const QString & ModelPart::version() {
@@ -511,27 +548,44 @@ bool ModelPart::ignoreTerminalPoints() {
 }
 
 bool ModelPart::isCore() {
-	return m_core;
+	return (m_locationFlags & CoreFlag) != 0;
 }
 
 void ModelPart::setCore(bool core) {
-	m_core = core;
+	setLocationFlag(core, CoreFlag);
 }
 
 bool ModelPart::isContrib() {
-	return m_contrib;
+	return (m_locationFlags & ContribFlag) != 0;
 }
 
 void ModelPart::setContrib(bool contrib) {
-	m_contrib = contrib;
+	setLocationFlag(contrib, ContribFlag);
 }
 
 bool ModelPart::isAlien() {
-	return m_alien;
+	return (m_locationFlags & AlienFlag) != 0;;
 }
 
 void ModelPart::setAlien(bool alien) {
-	m_alien = alien;
+	setLocationFlag(alien, AlienFlag);
+}
+
+bool ModelPart::isFzz() {
+	return (m_locationFlags & FzzFlag) != 0;;
+}
+
+void ModelPart::setFzz(bool fzz) {
+	setLocationFlag(fzz, FzzFlag);
+}
+
+void ModelPart::setLocationFlag(bool setting, LocationFlag flag) {
+	if (setting) {
+		m_locationFlags |= flag;
+	}
+	else {
+		m_locationFlags &= ~flag;
+	}
 }
 
 bool ModelPart::hasViewIdentifier(ViewIdentifierClass::ViewIdentifier viewIdentifier) {
@@ -572,44 +626,6 @@ QList<ModelPart*> ModelPart::getAllNonCoreParts() {
 	return retval;
 }
 
-QList<SvgAndPartFilePath> ModelPart::getAvailableViewFiles() {
-	QDomElement viewsElems = modelPartShared()->domDocument()->documentElement().firstChildElement("views");
-	QHash<ViewIdentifierClass::ViewIdentifier, SvgAndPartFilePath> viewImages;
-
-	grabImagePath(viewImages, viewsElems, ViewIdentifierClass::IconView);
-	grabImagePath(viewImages, viewsElems, ViewIdentifierClass::BreadboardView);
-	grabImagePath(viewImages, viewsElems, ViewIdentifierClass::SchematicView);
-	grabImagePath(viewImages, viewsElems, ViewIdentifierClass::PCBView);
-
-	return viewImages.values();
-}
-
-void ModelPart::grabImagePath(QHash<ViewIdentifierClass::ViewIdentifier, SvgAndPartFilePath> &viewImages, QDomElement &viewsElems, ViewIdentifierClass::ViewIdentifier viewId) {
-	QDomElement viewElem = viewsElems.firstChildElement(ViewIdentifierClass::viewIdentifierXmlName(viewId));
-	if(!viewElem.isNull()) {
-		QString partspath = FolderUtils::getUserDataStorePath("parts")+"/svg";
-		QDomElement layerElem = viewElem.firstChildElement("layers");
-		if (!layerElem.isNull()) {
-			QString imagepath = layerElem.attribute("image");
-			QString folderinparts = inWhichFolder(partspath, imagepath);
-			if(folderinparts != ___emptyString___) {
-				SvgAndPartFilePath st(partspath,folderinparts,imagepath);
-				viewImages[viewId] = st;
-			}
-		}
-	}
-}
-
-QString ModelPart::inWhichFolder(const QString &partspath, const QString &imagepath) {
-	QStringList pf = possibleFolders();
-	for(int i=0; i < pf.size(); i++) {
-		if (QFileInfo( partspath+"/"+pf[i]+"/"+imagepath ).exists()) {
-			return pf[i];
-		}
-	}
-	return ___emptyString___;
-}
-
 bool ModelPart::hasViewID(long id) {
 	foreach (ItemBase * item, m_viewItems) {
 		if (item->id() == id) return true;
@@ -836,3 +852,21 @@ ModelPart::ItemType ModelPart::itemType() const
 	return m_type; 
 };
 
+void ModelPart::setConnectorLocalName(const QString & id, const QString & name)
+{
+	if (id.isEmpty()) return;
+	Connector * connector = m_connectorHash.value(id, NULL);
+	if (connector) {
+		connector->setConnectorLocalName(name);
+	}
+}
+
+QString ModelPart::family(){
+	if (m_modelPartShared) return m_modelPartShared->family();
+
+	return "";
+}
+
+bool ModelPart::hasViewItems() {
+	return (m_viewItems.count() > 0);
+}
