@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5308 $:
+$Revision: 5970 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-07-30 21:09:56 +0200 (Sat, 30 Jul 2011) $
+$Date: 2012-04-10 19:16:12 +0200 (Tue, 10 Apr 2012) $
 
 ********************************************************************/
 
@@ -51,6 +51,24 @@ void copyPinAttributes(QDomElement & from, QDomElement & to)
 
 ///////////////////////////////////////////////
 
+const QString & ModelPartSharedRoot::icon() {
+	return m_icon;
+}
+
+void ModelPartSharedRoot::setIcon(const QString & filename) {
+	m_icon = filename;
+}
+
+const QString & ModelPartSharedRoot::searchTerm() {
+	return m_searchTerm;
+}
+
+void ModelPartSharedRoot::setSearchTerm(const QString & searchTerm) {
+	m_searchTerm = searchTerm;
+}
+
+///////////////////////////////////////////////
+
 const QString ModelPartShared::PartNumberPropertyName = "part number";
 
 ModelPartShared::ModelPartShared() {
@@ -285,6 +303,7 @@ void ModelPartShared::setFamily(const QString &family) {
 QHash<QString,QString> & ModelPartShared::properties() {
 	return m_properties;
 }
+
 void ModelPartShared::setProperties(const QHash<QString,QString> &properties) {
 	m_properties = properties;
 	ensurePartNumberProperty();
