@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5206 $:
+$Revision: 5759 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-07-08 23:59:08 +0200 (Fri, 08 Jul 2011) $
+$Date: 2012-01-16 07:45:36 +0100 (Mon, 16 Jan 2012) $
 
 ********************************************************************/
 
@@ -43,13 +43,16 @@ bool PaletteModel::CreateAllPartsBinFile = false;  // now generating the all par
 
 bool PaletteModel::CreateNonCorePartsBinFile = false;
 bool PaletteModel::CreateContribPartsBinFile = true;
+bool PaletteModel::CreateTempPartsBinFile = true;
 
 static bool JustAppendAllPartsInstances = false;
 static bool FirstTime = true;
+static bool FirstTimeWrite = true;
 
 QString PaletteModel::AllPartsBinFilePath = ___emptyString___;
 QString PaletteModel::NonCorePartsBinFilePath = ___emptyString___;
 QString PaletteModel::ContribPartsBinFilePath = ___emptyString___;
+QString PaletteModel::TempPartsBinFilePath = ___emptyString___;
 
 static QString FritzingContribPath;
 
@@ -100,6 +103,7 @@ void PaletteModel::initNames() {
 	AllPartsBinFilePath = FolderUtils::getApplicationSubFolderPath("bins")+"/allParts.dbg" + FritzingBinExtension;
 	NonCorePartsBinFilePath = FolderUtils::getUserDataStorePath("bins")+"/nonCoreParts" + FritzingBinExtension;
 	ContribPartsBinFilePath = FolderUtils::getUserDataStorePath("bins")+"/contribParts" + FritzingBinExtension;
+	TempPartsBinFilePath = FolderUtils::getUserDataStorePath("bins")+"/tempParts" + FritzingBinExtension;
 }
 
 ModelPart * PaletteModel::retrieveModelPart(const QString & moduleID) {
@@ -150,6 +154,7 @@ void PaletteModel::loadParts(bool fastLoad) {
 	int totalPartCount = 0;
 	emit loadedPart(0, totalPartCount);
 
+
 	QDir * dir1 = FolderUtils::getApplicationSubFolder("parts");
 	if (dir1 != NULL) {
 		countParts(*dir1, nameFilters, totalPartCount);
@@ -159,6 +164,7 @@ void PaletteModel::loadParts(bool fastLoad) {
 	QDir dir3(":/resources/parts");
 	countParts(dir3, nameFilters, totalPartCount);
 
+
 	int loadingPart = 0;
 	if (dir1 != NULL) {
 		loadPartsAux(*dir1, nameFilters, loadingPart, totalPartCount, fastLoad);
@@ -186,14 +192,16 @@ void PaletteModel::loadParts(bool fastLoad) {
 void PaletteModel::writeCommonBinsHeader() {
 	writeCommonBinsHeaderAux(CreateAllPartsBinFile, AllPartsBinFilePath, "All Parts");
 	writeCommonBinsHeaderAux(CreateNonCorePartsBinFile, NonCorePartsBinFilePath, "All my parts");
-	writeCommonBinsHeaderAux(CreateContribPartsBinFile, ContribPartsBinFilePath, "Contributed");
+	writeCommonBinsHeaderAux(CreateContribPartsBinFile, ContribPartsBinFilePath, "Contributed Parts");
+	writeCommonBinsHeaderAux(CreateTempPartsBinFile, TempPartsBinFilePath, "Temporary Parts");
 }
 
 void PaletteModel::writeCommonBinsHeaderAux(bool doIt, const QString &filename, const QString &binName) {
 	QString header =
-		"<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"+TextUtils::CreatedWithFritzingXmlComment+
-		QString("<module fritzingVersion='%1'>\n").arg(Version::versionString())+
-		QString("\t<title>%1</title>\n").arg(binName)+
+		"<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" + 
+		TextUtils::CreatedWithFritzingXmlComment +
+		QString("<module fritzingVersion='%1' icon='%2.png'>\n").arg(Version::versionString()).arg(binName) +
+		QString("\t<title>%1</title>\n").arg(binName) +
 		"\t<instances>\n";
 	writeCommonBinAux(header, QFile::WriteOnly, doIt, filename);
 }
@@ -202,6 +210,7 @@ void PaletteModel::writeCommonBinsFooter() {
 	writeCommonBinsFooterAux(CreateAllPartsBinFile, AllPartsBinFilePath);
 	writeCommonBinsFooterAux(CreateNonCorePartsBinFile, NonCorePartsBinFilePath);
 	writeCommonBinsFooterAux(CreateContribPartsBinFile, ContribPartsBinFilePath);
+	writeCommonBinsFooterAux(CreateTempPartsBinFile, TempPartsBinFilePath);
 }
 
 void PaletteModel::writeCommonBinsFooterAux(bool doIt, const QString &filename) {
@@ -263,6 +272,13 @@ void PaletteModel::writeCommonBinInstance(const QString &moduleID, const QString
 void PaletteModel::writeCommonBinAux(const QString &textToWrite, QIODevice::OpenMode openMode, bool doIt, const QString &filename) {
 	if(!doIt) return;
 
+	if (FirstTimeWrite) {
+		FirstTimeWrite = false;
+		QFileInfo info(filename);
+		QDir dir = info.absoluteDir();
+		dir.mkpath(info.absolutePath());
+	}
+
 	QFile file(filename);
 	if (file.open(openMode | QFile::Text)) {
 		QTextStream out(&file);
@@ -292,9 +308,10 @@ void PaletteModel::loadPartsAux(QDir & dir, QStringList & nameFilters, int & loa
     for (int i = 0; i < list.size(); ++i) {
         QFileInfo fileInfo = list.at(i);
         QString path = fileInfo.absoluteFilePath ();
-        // DebugDialog::debug(QString("part path:%1 core? %2").arg(path).arg(m_loadingCore? "true" : "false"));
+        //DebugDialog::debug(QString("part path:%1 core? %2").arg(path).arg(m_loadingCore? "true" : "false"));
         loadPart(path, false, fastLoad);
 		emit loadedPart(++loadingPart, totalPartCount);
+        //DebugDialog::debug("loadedok");
     }
 
     QStringList dirs = dir.entryList(QDir::AllDirs | QDir::NoSymLinks | QDir::NoDotAndDotDot);
