@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5388 $:
+$Revision: 5867 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-08-11 15:17:12 +0200 (Thu, 11 Aug 2011) $
+$Date: 2012-02-15 08:02:43 +0100 (Wed, 15 Feb 2012) $
 
 ********************************************************************/
 
@@ -109,6 +109,7 @@ void PartsEditorMainWindow::initText() {
 
 void PartsEditorMainWindow::setup(long id, ModelPart *modelPart, bool fromTemplate, ItemBase * fromItem)
 {
+	ModelPart * originalModelPart = NULL;
     QFile styleSheet(":/resources/styles/partseditor.qss");
     m_mainFrame = new QFrame(this);
     m_mainFrame->setObjectName("partsEditor");
@@ -139,7 +140,7 @@ void PartsEditorMainWindow::setup(long id, ModelPart *modelPart, bool fromTempla
 		m_updateEnabled = modelPart->isCore()?
 							CORE_EDITION_ENABLED:
 							(modelPart->isContrib() || modelPart->isAlien()? false: true);
-		m_fileName = modelPart->path();
+		m_fwFilename = modelPart->path();
 		setTitle();
 		UntitledPartIndex--; // TODO Mariano: not good enough
 	}
@@ -147,7 +148,7 @@ void PartsEditorMainWindow::setup(long id, ModelPart *modelPart, bool fromTempla
 	if(!fromTemplate) {
 		m_sketchModel = new SketchModel(true);
 	} else {
-		ModelPart * mp = m_paletteModel->loadPart(m_fileName, false, false);
+		ModelPart * mp = m_paletteModel->loadPart(m_fwFilename, false, false);
 	    // this seems hacky but maybe it's ok
 		if (mp == NULL || mp->modelPartShared() == NULL) {
 			QMessageBox::critical(this, tr("Parts Editor"),
@@ -171,6 +172,7 @@ void PartsEditorMainWindow::setup(long id, ModelPart *modelPart, bool fromTempla
 			}
 		}
 
+		originalModelPart = modelPart;
 		modelPart = mp;
 		m_sketchModel = new SketchModel(modelPart);
 	}
@@ -181,6 +183,17 @@ void PartsEditorMainWindow::setup(long id, ModelPart *modelPart, bool fromTempla
 	createCenter(mp, fromItem);
 	createFooter();
 
+	// copy connector local names here if any
+	if (originalModelPart) {
+		foreach (Connector * fromConnector, originalModelPart->connectors()) {
+			foreach (Connector * toConnector, mp->connectors()) {
+				if (toConnector->connectorSharedID().compare(fromConnector->connectorSharedID()) == 0) {
+					toConnector->setConnectorLocalName(fromConnector->connectorLocalName());
+				}
+			}
+		}
+	}
+
 	layout()->setMargin(0);
 	layout()->setSpacing(0);
 
@@ -486,71 +499,70 @@ bool PartsEditorMainWindow::save() {
 }
 
 bool PartsEditorMainWindow::saveAs() {
-	if(validateMinRequirements()) {
-		QString fileNameBU = m_fileName;
-		QString moduleIdBU = m_moduleId;
+	if(!validateMinRequirements()) return false;
 
-		m_moduleId = ___emptyString___;
-		QString title = m_title->text();
-		m_fileName = title != ___emptyString___ ? title+FritzingPartExtension : m_fileName;
+	QString fileNameBU = m_fwFilename;
+	QString moduleIdBU = m_moduleId;
 
-		// TODO Mariano: This folder should be defined in the preferences... some day
-		QString partsFolderPath = FolderUtils::getUserDataStorePath("parts")+"/";
-		QString userPartsFolderPath = partsFolderPath+"user/";
+	m_moduleId = ___emptyString___;
+	QString title = m_title->text();
+	m_fwFilename = title != ___emptyString___ ? title+FritzingPartExtension : QFileInfo(m_fwFilename).baseName();
 
-		bool firstTime = true; // Perhaps the user wants to use the default file name, confirm first
-		while(m_fileName.isEmpty()
-			  || QFileInfo(userPartsFolderPath+m_fileName).exists()
-			  || (FolderUtils::isEmptyFileName(m_fileName,untitledFileName()) && firstTime) ) 
-		{
-			bool ok;
-			m_fileName = QInputDialog::getText(
-				this,
-				tr("Save as new part"),
-				tr("There's already a file with this name.\nPlease, specify a new filename"),
-				QLineEdit::Normal,
-				m_fileName,
-				&ok
-			);
-			firstTime = false;
-			if (!ok) {
-				m_moduleId = moduleIdBU;
-				m_fileName = fileNameBU;
-				return false;
-			}
+	// TODO Mariano: This folder should be defined in the preferences... some day
+	QString partsFolderPath = FolderUtils::getUserDataStorePath("parts")+"/";
+	QString userPartsFolderPath = partsFolderPath+"user/";
+
+	bool firstTime = true; // Perhaps the user wants to use the default file name, confirm first
+	while(m_fwFilename.isEmpty()
+			|| QFileInfo(userPartsFolderPath+m_fwFilename).exists()
+			|| (FolderUtils::isEmptyFileName(m_fwFilename,untitledFileName()) && firstTime) ) 
+	{
+		bool ok;
+		m_fwFilename = QInputDialog::getText(
+			this,
+			tr("Save as new part"),
+			tr("There's already a file with this name.\nPlease, specify a new filename"),
+			QLineEdit::Normal,
+			m_fwFilename,
+			&ok
+		);
+		firstTime = false;
+		if (!ok) {
+			m_moduleId = moduleIdBU;
+			m_fwFilename = fileNameBU;
+			return false;
 		}
+	}
 
-		m_fileName.replace(QRegExp("[^A-Za-z0-9\\.]"), "_");
+	m_fwFilename.replace(QRegExp("[^A-Za-z0-9\\.]"), "_");
 
-		Qt::CaseSensitivity cs = Qt::CaseSensitive;
-	#ifdef Q_WS_WIN
-		// seems to be necessary for Windows: getUserDataStorePath() returns a string starting with "c:"
-		// but the file dialog returns a string beginning with "C:"
-		cs = Qt::CaseInsensitive;
-	#endif
+	Qt::CaseSensitivity cs = Qt::CaseSensitive;
+#ifdef Q_WS_WIN
+	// seems to be necessary for Windows: getUserDataStorePath() returns a string starting with "c:"
+	// but the file dialog returns a string beginning with "C:"
+	cs = Qt::CaseInsensitive;
+#endif
 
-		QString filename = !m_fileName.startsWith(userPartsFolderPath, cs)
-						&& !m_fileName.startsWith(partsFolderPath, cs)
-				? userPartsFolderPath+m_fileName
-				: m_fileName;
-		QString guid = "__"+FolderUtils::getRandText()+FritzingPartExtension;
-		if(!alreadyHasExtension(filename, FritzingPartExtension)) {
-			filename += guid;
-		} else {
-			filename.replace(FritzingPartExtension,guid);
-		}
+	QString filename = !m_fwFilename.startsWith(userPartsFolderPath, cs)
+					&& !m_fwFilename.startsWith(partsFolderPath, cs)
+			? userPartsFolderPath+m_fwFilename
+			: m_fwFilename;
+	QString guid = "__"+FolderUtils::getRandText()+FritzingPartExtension;
+	if(!alreadyHasExtension(filename, FritzingPartExtension)) {
+		filename += guid;
+	} else {
+		filename.replace(FritzingPartExtension,guid);
+	}
 
-		makeNonCore();
-		if (!saveAsAux(filename)) return false;
-
-		if(wannaSaveAfterWarning(true)) {
-			m_savedAsNewPart = true;
-			m_updateEnabled = true;
-			updateButtons();
-			if(m_closeAfterSaving) close();
-			return true;
-		}
+	makeNonCore();
+	if (!saveAsAux(filename)) return false;
 
+	if(wannaSaveAfterWarning(true)) {
+		m_savedAsNewPart = true;
+		m_updateEnabled = true;
+		updateButtons();
+		if(m_closeAfterSaving) close();
+		return true;
 	}
 
 	return false;
@@ -591,7 +603,7 @@ bool PartsEditorMainWindow::saveAsAux(const QString & fileName) {
 
     m_partUpdated = true;
 
-    m_fileName = fileName;
+    setFileName(fileName);
     //setCurrentFile(fileName);
 
    // mark the stack clean so we update the window dirty flag
@@ -651,7 +663,7 @@ ModelPartShared* PartsEditorMainWindow::createModelPartShared() {
 }
 
 void PartsEditorMainWindow::cleanUp() {
-	bool decUntitled = m_fileName.startsWith(untitledFileName());
+	bool decUntitled = m_fwFilename.startsWith(untitledFileName());
 	if(decUntitled) {
 		if(m_lastOpened == this) {
 			UntitledPartIndex -= m_closedBeforeCount;
@@ -678,7 +690,7 @@ void PartsEditorMainWindow::closeEvent(QCloseEvent *event) {
 		cleanUp();
 		QMainWindow::closeEvent(event);
 		if(m_partUpdated) {
-			emit partUpdated(m_fileName, m_id, !m_savedAsNewPart &&
+			emit partUpdated(m_fwFilename, m_id, !m_savedAsNewPart &&
 				(m_connsInfo->connectorsCountChanged() || m_views->connectorsPosOrSizeChanged())
 			);
 		}
