@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5308 $:
+$Revision: 5762 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-07-30 21:09:56 +0200 (Sat, 30 Jul 2011) $
+$Date: 2012-01-19 23:04:44 +0100 (Thu, 19 Jan 2012) $
 
 ********************************************************************/
 
@@ -50,6 +50,7 @@ $Date: 2011-07-30 21:09:56 +0200 (Sat, 30 Jul 2011) $
 #include "../utils/folderutils.h"
 #include "../utils/textutils.h"
 #include "../utils/graphicsutils.h"
+#include "../utils/ratsnestcolors.h"
 #include "../svg/gedaelement2svg.h"
 #include "../svg/kicadmodule2svg.h"
 #include "../svg/kicadschematic2svg.h"
@@ -248,8 +249,8 @@ void PartsEditorView::fitCenterAndDeselect() {
 
 void PartsEditorView::setDefaultBackground() {
 	QString bgColor = " PartsEditorView {background-color: rgb(%1,%2,%3);} ";
-	if(m_bgcolors.contains(m_viewIdentifier)) {
-		QColor c = m_bgcolors[m_viewIdentifier];
+	QColor c = standardBackground();
+	if (c.isValid()) {
 		setStyleSheet(styleSheet()+bgColor.arg(c.red()).arg(c.green()).arg(c.blue()));
 	}
 }
