@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5375 $:
+$Revision: 5759 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-08-08 21:38:25 +0200 (Mon, 08 Aug 2011) $
+$Date: 2012-01-16 07:45:36 +0100 (Mon, 16 Jan 2012) $
 
 ********************************************************************/
 
@@ -526,6 +526,11 @@ void ProgramTab::setClean() {
 	textChanged();
 }
 
+void ProgramTab::setDirty() {
+	m_textEdit->document()->setModified(true);
+	textChanged();
+}
+
 void ProgramTab::save() {
 	emit wantToSave(m_tabWidget->currentIndex());
 }
@@ -808,3 +813,8 @@ void ProgramTab::enableProgramButton() {
 
 	m_programButton->setEnabled(enabled);
 }
+
+void ProgramTab::appendToConsole(const QString & text)
+{
+	m_console->append(text);
+}
