@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5375 $:
+$Revision: 5787 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-08-08 21:38:25 +0200 (Mon, 08 Aug 2011) $
+$Date: 2012-01-27 01:48:06 +0100 (Fri, 27 Jan 2012) $
 
 ********************************************************************/
 
@@ -370,8 +370,16 @@ void ProgramWindow::setup(const QList<LinkedFile *> & linkedFiles, const QString
                 programTab = addTab();
             }
             QDir dir(alternativePath);
-            QFileInfo fileInfo(linkedFile->filename);
-            programTab->loadProgramFile(linkedFile->filename, dir.absoluteFilePath(fileInfo.fileName()), false);
+            QFileInfo fileInfo(linkedFile->linkedFilename);
+            programTab->loadProgramFile(linkedFile->linkedFilename, dir.absoluteFilePath(fileInfo.fileName()), false);
+			if ((linkedFile->fileFlags & LinkedFile::InBundleFlag) && ((linkedFile->fileFlags & LinkedFile::ReadOnlyFlag) == 0)) { 
+				if (linkedFile->fileFlags & LinkedFile::SameMachineFlag) {
+					programTab->appendToConsole(tr("File '%1' was restored from the .fzz file; the local copy was not found.").arg(fileInfo.fileName()));
+				}
+				else {
+					programTab->appendToConsole(tr("File '%1' was restored from the .fzz file; save a local copy to work with an external editor.").arg(fileInfo.fileName()));
+				}
+			}
 			if (!m_languages.value(linkedFile->language, "").isEmpty()) {
 				programTab->setLanguage(linkedFile->language, false);
 			}
