@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5204 $:
+$Revision: 5451 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-07-08 13:16:29 +0200 (Fri, 08 Jul 2011) $
+$Date: 2011-09-02 14:14:56 +0200 (Fri, 02 Sep 2011) $
 
 ********************************************************************/
 
@@ -202,7 +202,7 @@ QString SqliteReferenceModel::retrieveModuleId(const Part *examplePart, const QS
 			if(query.next()) {
 				moduleId =  query.value(0).toString(); //grab the first
 			}
-			DebugDialog::debug("SQLITE: found: "+moduleId);
+			//DebugDialog::debug("SQLITE: found: "+moduleId);
 			query.clear();
 		} else {
 			moduleId = ___emptyString___;
@@ -248,7 +248,7 @@ QStringList SqliteReferenceModel::getPossibleMatches(const Part *examplePart, co
 			"SELECT * FROM properties prop \n"
 			"WHERE prop.part_id = part.id %1 \n"
 		") ";
-	dbgQueryStr = dbgQueryStr.arg((propertyName.isEmpty()) ? "" : "AND prop.name = "+propertyName+"  AND prop.value = "+propertyValue); 
+    dbgQueryStr = dbgQueryStr.arg((propertyName.isEmpty()) ? QString("") : QString("AND prop.name = ")+propertyName+"  AND prop.value = "+propertyValue);
 	QSqlQuery query;
 	query.prepare(queryStr);
 
@@ -476,11 +476,11 @@ qlonglong SqliteReferenceModel::partId(QString moduleID) {
 	query.bindValue(":moduleID",moduleID);
 
 	if(query.exec()) {
-		DebugDialog::debug("SQLITE: retrieving id: "+moduleID);
+		//DebugDialog::debug("SQLITE: retrieving id: "+moduleID);
 		if(query.next()) {
 			partId =  query.value(0).toULongLong(); //grab the first
 		}
-		DebugDialog::debug(QString("SQLITE: found: %1").arg(partId));
+		//DebugDialog::debug(QString("SQLITE: found: %1").arg(partId));
 		query.clear();
 	} else {
 		DebugDialog::debug(
