@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5409 $:
+$Revision: 5961 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-08-17 11:25:51 +0200 (Wed, 17 Aug 2011) $
+$Date: 2012-04-09 02:13:59 +0200 (Mon, 09 Apr 2012) $
 
 ********************************************************************/
 
@@ -46,11 +46,6 @@ $Date: 2011-08-17 11:25:51 +0200 (Wed, 17 Aug 2011) $
 #include "../utils/misc.h"
 #include "../commands.h"
 
-QT_BEGIN_NAMESPACE
-class QDragEnterEvent;
-class QDropEvent;
-QT_END_NAMESPACE
-
 struct ItemCount {
 	int selCount;
 	int hasLabelCount;
@@ -122,13 +117,13 @@ public:
 	void addBendpoint(ItemBase * lastHoverEnterItem, ConnectorItem * lastHoverEnterConnectorItem, QPointF lastLocation);
 	void flattenCurve(ItemBase * lastHoverEnterItem, ConnectorItem * lastHoverEnterConnectorItem, QPointF lastLocation);
 
-	virtual void deleteSelected(Wire *);
+	void deleteSelected(Wire *);
 	PaletteItem *getSelectedPart();
 
     void addViewLayer(ViewLayer *);
     void setAllLayersVisible(bool visible);
-    void setLayerVisible(ViewLayer * viewLayer, bool visible);
-	void setLayerVisible(ViewLayer::ViewLayerID viewLayerID, bool visible);
+    void setLayerVisible(ViewLayer * viewLayer, bool visible, bool doChildLayers);
+	void setLayerVisible(ViewLayer::ViewLayerID viewLayerID, bool visible, bool doChildLayers);
     void setLayerActive(ViewLayer * viewLayer, bool active);
 	void setLayerActive(ViewLayer::ViewLayerID viewLayerID, bool active);
     bool layerIsVisible(ViewLayer::ViewLayerID);
@@ -138,7 +133,7 @@ public:
  	void mousePressConnectorEvent(ConnectorItem *, QGraphicsSceneMouseEvent *);
  	void setBackground(QColor);
  	const QColor& background();
-	const QColor& standardBackground();
+	QColor standardBackground();
  	void setItemMenu(QMenu*);
  	void setWireMenu(QMenu*);
 	virtual void changeConnection(long fromID,
@@ -183,7 +178,8 @@ public:
 	void hideConnectors(bool hide);
 	void saveLayerVisibility();
 	void restoreLayerVisibility();
-	virtual void updateRoutingStatus(CleanUpWiresCommand*, RoutingStatus &, bool manual);
+	void updateRoutingStatus(CleanUpWiresCommand*, RoutingStatus &, bool manual);
+	void updateRoutingStatus(RoutingStatus &, bool manual);
 	virtual bool hasAnyNets();
 	void ensureLayerVisible(ViewLayer::ViewLayerID);
 
@@ -208,16 +204,18 @@ public:
 	void noteSizeChanged(ItemBase * itemBase, const QSizeF & oldSize, const QSizeF & newSize);
 	void resizeNote(long itemID, const QSizeF & );
 	class SelectItemCommand* stackSelectionState(bool pushIt, QUndoCommand * parentCommand);
-	QString renderToSVG(double printerScale, const LayerList & partLayers, const LayerList & wireLayers, 
-						bool blackOnly, QSizeF & imageSize, ItemBase * offsetPart, double dpi, 
+	QString renderToSVG(double printerScale, const LayerList &, 
+						bool blackOnly, QSizeF & imageSize, ItemBase * board, double dpi, 
 						bool selectedItems, bool flatten, bool fillHoles, bool & empty);
-	QString renderToSVG(double printerScale, const LayerList & partLayers, const LayerList & wireLayers, 
-								  bool blackOnly, QSizeF & imageSize, ItemBase * offsetPart, double dpi, 
+	QString renderToSVG(double printerScale, const LayerList &, 
+						bool blackOnly, QSizeF & imageSize, QRectF & offsetRect, double dpi, 
+						bool selectedItems, bool flatten, bool fillHoles, bool & empty);
+	QString renderToSVG(double printerScale, bool blackOnly, QSizeF & imageSize, QRectF & offsetRect, double dpi, 
 								  bool flatten, bool fillHoles, 
-								  QList<ItemBase *> & itemBases, QRectF itemsBoundingRect, bool & empty);
+								  QList<QGraphicsItem *> & itemsAndLabels, QRectF itemsBoundingRect, bool & empty);
 
 	bool spaceBarIsPressed();
-	virtual long setUpSwap(ItemBase *, long newModelIndex, const QString & newModuleID, ViewLayer::ViewLayerSpec, bool doEmit, bool noFinalChangeWiresCommand, QUndoCommand * parentCommand);
+	virtual long setUpSwap(ItemBase *, long newModelIndex, const QString & newModuleID, ViewLayer::ViewLayerSpec, bool doEmit, bool noFinalChangeWiresCommand, QList<Wire *> & wiresToDelete, QUndoCommand * parentCommand);
 	ConnectorItem * lastHoverEnterConnectorItem();
 	ItemBase * lastHoverEnterItem();
 	LayerHash & viewLayers();
@@ -237,6 +235,7 @@ public:
 	long createWire(ConnectorItem * from, ConnectorItem * to, ViewGeometry::WireFlags, bool dontUpdate, BaseCommand::CrossViewType, QUndoCommand * parentCommand);
 	int selectAllObsolete();
 	int selectAllMoveLock();
+	void setMoveLock(long id, bool lock);
 	int selectAllItemType(ModelPart::ItemType);
 	bool partLabelsVisible();
 	void restorePartLabel(long itemID, QDomElement & element);
@@ -247,6 +246,8 @@ public:
 
 	void alignToGrid(bool);
 	bool alignedToGrid();
+	void showGrid(bool);
+	bool showingGrid();
 	void saveZoom(double);
 	double retrieveZoom();
 	void initGrid();
@@ -259,12 +260,13 @@ public:
 	virtual void changeLayer(long id, double z, ViewLayer::ViewLayerID viewLayerID);
 	void ratsnestConnect(ConnectorItem * connectorItem, bool connect);
 	void ratsnestConnect(ItemBase *, bool connect);
+	void ratsnestConnect(ConnectorItem * c1, ConnectorItem * c2, bool connect);
 	virtual void addDefaultParts();
 	float getTopZ();
 	QGraphicsItem * addWatermark(const QString & filename);
 	void copyHeart(QList<ItemBase *> & bases, bool saveBoundingRects, QByteArray & itemData, QList<long> & modelIndexes);
 	void pasteHeart(QByteArray & itemData, bool seekOutsideConnections);
-	virtual ViewGeometry::WireFlag getTraceFlag();
+	ViewGeometry::WireFlag getTraceFlag();
 	void changeBus(ItemBase *, bool connec, const QString & oldBus, const QString & newBus, QList<ConnectorItem *> &, const QString & message);
 	const QString & filenameIf();
 	void setItemDropOffset(long id, QPointF offset);
@@ -280,6 +282,18 @@ public:
 	bool curvyWires();
 	void setCurvyWires(bool);
 	bool curvyWiresIndicated(Qt::KeyboardModifiers);
+	void triggerRotate(ItemBase *, double degrees);
+	void makeWiresChangeConnectionCommands(const QList<Wire *> & wires, QUndoCommand * parentCommand);
+	void renamePins(ItemBase *, const QStringList & oldLabels, const QStringList & newLabels, bool singleRow);
+	void renamePins(long itemID, const QStringList & labels, bool singleRow);
+	void getRatsnestColor(QColor &);
+	VirtualWire * makeOneRatsnestWire(ConnectorItem * source, ConnectorItem * dest, bool routed, QColor color);
+	virtual double getRatsnestOpacity();
+	virtual double getRatsnestWidth();
+	void setAnyInRotation();
+	ConnectorItem * findConnectorItem(ConnectorItem * foreignConnectorItem);
+	void setGroundFillSeed(long id, const QString & connectorID, bool seed);
+	void setWireExtras(long id, const QDomElement &);
 
 protected:
     void dragEnterEvent(QDragEnterEvent *event);
@@ -333,7 +347,8 @@ protected:
 	void keyPressEvent(QKeyEvent *);
 	void keyReleaseEvent(QKeyEvent *);
 	void clearTemporaries();
-	virtual void dragWireChanged(class Wire* wire, ConnectorItem * from, ConnectorItem * to);
+	void dragWireChanged(class Wire* wire, ConnectorItem * from, ConnectorItem * to);
+	void dragRatsnestChanged();
 	void killDroppingItem();
 	ViewLayer::ViewLayerID getViewLayerID(ModelPart *, ViewIdentifierClass::ViewIdentifier, ViewLayer::ViewLayerSpec);
 	ItemBase * overSticky(ItemBase *);
@@ -346,7 +361,6 @@ protected:
 	void resizeEvent(QResizeEvent *);
 
 	void addViewLayersAux(const LayerList &layers, float startZ = 1.5);
-	void tempConnectWire(Wire * wire, ConnectorItem * from, ConnectorItem * to);
 	virtual bool disconnectFromFemale(ItemBase * item, QHash<long, ItemBase *> & savedItems, ConnectorPairHash &, bool doCommand, bool rubberBandLegEnabled, QUndoCommand * parentCommand);
 	void clearDragWireTempCommand();
 	bool draggingWireEnd();
@@ -357,31 +371,30 @@ protected:
 	virtual bool canChainWire(Wire *);
 	virtual bool canDragWire(Wire *);
 	virtual bool canCreateWire(Wire * dragWire, ConnectorItem * from, ConnectorItem * to);
-	virtual bool modifyNewWireConnections(Wire * dragWire, ConnectorItem * fromOnWire, ConnectorItem * from, ConnectorItem * to, QUndoCommand * parentCommand);
+	//virtual bool modifyNewWireConnections(Wire * dragWire, ConnectorItem * fromOnWire, ConnectorItem * from, ConnectorItem * to, QUndoCommand * parentCommand);
+	virtual void setUpColor(ConnectorItem * fromConnectorItem, ConnectorItem * toConnectorItem, Wire * wire, QUndoCommand * parentCommand);
 	void setupAutoscroll(bool moving);
 	void turnOffAutoscroll();
 	bool checkAutoscroll(QPoint globalPos);
 	virtual void setWireVisible(Wire *);
-	virtual void chainVisible(ConnectorItem * fromConnectorItem, ConnectorItem * toConnectorItem, bool connect);
 	bool matchesLayer(ModelPart * modelPart);
 
 	QByteArray removeOutsideConnections(const QByteArray & itemData, QList<long> & modelIndexes);
 	void addWireExtras(long newID, QDomElement & view, QUndoCommand * parentCommand);
-	virtual bool doRatsnestOnCopy();
 	virtual const QString & hoverEnterWireConnectorMessage(QGraphicsSceneHoverEvent * event, ConnectorItem * item);
 	virtual const QString & hoverEnterPartConnectorMessage(QGraphicsSceneHoverEvent * event, ConnectorItem * item);
 	void partLabelChangedAux(ItemBase * pitem,const QString & oldText, const QString &newText);
 	void drawBackground( QPainter * painter, const QRectF & rect );
 	void handleConnect(QDomElement & connect, ModelPart *, const QString & fromConnectorID, ViewLayer::ViewLayerID, QStringList & alreadyConnected, 
 						QHash<long, ItemBase *> & newItems, QUndoCommand * parentCommand, bool seekOutsideConnections);
-	void setUpSwapReconnect(ItemBase* itemBase, long newID, const QString & newModuleID, bool master, QUndoCommand * parentCommand);
+	void setUpSwapReconnect(ItemBase* itemBase, long newID, const QString & newModuleID, bool master, QList<Wire *> & wiresToDelete, QUndoCommand * parentCommand);
 	bool swappedGender(ConnectorItem * originalConnectorItem, Connector * newConnector);
 	void setLastPaletteItemSelected(PaletteItem * paletteItem);
 	void setLastPaletteItemSelectedIf(ItemBase * itemBase);
 	void prepDragBendpoint(Wire *, QPoint eventPos, bool dragCurve);
 	void prepDragWire(Wire *);
 	void clickBackground(QMouseEvent *);
-	void categorizeDragWires(QSet<Wire *> & wires);
+	void categorizeDragWires(QSet<Wire *> & wires, QList<ItemBase *> & freeWires);
 	void categorizeDragLegs(bool rubberBandLegEnabled);
 	void prepMove(ItemBase * originatingItem, bool rubberBandLegEnabled);
 	void initBackgroundColor();
@@ -412,8 +425,6 @@ protected:
 	virtual bool resizingJumperItemRelease();
 	virtual bool resizingBoardPress(QGraphicsItem * item);
 	virtual bool resizingBoardRelease();
-	bool connectedDirectlyTo(ConnectorItem * from, ConnectorItem * to, QList<ConnectorItem *> & byBus);
-	bool connectedDirectlyTo(ConnectorItem * from, ConnectorItem * to, QList<ConnectorItem *> & byBus, QList<ConnectorItem *> & visited);
 	virtual QPoint calcFixedToCenterItemOffset(const QRect & viewPortRect, const QSizeF & helpSize);
 	virtual bool acceptsTrace(const ViewGeometry &);
 	virtual ItemBase * placePartDroppedInOtherView(ModelPart *, ViewLayer::ViewLayerSpec, const ViewGeometry & viewGeometry, long id, SketchWidget * dropOrigin);
@@ -436,7 +447,13 @@ protected:
 	void moveLegBendpoints(bool undoOnly, QUndoCommand * parentCommand);
 	void moveLegBendpointsAux(ConnectorItem * connectorItem, bool undoOnly, QUndoCommand * parentCommand);
 	virtual void rotatePartLabels(double degrees, QTransform &, QPointF center, QUndoCommand * parentCommand);
-
+	bool checkUpdateRatsnest(QList<ConnectorItem *> & connectorItems);
+	void makeRatsnestViewGeometry(ViewGeometry & viewGeometry, ConnectorItem * source, ConnectorItem * dest); 
+	virtual double getTraceWidth();
+	virtual const QString & traceColor(ViewLayer::ViewLayerSpec);
+	void createTrace(Wire * fromWire, const QString & commandString, ViewGeometry::WireFlag);
+	bool createOneTrace(Wire * wire, ViewGeometry::WireFlag flag, bool allowAny, QList<Wire *> & done, QUndoCommand * parentCommand);
+	void removeWire(Wire * w, QList<ConnectorItem *> & ends, QList<Wire *> & done, QUndoCommand * parentCommand);
 
 protected:
 	static bool lessThan(int a, int b);
@@ -458,7 +475,8 @@ signals:
 	void selectionChangedSignal();
 
 	void resizeSignal();
-	void dropSignal(const QPoint &pos);
+	void dropSignal(const QPoint &pos, ModelPart *);
+	void dropTempSignal(ModelPart *, QWidget *);
 	void routingStatusSignal(SketchWidget *, const RoutingStatus &);
 	void movingSignal(SketchWidget *, QUndoCommand * parentCommand);
 	void selectAllItemsSignal(bool state, bool doEmit);
@@ -472,7 +490,6 @@ signals:
 	void dropPasteSignal(SketchWidget *);
 	void changeBoardLayersSignal(int, bool doEmit);
 	void firstTimeHelpHidden();
-	void disconnectWireSignal(QSet<ItemBase *> &, QList<long> &, QUndoCommand * parentCommand);
 	void deleteTracesSignal(QSet<ItemBase *> & deletedItems, QHash<ItemBase *, SketchWidget *> & otherDeletedItems, QList<long> & deletedIDs, bool isForeign, QUndoCommand * parentCommand);
 	void makeDeleteItemCommandPrepSignal(ItemBase * itemBase, bool foreign, QUndoCommand * parentCommand);
 	void makeDeleteItemCommandFinalSignal(ItemBase * itemBase, bool foreign, QUndoCommand * parentCommand);
@@ -481,6 +498,8 @@ signals:
 	void ratsnestConnectSignal(long id, const QString & connectorID, bool connect, bool doEmit);
 	void updatePartLabelInstanceTitleSignal(long itemID);
 	void filenameIfSignal(QString & filename);
+	void collectRatsnestSignal(QList<SketchWidget *> & foreignSketchWidgets);
+	void removeRatsnestSignal(QList<struct ConnectorEdge *> & cutSet, QUndoCommand * parentCommand); 
 
 protected slots:
 	void itemAddedSlot(ModelPart *, ViewLayer::ViewLayerSpec, const ViewGeometry &, long id, SketchWidget * dropOrigin);
@@ -509,13 +528,17 @@ protected slots:
 	void rememberSticky(long id, QUndoCommand * parentCommand);
 	void rememberSticky(ItemBase *, QUndoCommand * parentCommand);
 	void copyBoundingRectsSlot(QHash<QString, QRectF> &);
-	void disconnectWireSlot(QSet<ItemBase *> &, QList<long> & deletedIDs, QUndoCommand * parentCommand);
+	void deleteRatsnest(Wire *, QUndoCommand * parentCommand);
 	void deleteTracesSlot(QSet<ItemBase *> & deletedItems, QHash<ItemBase *, SketchWidget *> & otherDeletedItems, QList<long> & deletedIDs, bool isForeign, QUndoCommand * parentCommand);
 	void vScrollToZero();
 	void arrowTimerTimeout();
 	void makeDeleteItemCommandPrepSlot(ItemBase * itemBase, bool foreign, QUndoCommand * parentCommand);
 	void makeDeleteItemCommandFinalSlot(ItemBase * itemBase, bool foreign, QUndoCommand * parentCommand);
 	void updatePartLabelInstanceTitleSlot(long itemID);
+	void changePinLabelsSlot(ItemBase * itemBase, bool singleRow);
+	void changePinLabels(ItemBase *, bool singleRow);
+	void collectRatsnestSlot(QList<SketchWidget *> & foreignSketchWidgets);
+	void removeRatsnestSlot(QList<struct ConnectorEdge *> & cutSet, QUndoCommand * parentCommand);
 
 public slots:
 	void changeWireColor(const QString newColor);
@@ -533,6 +556,7 @@ public slots:
 	void setResistance(QString resistance, QString pinSpacing);
 	void setProp(long itemID, const QString & prop, const QString & value, bool redraw, bool doEmit);
 	void setProp(ItemBase *, const QString & propName, const QString & translatedPropName, const QString & oldValue, const QString & newValue, bool redraw);
+	void setHoleSize(ItemBase *, const QString & propName, const QString & translatedPropName, const QString & oldValue, const QString & newValue, QRectF & oldRect, QRectF & newRect, bool redraw);
 	void setSpacing(const QString & spacing);
 	void setForm(const QString & form);
 	virtual void showLabelFirstTime(long itemID, bool show, bool doEmit);
@@ -614,7 +638,6 @@ protected:
 	QPointer<Wire> m_dragBendpointWire;
 	bool m_dragCurve;
 	QPoint m_dragBendpointPos;
-	QColor m_standardBackgroundColor;
 	StatusConnectStatus m_statusConnectState;
 	QList<QGraphicsItem *> m_inFocus;
 	QString m_viewName;
@@ -623,6 +646,7 @@ protected:
 	int m_arrowTotalY;
 	bool m_movingByMouse;
 	bool m_alignToGrid;
+	bool m_showGrid;
 	double m_gridSizeInches;
 	QPointer<ItemBase> m_alignmentItem;
 	QPointF m_alignmentStartPoint;
@@ -646,12 +670,14 @@ protected:
 	QMultiHash<ItemBase *, ConnectorItem *> m_stretchingLegs;
 	bool m_curvyWires;
 	bool m_rubberBandLegWasEnabled;
+	RoutingStatus m_routingStatus;
+	bool m_anyInRotation;
 
 public:
 	static ViewLayer::ViewLayerID defaultConnectorLayer(ViewIdentifierClass::ViewIdentifier viewId);
+	static const int PropChangeDelay;
 
 protected:
-	static QHash<ViewIdentifierClass::ViewIdentifier,QColor> m_bgcolors;
 	static const int MoveAutoScrollThreshold;
 	static const int DragAutoScrollThreshold;
 };
