@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5396 $:
+$Revision: 5614 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-08-15 01:36:25 +0200 (Mon, 15 Aug 2011) $
+$Date: 2011-11-12 13:32:21 +0100 (Sat, 12 Nov 2011) $
 
 ********************************************************************/
 
@@ -45,7 +45,9 @@ bool fillNotStroke(QDomElement & element, SVG2gerber::ForWhy forWhy) {
 	if (stroke.compare("none") == 0) return true;
 
 	// both fill and stroke
-	return false;
+
+	// note: this originally returned 'false' which suggests there may be some parts that will surprise us next time they are exported to gerber
+	return true;
 }
 
 //TODO: currently only supports one board per sketch (i.e. multiple board outlines will mess you up)
@@ -63,7 +65,7 @@ int SVG2gerber::convert(const QString & svgStr, bool doubleSided, const QString
     int errorColumn;
     bool result = m_SVGDom.setContent(svgStr, &errorStr, &errorLine, &errorColumn);
     if (!result) {
-        DebugDialog::debug(QString("gerber svg failed %1 %2 %3 %4").arg(svgStr).arg(errorStr).arg(errorLine).arg(errorColumn));
+        DebugDialog::debug(QString("gerber svg failed %2 %3 %4 %1").arg(svgStr).arg(errorStr).arg(errorLine).arg(errorColumn));
     }
 
 #ifndef QT_NO_DEBUG
@@ -221,6 +223,15 @@ void SVG2gerber::convertShapes2paths(QDomNode node){
         else if((tag=="path") || (tag=="svg:path")){
             path = element;
         }
+		else if (tag == "g") {
+			// no op
+		}
+		else if (tag == "#comment") {
+			// no op
+		}
+		else if (tag == "#text") {
+			// no op
+		}
         else {
             DebugDialog::debug("svg2gerber ignoring SVG element: " + tag);
         }
@@ -286,7 +297,7 @@ int SVG2gerber::allPaths2gerber(ForWhy forWhy) {
     //DebugDialog::debug("polygons to gerber: " + QString::number(polyList.length()));
 
     QDomNodeList polyLineList = m_SVGDom.elementsByTagName("polyline");
-    DebugDialog::debug("polylines to gerber: " + QString::number(polyLineList.length()));
+    //DebugDialog::debug("polylines to gerber: " + QString::number(polyLineList.length()));
 
 	QDomNodeList lineList = m_SVGDom.elementsByTagName("line");
     //DebugDialog::debug("lines to gerber: " + QString::number(lineList.length()));
@@ -294,9 +305,6 @@ int SVG2gerber::allPaths2gerber(ForWhy forWhy) {
 	QDomNodeList pathList = m_SVGDom.elementsByTagName("path");
     //DebugDialog::debug("paths to gerber: " + QString::number(pathList.length()));
 
-	int totalCount = circleList.count() + rectList.count() + polyList.count() + lineList.count() + pathList.count();
-	int outlineCount = 0;
-
     // if this is the board outline, use it as the contour
     if (forWhy == ForOutline) {
         //DebugDialog::debug("drawing board outline");
@@ -313,19 +321,13 @@ int SVG2gerber::allPaths2gerber(ForWhy forWhy) {
 			continue;
 		}
 
-		if (forWhy == ForOutline && totalCount > 1) {
-			if (circle.attribute("id").compare("boardoutline", Qt::CaseInsensitive) != 0) continue;
-		}
-
-		outlineCount++;
-
         double centerx = circle.attribute("cx").toDouble();
         double centery = circle.attribute("cy").toDouble();
         double r = circle.attribute("r").toDouble();
 		if (r == 0) continue;
 
         double stroke_width = circle.attribute("stroke-width").toDouble();
-		double hole = ((2*r) - stroke_width)/1000;
+		double hole = ((2*r) - stroke_width) / 1000;
 
 		if (forWhy == ForDrill) {
 			QString drill_cx = QString::number(flipxNoRound(centerx) / 1000, 'f');				// drill file seems to be in inches
@@ -358,7 +360,7 @@ int SVG2gerber::allPaths2gerber(ForWhy forWhy) {
 			diam += 2 * MaskClearance;
 		}
 
-        if(false && fill=="none" && forWhy != ForMask){
+        if (forWhy != ForCopper && fill=="none" && forWhy != ForMask){
 			aperture = QString("C,%1X%2").arg(diam, 0, 'f').arg(hole);
         }
         else {
@@ -401,9 +403,6 @@ int SVG2gerber::allPaths2gerber(ForWhy forWhy) {
 		// rects
 		for(uint j = 0; j < rectList.length(); j++){
 			QDomElement rect = rectList.item(j).toElement();
-			if (forWhy == ForOutline && totalCount > 1) {
-				if (rect.attribute("id").compare("boardoutline", Qt::CaseInsensitive) != 0) continue;
-			}
 
 			QString aperture;
 
@@ -441,7 +440,7 @@ int SVG2gerber::allPaths2gerber(ForWhy forWhy) {
 			}
 
 
-			if(false && fill=="none" && forWhy != ForMask) {
+			if(forWhy != ForCopper && fill=="none" && forWhy != ForMask) {
 				aperture = QString("R,%1X%2X%3X%4").arg(totalx, 0, 'f').arg(totaly, 0, 'f').arg(holex, 0, 'f').arg(holey, 0, 'f');
 			}
 			else {
@@ -455,7 +454,11 @@ int SVG2gerber::allPaths2gerber(ForWhy forWhy) {
 				dcode_index++;
 			}
 
-			if (forWhy != ForOutline) {
+			bool doLines = false;
+			if (forWhy == ForOutline) doLines = true;
+			else if (forWhy == ForSilk && fill == "none") doLines = true;
+
+			if (!doLines) {
 				QString dcode = apertureMap[aperture];
 				if(current_dcode != dcode){
 					//switch to correct aperture
@@ -468,7 +471,6 @@ int SVG2gerber::allPaths2gerber(ForWhy forWhy) {
 			else {
 				// draw 4 lines
 
-				outlineCount++;
 				standardAperture(rect, apertureMap, current_dcode, dcode_index, 0);
 				m_gerber_paths += "X" + QString::number(flipx(x)) + "Y" + QString::number(flipy(y)) + "D02*\n";
 				m_gerber_paths += "X" + QString::number(flipx(x+width)) + "Y" + QString::number(flipy(y)) + "D01*\n";
@@ -500,8 +502,6 @@ int SVG2gerber::allPaths2gerber(ForWhy forWhy) {
 				}
 			}
 
-			outlineCount++;
-
 			//go to start - light off
 			m_gerber_paths += "X" + QString::number(flipx(x1)) + "Y" + QString::number(flipy(y1)) + "D02*\n";
 			//go to end point - light on
@@ -514,22 +514,17 @@ int SVG2gerber::allPaths2gerber(ForWhy forWhy) {
 		// polys - NOTE: assumes comma- or space- separated formatting
 		for(uint p = 0; p < polyList.length(); p++) {
                         QDomElement polygon = polyList.item(p).toElement();
-                        doPoly(polygon, forWhy, totalCount, true, apertureMap, current_dcode, dcode_index, outlineCount);
+                        doPoly(polygon, forWhy, true, apertureMap, current_dcode, dcode_index);
 		}
                 for(uint p = 0; p < polyLineList.length(); p++) {
-                        QDomElement polygon = polyList.item(p).toElement();
-                        doPoly(polygon, forWhy, totalCount, false, apertureMap, current_dcode, dcode_index, outlineCount);
+                        QDomElement polygon = polyLineList.item(p).toElement();
+                        doPoly(polygon, forWhy, false, apertureMap, current_dcode, dcode_index);
 		}
 	}
 
     // paths - NOTE: this assumes circular aperture
     for(uint n = 0; n < pathList.length(); n++){
         QDomElement path = pathList.item(n).toElement();
-		if (forWhy == ForOutline && totalCount > 1) {
-			if (path.attribute("id").compare("boardoutline", Qt::CaseInsensitive) != 0) continue;
-		}
-
-		outlineCount++;
 
 		if (forWhy == ForDrill) {
 			handleOblongPath(path, dcode_index);
@@ -605,23 +600,14 @@ int SVG2gerber::allPaths2gerber(ForWhy forWhy) {
     if (forWhy == ForOutline) {
         // add circular aperture with 0 width
         m_gerber_header += "%ADD10C,0.008*%\n";
-		if (outlineCount != 1) {
-			// if we can't find one and only one svg element as the board outline, then go to plan b
-			invalidPathsCount++;
-		}
     }
 
 	return invalidPathsCount;
 }
 
-void SVG2gerber::doPoly(QDomElement & polygon, ForWhy forWhy, int totalCount, bool closedCurve,
-					QHash<QString, QString> & apertureMap, QString & current_dcode, int & dcode_index, int & outlineCount) 
+void SVG2gerber::doPoly(QDomElement & polygon, ForWhy forWhy, bool closedCurve,
+					QHash<QString, QString> & apertureMap, QString & current_dcode, int & dcode_index) 
 {
-	if (forWhy == ForOutline && totalCount > 1) {
-		if (polygon.attribute("id").compare("boardoutline", Qt::CaseInsensitive) != 0) return;
-	}
-
-	outlineCount++;
 
 	//QString temp;
 	//QTextStream tempStream(&temp);
