@@ -18,9 +18,9 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 4542 $:
+$Revision: 5848 $:
 $Author: cohen@irascible.com $:
-$Date: 2010-10-15 07:15:26 +0200 (Fri, 15 Oct 2010) $
+$Date: 2012-02-07 11:30:53 +0100 (Tue, 07 Feb 2012) $
 
 ********************************************************************/
 
@@ -30,13 +30,25 @@ $Date: 2010-10-15 07:15:26 +0200 (Fri, 15 Oct 2010) $
 #include "../connectors/connectoritem.h"
 #include "../routingstatus.h"
 
+struct ConnectorEdge {
+	int head;
+	int tail;
+	int weight;
+	ConnectorItem * c0;
+	ConnectorItem * c1;
+	class Wire * wire;
+	bool visible;
+
+	void setHeadTail(int head, int tail);
+};
+
 class GraphUtils
 {
 
 public:
-	static bool chooseRatsnestGraph(const QList<ConnectorItem *> & equipotentials, ConnectorPairHash & result);
-	static bool scoreOneNet(QList<ConnectorItem *> & partConnectorItems, RoutingStatus & routingStatus);
-
+	static bool chooseRatsnestGraph(const QList<ConnectorItem *> * equipotentials, ViewGeometry::WireFlags, ConnectorPairHash & result);
+	static bool scoreOneNet(QList<ConnectorItem *> & partConnectorItems, ViewGeometry::WireFlags, RoutingStatus & routingStatus);
+	static void minCut(QList<ConnectorItem *> & connectorItems, QList<class SketchWidget *> & foreighSketchWidgets, ConnectorItem * source, ConnectorItem * sink, QList<ConnectorEdge *> & cutSet); 
 };
 
 #endif
