@@ -18,13 +18,15 @@ along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
 ********************************************************************
 
-$Revision: 5345 $:
+$Revision: 5917 $:
 $Author: cohen@irascible.com $:
-$Date: 2011-08-05 02:49:28 +0200 (Fri, 05 Aug 2011) $
+$Date: 2012-03-17 10:19:17 +0100 (Sat, 17 Mar 2012) $
 
 ********************************************************************/
 
+#include <QNetworkInterface>
 #include <QTextDocument>
+
 #include "textutils.h"
 #include "misc.h"
 //#include "../debugdialog.h"
@@ -171,25 +173,8 @@ QString TextUtils::replaceTextElement(const QString & svg, const QString & id, c
 
 		if (node.attribute("id").compare(id) != 0) continue;
 
-		node.normalize();
-
-		bool gotText = false;
-		QDomNodeList childList = node.childNodes();
-		for (int j = 0; j < childList.count(); j++) {
-			QDomNode child = childList.item(i);
-			if (child.isText()) {
-				gotText = true;
-				child.setNodeValue(newValue);
-				return doc.toString();
-			}
-		}
-
-		if (!gotText) {
-			QDomText t = doc.createTextNode(newValue);
-			node.appendChild(t);
-			return doc.toString();
-		}
-
+		replaceChildText(doc, node, newValue);
+		return doc.toString();
 	}
 		
 	return svg;
@@ -211,25 +196,8 @@ QString TextUtils::replaceTextElements(const QString & svg, const QHash<QString,
 		foreach (QString id, hash.keys()) {
 			if (node.attribute("id").compare(id) != 0) continue;
 
-			node.normalize();
-
-			bool gotText = false;
-			QDomNodeList childList = node.childNodes();
-			for (int j = 0; j < childList.count(); j++) {
-				QDomNode child = childList.item(i);
-				if (child.isText()) {
-					gotText = true;
-					changed = true;
-					child.setNodeValue(hash.value(id));
-					break;
-				}
-			}
-
-			if (!gotText) {
-				changed = true;
-				QDomText t = doc.createTextNode(hash.value(id));
-				node.appendChild(t);
-			}
+			replaceChildText(doc, node, hash.value(id));
+			changed = true;
 			break;
 		}
 	}
@@ -239,6 +207,32 @@ QString TextUtils::replaceTextElements(const QString & svg, const QHash<QString,
 	return doc.toString();
 }
 
+
+void TextUtils::replaceElementChildText(QDomDocument & doc, QDomElement & root, const QString & elementName, const QString & text) {
+	QDomElement element = root.firstChildElement(elementName);
+	if (element.isNull()) {
+		doc.createElement(elementName);
+		root.appendChild(element);
+	}
+	replaceChildText(doc, element, text);
+}
+
+void TextUtils::replaceChildText(QDomDocument & doc, QDomNode & node, const QString & text) {
+	node.normalize();
+
+	QDomNodeList childList = node.childNodes();
+	for (int j = 0; j < childList.count(); j++) {
+		QDomNode child = childList.item(j);
+		if (child.isText()) {
+			child.setNodeValue(text);
+			return;
+		}
+	}
+
+	QDomText t = doc.createTextNode(text);
+	node.appendChild(t);
+}
+
 bool TextUtils::mergeSvg(QDomDocument & doc1, const QString & svg, const QString & id) 
 {
 	QString errorStr;
@@ -561,7 +555,7 @@ bool TextUtils::getSvgSizes(QDomDocument & doc, double & sWidth, double & sHeigh
 	return true;
 }
 
-bool TextUtils::findText(QDomNode & node, QString & text) {
+bool TextUtils::findText(const QDomNode & node, QString & text) {
 	if (node.isText()) {
 		text = node.nodeValue();
 		return true;
@@ -591,6 +585,21 @@ QString TextUtils::escapeAnd(const QString & string) {
 	return s;
 }
 
+QString TextUtils::convertExtendedChars(const QString & str) 
+{
+	QString result;
+    foreach (QChar c, str) {
+		if (c < 128) {
+			result.append(c);
+		}
+		else {
+			result.append(QString("&#x%1;").arg(c.unicode(), 0, 16));
+		}
+	}
+
+	return result;
+}
+
 QString TextUtils::stripNonValidXMLCharacters(const QString & str) 
 {
 	QString result;
@@ -613,7 +622,12 @@ QString TextUtils::stripNonValidXMLCharacters(const QString & str)
 				result.append(hs);
 				in_hs = false;
 			}
-			result.append(c);
+			if (c > 255) {
+				result.append(QString("&#%1").arg(c, 0, 16));
+			}
+			else {
+				result.append(c);
+			}
 		}
 		else {
 			in_hs = false;
@@ -739,6 +753,19 @@ void TextUtils::collectLeaves(QDomElement & element, int & index, QVector<QDomEl
 	}
 }
 
+void TextUtils::collectLeaves(QDomElement & element, QList<QDomElement> & leaves) {
+	if (element.hasChildNodes()) {
+		QDomElement c = element.firstChildElement();
+		while (!c.isNull()) {
+			collectLeaves(c, leaves);
+			c = c.nextSiblingElement();
+		}
+	}
+	else {
+		leaves.append(element);
+	}
+}
+
 QMatrix TextUtils::elementToMatrix(QDomElement & element) {
 	QString transform = element.attribute("transform");
 	if (transform.isEmpty()) return QMatrix();
@@ -902,19 +929,21 @@ QDomElement TextUtils::copyText(QDomDocument & svgDom, QDomElement & parent, QDo
 	return ___emptyElement___;
 }
 
-void TextUtils::slamStrokeAndFill(QDomElement & element, const QString & stroke, const QString & fill)
+void TextUtils::slamStrokeAndFill(QDomElement & element, const QString & stroke, const QString & strokeWidth, const QString & fill)
 {
 	// assumes style elements have been normalized already
 	QString strokeAtt = element.attribute("stroke");
 	QString fillAtt = element.attribute("fill");
-	if (!strokeAtt.isEmpty() || !fillAtt.isEmpty()) {
+	QString strokeWidthAtt = element.attribute("stroke-width");
+	if (!strokeAtt.isEmpty() || !fillAtt.isEmpty() || !strokeWidthAtt.isEmpty()) {
 		element.setAttribute("stroke", stroke);
 		element.setAttribute("fill", fill);
+		element.setAttribute("stroke-width", strokeWidth);
 	}
 
 	QDomElement child = element.firstChildElement();
 	while (!child.isNull()) {
-		slamStrokeAndFill(child, stroke, fill);
+		slamStrokeAndFill(child, stroke, strokeWidth, fill);
 		child = child.nextSiblingElement();
 	}
 }
@@ -927,17 +956,17 @@ struct MatchThing
 	double val;
 };
 
-QString TextUtils::incrementTemplate(const QString & filename, int pins, double increment, MultiplyPinFunction multiFun, CopyPinFunction copyFun) 
+QString TextUtils::incrementTemplate(const QString & filename, int pins, double increment, MultiplyPinFunction multiFun, CopyPinFunction copyFun, void * userData) 
 {
 	QFile file(filename);
 	file.open(QFile::ReadOnly);
 	QString templateString = file.readAll();
 	file.close();
 
-	return incrementTemplateString(templateString, pins, increment, multiFun, copyFun);
+	return incrementTemplateString(templateString, pins, increment, multiFun, copyFun, userData);
 }
 
-QString TextUtils::incrementTemplateString(const QString & templateString, int pins, double increment, MultiplyPinFunction multiFun, CopyPinFunction copyFun)
+QString TextUtils::incrementTemplateString(const QString & templateString, int pins, double increment, MultiplyPinFunction multiFun, CopyPinFunction copyFun, void * userData)
 {
 	QString string;
 
@@ -960,13 +989,13 @@ QString TextUtils::incrementTemplateString(const QString & templateString, int p
 			MatchThing * mt = &matchThings[j];
 			argCopy.replace(mt->pos, mt->len, (*multiFun)(i, increment, mt->val));   
 		}
-		string += (*copyFun)(i, argCopy);
+		string += (*copyFun)(i, argCopy, userData);
 	}
 
 	return string;
 }
 
-QString TextUtils::standardCopyPinFunction(int pin, const QString & argString)
+QString TextUtils::standardCopyPinFunction(int pin, const QString & argString, void *)
 {
 	return argString.arg(pin);
 }
@@ -982,12 +1011,23 @@ QString TextUtils::incMultiplyPinFunction(int pin, double increment, double valu
 	return QString::number(value + ((pin + 1) * increment));
 }
 
+QString TextUtils::incCopyPinFunction(int pin, const QString & argString, void *) 
+{ 
+	return argString.arg(pin + 1); 
+}
 
-QString TextUtils::noCopyPinFunction(int, const QString & argString) 
+QString TextUtils::noCopyPinFunction(int, const QString & argString, void *) 
 { 
 	return argString; 
 }
 
+QString TextUtils::negIncCopyPinFunction(int pin, const QString & argString, void * userData) 
+{ 
+	int pins = *((int *) userData);
+	int offset = *(((int *) userData) + 1);
+	return argString.arg(pins - (pin + offset)); 
+}
+
 double TextUtils::getViewBoxCoord(const QString & svg, int coord)
 {
 	QRegExp re("viewBox=['\\\"]([^'\\\"]+)['\\\"]");
@@ -1090,3 +1130,80 @@ QString TextUtils::pointToSvgString(QPointF p, QPointF offset, double dpi, doubl
 	return point;
 }
 
+QString TextUtils::removeSVGHeader(QString & string) {
+	int ix = string.indexOf("<svg");
+	if (ix < 0) return string;
+
+	ix = string.indexOf(">", ix);
+	if (ix < 0) return string;
+
+	int jx = string.indexOf("</svg>");
+	if (jx < 0) return string;
+
+	string.remove(jx, 6);
+	string.remove(0, ix + 1);
+	return string;
+}
+
+
+QString TextUtils::getMacAddress()
+{
+	// http://stackoverflow.com/questions/7609953/obtaining-mac-address-on-windows-in-qt
+    foreach (QNetworkInterface interface, QNetworkInterface::allInterfaces())
+    {
+        // Return only the first non-loopback MAC Address
+        if (!(interface.flags() & QNetworkInterface::IsLoopBack))
+            return interface.hardwareAddress();
+    }
+    return QString();
+}
+
+
+QString TextUtils::expandAndFill(const QString & svg, const QString & color, double expandBy)
+{
+	QDomDocument domDocument;
+	QString errorStr;
+	int errorLine;
+	int errorColumn;
+	bool result = domDocument.setContent(svg, &errorStr, &errorLine, &errorColumn);
+	if (!result) {
+		return "";
+	}
+
+	QDomElement root = domDocument.documentElement();
+	expandAndFillAux(root, color, expandBy);
+
+	return domDocument.toString();
+}
+
+void TextUtils::expandAndFillAux(QDomElement & element, const QString & color, double expandBy)
+{
+	bool gotChildren = false;
+	QDomElement child = element.firstChildElement();
+	while (!child.isNull()) {
+		gotChildren = true;
+		expandAndFillAux(child, color, expandBy);
+		child = child.nextSiblingElement();
+	}	
+
+	if (gotChildren) return;
+
+	QString fill = element.attribute("fill");
+	QString stroke = element.attribute("stroke");
+	QString strokeWidth = element.attribute("stroke-width");
+	if (stroke.isEmpty() && fill.isEmpty()) {
+		return;
+	}
+		
+	element.setAttribute("fill", color);
+	element.setAttribute("stroke", color);
+
+	if (strokeWidth.isEmpty()) {
+		strokeWidth = "0";
+	}
+
+	double sw = strokeWidth.toDouble();
+	sw += expandBy;
+	element.setAttribute("stroke-width", QString::number(sw));
+
+}
